/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */

package Checkers;

import java.awt.*;
import ncsa.habanero.MirrorFrame;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;

/***
 ***  Board - holds a game board
 ***/

public class Board extends ncsa.habanero.Marshalling {

  //  Local Variables
  protected int squareSize;              //  The size of each square
  static  protected int nMargin = 5;     //  The margin around the edge
  
  /**
   *
   *@return a new board for a game
   *
   */

  public Board ()
    {
      //System.out.println ("Board::Board");      
      squareSize = 40;
    }
  
  /**  
   *
   *  draws a checker board, with no pieces on it
   *
   *@param G         the graphics object
   *@param bBlackUp  whether the lower left square should be black
   *@return          draws the checker board
   *
   */
  
  public void paint (Graphics G, boolean bBlackUp)
    {
      boolean bColor = false;
      for (int ndx1 = 0; ndx1 < 8; ndx1++) {
	for (int ndx2 = 0; ndx2 < 8; ndx2++) {
	  G.setColor ((bColor ? Color.black : Color.red));
	  G.fillRect (ndx1 * squareSize + nMargin, ndx2 * squareSize + nMargin,
		      squareSize, squareSize);
	  bColor = !bColor;
	}
	bColor = !bColor;
      }
    }
  
  
  /**
   *
   *  returns the piece that was hit
   *
   *@param nQX     the x coordinate, 0 - 7
   *@param nQY     the y coordinate, 0 - 7
   *@return     always returns null, since the board doesn't hold any pieces
   *
   */
  
  public Piece findHit (int nQX, int nQY)
    {
      //System.out.println ("Board::findHit");
      return null;
    }
  
  /**  
   *  
   *  drawPieceX
   *  
   *@param nX         the x coordinate of a piece, 0 - 7
   *@param bBlackUp   whether black should be up
   *@return           returns a number representing where the piece 
   *                     should be drawn in pixels
   *
   */
  
  public int drawPieceX (int nX, boolean bBlackUp)
    {
      int nXPos = (bBlackUp ? (7 - nX) : nX);
      return nXPos * squareSize + nMargin + 2;
    }
  
  
  /**  
   *  
   *  drawPieceY
   *  
   *@param   nY        the y coordinate of a piece, 0 - 7
   *@param   bBlackUp  whether black should be up
   *@return  returns a number representing where the piece should
   *            be drawn in pixels
   *
   */
  
  public int drawPieceY (int nY, boolean bBlackUp)
    {
      int nYPos = (bBlackUp ? (7 - nY) : nY);
      return nYPos * squareSize + nMargin + 2;
    }
  
  
  /**  
   *  
   *@param   nX        the x coordinate of an event
   *@param   bBlackUp  whether black should be up
   *@return  returns a number between 0 and 7
   *
   **/
  
  public int transformX (int nX, boolean bBlackUp)
    {
      int nTemp = (nX - nMargin) / squareSize;
      return (bBlackUp ? 7 - nTemp : nTemp);
    }
  
  
  /**  
   *  
   *@param   nY - the y coordinate of an event
   *@param   bBlackUp - whether black should be up
   *@return  returns a number between 0 and 7
   *
   **/
  
  public int transformY (int nY, boolean bBlackUp)
    {
      int nTemp = (nY - nMargin) / squareSize;
      return (bBlackUp ? 7 - nTemp : nTemp);
    }
  
  /**  
   *  
   *@return computes and returns the width of the checker board
   *
   **/
  
  public int requiredWidth ()
    {
      return 8 * squareSize + 2 * nMargin;
    }
  
  
  /**  
   *  
   *@return computes and returns the height of the checker board
   *
   **/
  
  public int requiredHeight ()
    {
      return 8 * squareSize + 2 * nMargin;
    }
  
  
  /**
   *
   *@param   out     the output stream
   *@return  writes the checker board to a stream (actually, 
   *            there is nothing to write)
   *
   */
  public void MarshallSelf(DataOutput out) throws IOException
    {
      //System.out.println ("Board::MarshallSelf");
    }
  

  /**
   *
   *@param   in      the input stream
   *@return  reads the checker board from a stream (actually, 
   *            there is nothing to read)
   *
   */

  public void UnmarshallSelf(DataInput in) throws IOException
    {
      //System.out.println ("Board::UnmarshallSelf");
    }
  
  //  END of Board
}


