/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package Checkers;
import java.awt.*;
import ncsa.habanero.MirrorFrame;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;

/***
 ***  Checker - holds a checker
 ***/

public class Checker extends Piece implements ncsa.habanero.Marshallable {
  //  Local Variables
  protected boolean bLost;   // Is this checker captured?
  protected boolean bKing;   // Is this checker a king?
  protected boolean bBlack;  // Is this checker a black checker?

  /**
   *
   *@return creates a checker, should only be used when marshalling a checker
   *
   */
  
  public Checker ()
    {
      super();
      bLost = false;
    }
  
  /**
   *
   *@param   bIsBlack   true if the checker should be a black checker
   *@return  creates a checker
   *
   */
  
  public Checker (boolean bIsBlack)
    {
      super();
      bLost = false;
      bKing = false;
      bBlack = bIsBlack;
    }


  /**
   *
   *@param   G         a graphics object to draw in
   *@param   board     the board where the checker is to be drawn
   *@param   bBlackUp  true if the black checkers are at the top of the page
   *@return  draws the checker
   *
   */
  
  public void paint (Graphics G, Board board, boolean bBlackUp)
    {
      if (bLost)
	return;
      
      //  Draw the object here
      if (bBlack)
	G.setColor (Color.black);
      else
	G.setColor (Color.red);
      
      G.fillOval (board.drawPieceX (nX, bBlackUp),
		  board.drawPieceY (nY, bBlackUp),
		  board.squareSize-4,
		  board.squareSize-4);
      
      G.setColor (Color.yellow);
      G.drawOval (board.drawPieceX (nX, bBlackUp),
		  board.drawPieceY (nY, bBlackUp),
		  board.squareSize-4,
		  board.squareSize-4);
      
      if (bKing) {
	//  Draw the crown
	int nStep = (board.squareSize - 8) / 5;
	
	int pnXVals [] = {0, 0, 1, 2, 3, 4, 4, 0};
	int pnYVals [] = {2, 0, 1, 0, 1, 0, 2, 2};
	
	for (int ndx = 0; ndx < 8; ndx++) {
	  pnXVals [ndx] *= nStep;
	  pnXVals [ndx] += board.drawPieceX (nX, bBlackUp) +
	    (board.squareSize + 32) / 10;
	  pnYVals [ndx] *= nStep;
	  pnYVals [ndx] += board.drawPieceY (nY, bBlackUp) + 
	    (3 * board.squareSize + 16) / 10;
	}
	
	G.drawPolygon (pnXVals, pnYVals, 8);        
      }
    }
  
  /**  
   *
   *  
   *@param   nQueryX    the x coordinate
   *@param   nQueryY   the y coordinate
   *@return  returns true if this is the checker
   *
   */

  public boolean hitP (int nQueryX, int nQueryY)
    {
      return (!bLost && super.hitP(nQueryX, nQueryY));
    }
  
  /**  
   *  
   *@return  Captures the checker
   *
   */

    public void capture ()
      {
        bLost = true;
        moveTo (-1, -1);
      }
  
  /**  
   *  
   *@return  returns true if the checker is a king
   *
   */

    public boolean isKingP () 
      {
        return bKing;
      }

  /**  
    *  
    *@return  returns true if the checker is black
    *
    */
  
  public boolean isBlackP ()
    {
      return bBlack;
    }
  
  
  /**  
    *  
    *@return  returns true if the checker is captured
    *
    */
  
  public boolean isCapturedP ()
    {
      return bLost;
    }


  /**  
   *
   *@return  returns true if the checker should be kinged
   *
   */

  public boolean shouldKingMeP () 
    {
      //System.out.println ("Checker:shouldKingMeP");
      return ((!bBlack && (nY == 7)) || (bBlack && (nY == 0)));
    }
  
  /**  
   *  
   *@return  kings the checker
   *
   */
  
  public void kingMe () 
    {
      //System.out.println ("Checker:kingMe");
      bKing = true;
    }
  
  /**  
   *  
   *@param  src - the checker to copy
   *@return  copies a checker
   *
   */
  
  public void copy (Checker src)
    {
      super.copy (src);
      bLost  = src.bLost;
      bKing  = src.bKing;
      bBlack = src.bBlack;
    }
  
  /**
   *  
   *@return  creates a new checker, and copies this checker into it
   *
   */
  
  public Object clone () 
    {
      Checker chNew = new Checker (bBlack);
      chNew.copy(this);
      chNew.bKing = bKing;
      chNew.bLost = bLost;
      return (Object) chNew;
    }
  
  /**
   *  
   *@return  returns true if the checker is on the board
   *
   */
  
  public boolean legalPosition ()
    {
      return ((nX >= 0) && (nX <= 7) && (nY >= 0) && (nY <= 7));
    }
  
  
  /**  
   *  
   *@param   other - the other checker
   *@return  returns the checker between other and this
   *
   */
  
  public Checker between (Checker other)
    {
      Checker chNew;
      
      chNew = (Checker) this.clone();
      
      chNew.nX = (nX + other.nX) / 2;
      chNew.nY = (nY + other.nY) / 2;
      
      return chNew;
    }
  
  
  /**  
   *  
   *@param   out  the output stream
   *@return  Writes the object to the output stream
   *
   */
  
  public void MarshallSelf(DataOutput out) throws IOException
    {
      //System.out.println ("DEBUG:  Checker::MarshallSelf");
      
      super.MarshallSelf (out);
      out.writeBoolean(bLost);
      out.writeBoolean(bKing);
      out.writeBoolean(bBlack);
    }
  
  /**  
   *  
   *@param   in     the input stream
   *@return  Reads the object to the input stream
   *
   */
  
  public void UnmarshallSelf(DataInput in) throws IOException
    {
      //System.out.println ("DEBUG:  Checker::UnmarshallSelf");
      
      super.UnmarshallSelf (in);
      bLost  = in.readBoolean();
      bKing  = in.readBoolean();
      bBlack  = in.readBoolean();
    }
  
  
  /**
   *  
   *@return  Translates a checker into a string
   *
   */
  
  public String toString() 
    {
      return super.toString() + (isBlackP() ? " black" : " red") + 
	(isKingP() ? " king": "");
    }
  
  //  END of Checker
}


