/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package Checkers;
import ncsa.habanero.*;
import ncsa.habanero.granted.*;
import ncsa.habanero.tickets.*;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;
import java.util.Vector;
import java.util.Enumeration;
import java.awt.Event;

/**
 * CheckerArbitrator contains the specific rules for a checker game
*/
public class CheckerArbitrator extends TurnTakerArbitrator {
  
  //  Local Variables
  boolean bBlacksTurn = false;               //  Blacks move - red goes first
  CheckerBoard board = new CheckerBoard ();  //  The checker board
  int nLocalPlayers = 0;                     //  The number of people who have checked in
  boolean bMoreJumps = false;                //  Is the player in the middle of a 
                                             //     multi checker jump?
  
  /**
   *
   *@param parent the parent arbitrator
   *@param r      the router
   *@return  creates a checkers arbitrator
   *
   */
  public CheckerArbitrator(Arbitrator parent, Router r) 
    {
      super(parent, r);
      //System.out.println ("CheckerArbitrator::CheckerArbitrator");
    }
  
  /**
   *
   *@return   creates a checkers arbitrator
   *
   */
  public CheckerArbitrator() 
    {
      super();
      //System.out.println ("CheckerArbitrator::CheckerArbitrator");
    }


  /**
   * getLock
   *
   *@param   lpdRequest - what you would like to do
   *@param   oldKey - the key that permits us to get the lock
   *@return  checks to see if you are allowed to do that, and 
   *            returns a key
   *
   */
  public synchronized Key getLock(Lock ldRequest, Key oldKey) 
    {
      //System.out.println ("CheckerArbitrator::getLock");
      
      //  Now check to see if they are requesting a color
      if (ldRequest instanceof ColorLock) {
	//System.out.println ("CheckerArbitrator::getLock -- have a ColorLock");
	ColorLock cldRequest = (ColorLock) ldRequest;

        //  Try to give them that color
	Key kyGranted;
	if (nLocalPlayers < 2) {
	  kyGranted = super.getLock (ldRequest, oldKey);

	  if (kyGranted.equals(Key.NoKey))
	    return Key.NoKey;

	  //  Keep track of the people actually granted a color
	  nLocalPlayers++;

	  //  Set the turn to blacks turn if necessary
	  if ((getPlayers () == 2) && (bBlacksTurn) && (nLocalPlayers == 2))
	    setTurn (2);
	}
	else
	  kyGranted = Key.NoKey;

        //  Once we have two players, thats it folks...
        if (nLocalPlayers >= 2) {
            Event ac = 
                new Event(this, Event.ACTION_EVENT, "Observer");
 
            //  Broadcast that all players are selected
            route_to_everyone(grantNewAction(new AppletAction(ac), owner));
            }

        //  Remember to give them the key that they asked for
        return kyGranted;
      }
      
      //  See if this key is a movement and it has already been granted
      if (ldRequest instanceof CheckerMoveLock) {
	//System.out.println ("CheckerArbitrator::getLock --  Have a CheckerMoveLock");

	Key keyTemp = lookupLock (ldRequest, oldKey);
	if (!keyTemp.equals(Key.NoKey)) {
	  //System.out.println ("CheckerArbitrator::getLock --  Found an old Lock");
	  return keyTemp;
	}

	CheckerMoveLock cmlLock = (CheckerMoveLock) ldRequest;
	if (cmlLock.blackColor() != bBlacksTurn) {
	  //System.out.println ("CheckerArbitrator::getLock --  Not that turn");
	  return Key.NoKey;
	}
      }
      
      //System.out.println ("CheckerArbitrator::GetLock --  Have a Miscellaneous Lock");
      return super.getLock (ldRequest, oldKey);
    }
  
  
  /**
   *
   *@param   kFinished - the key of the lock that is no longer needed
   *@return  releases the lock, if we aren't doing a jump
   *
   */
  public synchronized void releaseLock (Key kFinished) {
    if (!bMoreJumps)
      super.releaseLock (kFinished);
  }

  /**
   * The arbitrate routine simply grants the request and asks
   * the router to send it to everyone
   *@param request the ActionRequest we want to grant
   *@return true if the action was sent out, false otherwise
   *
   */
  public synchronized boolean arbitrate(ActionRequest request) 
    {
      //System.out.println ("CheckerArbitrator::arbitrate");

      Action actThis = request.action();
      
      MoveChecker mckAction = null;
      try {
	mckAction = (MoveChecker) ((AppletAction)actThis).getEvent().arg;
      }
      catch (java.lang.ClassCastException cce) {
	System.out.println ("CheckerArbitrator::arbitrate bad cast");
	return super.arbitrate(request);
      }
      
      if (legalAdvanceP (mckAction) && super.arbitrate (request)) {
        advance (mckAction);
	return true;
      }
      
      if (legalJumpP (mckAction) && super.arbitrate (request)) {
        jump (mckAction);
	return true;
      }
      
      return false;
    }

  /**
   *
   *@param mckAction the action, perhaps an advancing checker
   *@returns true if the action is a legal advance
   *  
   **/
  
  private boolean legalAdvanceP (MoveChecker mckAction) 
    {
      return (oneStep (mckAction) && legalDirection (mckAction) 
	      && !landOnOther(mckAction));
    }
  
  /**
   *
   *@param  mckAction the action, an advance
   *@returns nothing, but performs the advance on the local board
   *  
   **/
  
  private void advance (MoveChecker mckAction)
    {
      board.moveChecker(mckAction.startChecker(), mckAction.endChecker());
      bBlacksTurn = !bBlacksTurn;
    }
  
  /**
   *
   *@param  mckAction   the action, perhaps a jumping checker
   *@returns true if the action is a legal jump
   *  
   **/
  
  private boolean legalJumpP (MoveChecker mckAction)
    {
      return (twoStep (mckAction) && legalDirection (mckAction) 
	      && !landOnOther(mckAction) && capture (mckAction));
    }
  
  /**
   *
   *@param  mckAction   the action, a jump
   *@returns nothing, but performs the advance on the local board and broadcast
   *     the results to other systems
   *  
   **/
  
  private void jump (MoveChecker mckAction)
    {
      //System.out.println ("CheckerArbitrator::jump");
      
      //  Local Movement
      board.moveChecker(mckAction.startChecker(), mckAction.endChecker());
      
      //  Broadcast the removal
      Checker chDead = (mckAction.startChecker()).between (mckAction.endChecker());
      Event ac = 
	new Event(this, Event.ACTION_EVENT, new RemoveChecker (chDead));
      route_to_everyone(grantNewAction(new AppletAction(ac), owner));
      
      //  Local capture
      board.captureChecker(chDead);
      
      //  Determine if there are more actions
      if (moreCaptures (mckAction)) {
        //  Create a new Locker, and discard the key
        Locker lkResult = 
	  new Locker (new CheckerMoveLock (mckAction.endChecker()));
        validLockSet.addElement (lkResult);
	
        //  Turn remains the same
	//System.out.println ("CheckerArbitrator::jump - same turn");
	if (mckAction.startChecker().isBlackP())
	  setTurn (2);
	else
	  setTurn (1);

	//  Remember that we can't release the lock until the jump is complete
	bMoreJumps = true;
      }
      else {
        //  Switch turns
        bBlacksTurn = !bBlacksTurn;

	//  Allow the person to release the lock
	bMoreJumps = false;
      } 
    }
  
  /**
   *
   *@param   lpdRequest - what you would like to do
   *@param   oldKey - the key that permits us to get the lock
   *@return  returns the old key, if there is one
   *
   */
  private Key lookupLock (Lock ldRequest, Key oldKey) 
    {
      //System.out.println ("CheckerArbitrator::lookupLock");
      Enumeration AllLocks = validLockSet.elements();
      
      while (AllLocks.hasMoreElements()) {
	Locker aLocker = (Locker) AllLocks.nextElement();
	//System.out.println ("CheckerArbitrator::lookupLock - trying");
	if (aLocker.equals(ldRequest)) {
	  //System.out.println ("CheckerArbitrator::lookupLock - found a matching lock");
	  return aLocker.getKey();
	}
      }
      
      //System.out.println ("CheckerArbitrator::lookupLock - no existing locks");
      return Key.NoKey;
    }
  
  /**
   *
   *@param   mckAction - the checker action
   *@return  returns true if the checker only moves one square
   *
   */
  private boolean oneStep (MoveChecker mckAction)
    {
      //System.out.println ("CheckerArbitrator::oneStep");
      
      Checker ch1 = mckAction.startChecker();
      Checker ch2 = mckAction.endChecker();
      
      return ((Math.abs (ch1.getXDist (ch2)) == 1) && 
	      (Math.abs (ch1.getYDist (ch2)) == 1));
    }
  
  
  /**
   *
   *@param   mckAction - the checker action
   *@return  returns true if the checker only moves two squares
   *
   */
  private boolean twoStep (MoveChecker mckAction)
    {
      //System.out.println ("CheckerArbitrator::twoStep");
      
      Checker ch1 = mckAction.startChecker();
      Checker ch2 = mckAction.endChecker();
      
      return ((Math.abs(ch1.getXDist (ch2)) == 2) && (Math.abs(ch1.getYDist (ch2)) == 2));
    }
  
  /**
   *
   *@param   mckAction - the checker action
   *@return  returns true if the checker can move in that direction
   *
   */
  private boolean legalDirection (MoveChecker mckAction)
    {
      //System.out.println ("CheckerArbitrator::legalDirection");
      
      Checker ch1 = mckAction.startChecker();
      Checker ch2 = mckAction.endChecker();
      
      return (((ch1.getYDist (ch2) < 0) && (ch1.isBlackP())) ||
	      ((ch1.getYDist (ch2) > 0) && (!ch1.isBlackP()))
	      || (ch1.isKingP()));
    }
  
  /**
   *
   *@param   mckAction - the checker action
   *@return  returns true if the checker will not land on another checker
   *
   */
  private boolean landOnOther (MoveChecker mckAction)
    {
      //System.out.println ("CheckerArbitrator::landOnOther");
      
      Checker ch2 = mckAction.endChecker();
      
      return (ch2.legalPosition () && 
	      (board.anyCheckerHere (ch2, true) ||
	       board.anyCheckerHere (ch2, false)));
    }
  
  
  /**
   *
   *@param   mckAction - the checker action
   *@return  returns true if the checker can capture more pieces
   *
   */
  private boolean moreCaptures (MoveChecker mckAction)
    {
      //System.out.println ("CheckerArbitrator::moreCaptures");
      
      return board.moreCaptures (mckAction.startChecker(), mckAction.endChecker());
    }
  
  /**
   *
   *@param   mckAction - the checker action
   *@return  returns true if the checker can capture one piece
   *
   */
  private boolean capture (MoveChecker mckAction)
    {
      //System.out.println ("CheckerArbitrator::capture");
      
      Checker ch1 = mckAction.startChecker();
      Checker ch2 = mckAction.endChecker();
      Checker dead = ch1.between (ch2);

      return board.anyCheckerHere (dead, !ch1.isBlackP ());
    }
}






