/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package Checkers;
import java.awt.*;
import ncsa.habanero.MirrorFrame;
import ncsa.habanero.Marshalling;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;

/***
 ***  CheckerBoard - holds a checker board
 ***/

public class CheckerBoard extends Board implements ncsa.habanero.Marshallable {

  //  Local Variables
  Checker pckPieces[] = null;  //  The checker peices

  /**
   *
   *@return a new board for a checker game, filled with pieces
   *
   */
  
  public CheckerBoard ()
    {
      super();
      //System.out.println ("DEBUG:  CheckerBoard::CheckerBoard");
      
      pckPieces = new Checker [24];
      
      int ndx;
      for (ndx = 0; ndx < 12; ndx++) {
	pckPieces [ndx] = new Checker (false);
	pckPieces [ndx].moveTo (2 * (ndx % 4) + (((ndx/4) != 1) ? 1 : 0), 
				ndx/4);
      }
      
      for (ndx = 12; ndx < 24; ndx++) {
	pckPieces [ndx] = new Checker (true);
	pckPieces [ndx].moveTo (2 * (ndx % 4) + (((ndx/4) == 4) ? 1 : 0), 
				ndx/4 + 2);
      }
    }
  
  /**  
   *
   *  draws a checker board, with the checker pieces on it
   *
   *@param G         the graphics object
   *@param bBlackUp  whether the lower left square should be black
   *@return          draws the checker board
   *
   */
  
  public void paint (Graphics G, boolean bBlackUp)
    {
      super.paint(G, bBlackUp);
      
      for (int ndx = 0; ndx < 24; ndx++) {
	pckPieces [ndx].paint (G, this, bBlackUp);
      }
    }
  

  /**
   *
   *  returns the piece that was hit
   *
   *@param   nQX     the x coordinate, 0 - 7
   *@param   nQY     the y coordinate, 0 - 7
   *@return  returns the checker hit or null if none were hit   
   *
   */

  public Piece findHit (int nQX, int nQY)
    {
      //System.out.println ("DEBUG:  CheckerBoard::findHit");
      
      for (int ndx = 0; ndx < 24; ndx++) {
	if (pckPieces [ndx].hitP (nQX, nQY)) {
	  return pckPieces[ndx];
	}
      }
      
      return (Piece) null;
    }


  /**  
   *
   *  moves a checker on the board
   *  
   *@param   chOld    the old checker location, stored as a checker
   *@param   chNew    the new checker location, stored as a checker
   *@return  moves the checker from the old location to the new location
   *
   */

    public void moveChecker (Checker chOld, Checker chNew)
      {
        //System.out.println("DEBUG:  CheckerBoard::moveChecker");
	
        for (int ndx = 0; ndx < 24; ndx++) {
	  if (pckPieces [ndx].equals (chOld)) {
	    //  Determine if someone is kinged
	    if (chNew.shouldKingMeP()) {
	      //  King the checker
	      pckPieces [ndx].kingMe();
	      chNew.kingMe();
	    }
	    
	    pckPieces [ndx].copy (chNew);
	  }
	}
      }
  

  /**  
   *
   *  removes a checker from the board
   *
   *@param   chDead the captured checker
   *@return  removes the checker from the board
   *
   */
  
  public void captureChecker (Checker chDead)
    {
      //System.out.println ("DEBUG:  CheckerBoard::captureChecker");
      
      for (int ndx = 0; ndx < 24; ndx++) {
	if (pckPieces [ndx].equals (chDead)) {
	  pckPieces [ndx].capture ();	  
	  return;
	}
      }
    }


  /**
   *
   *@param   chMove    the location, stored as a checker
   *@return  true if any checkers are at that location, and false otherwise
   *
   */

  public boolean anyCheckerHere (Checker chMove, boolean bBlackP)
    {
      //System.out.println ("DEBUG:  CheckerBoard::anyCheckerHere");
      Checker chActual = (Checker) findHit (chMove.nX, chMove.nY);
      
      if (chActual == null)
	return false;
      else
	return (chActual.isBlackP () == bBlackP);
    }
  

  /**
   *
   *  Determines if someone has one
   *
   *@param   bBlackWin   are we trying to determine if black won?
   *@return  returns true if bBlackWin has won, false otherwise
   *
   */
  
  public boolean winnerP (boolean bBlackP)
    {
      //System.out.println ("DEBUG:  CheckerBoard::winnerP");
      
      int nOffset = (bBlackP ? 0 : 12);
      
      for (int ndx = 0; ndx < 12; ndx++) {
	if (!pckPieces [nOffset + ndx].isCapturedP()) {
	  return false;
	}
      }
      return true;
    }
  

  /**
   *
   *@param   chPiece   the checker we are looking for
   *@param   nXD       the x direction
   *@param   nYD       the y direction
   *@return  returns true if there is a piece to capture in that direction
   *
   */

  private boolean directedCapture (Checker chPiece, int nXD, int nYD)
    {
      //System.out.println ("DEBUG:  CheckerBoard::directedCapture");
      
      int nDestX = chPiece.nX + 2 * nXD;
      int nDestY = chPiece.nY + 2 * nYD;
      
      if ((nDestX < 0) || (nDestX > 7) || (nDestY < 0) || (nDestY > 7))
	return false;
      
      Checker chVictim = (Checker) findHit (chPiece.nX + nXD, chPiece.nY + nYD);
      
      if ((chVictim != null) && 
	  (chVictim.isBlackP() != chPiece.isBlackP()) &&
	  (findHit (nDestX, nDestY) == (Piece) null)) {
	
	return true;
      }
      
      return false;
    }
  

  /**
   *
   *  returns true if there are more pieces to capture
   *
   *@param   chStart   where the checker started from
   *@param   chPiece   the piece we are checking to see if it can capture more
   *                   checkers
   *@return  returns true if it can capture more, uses the direction that the
   *                   checker came from to ignore
   */

    public boolean moreCaptures (Checker chStart, Checker chPiece)
      {
        //System.out.println ("DEBUG:  CheckerBoard::moreCaptures");
	
        //  Determine the direction of movement
        int nDX = chPiece.getXDist (chStart);
        int nDY = chPiece.getYDist (chStart);
	
        if (chPiece.isBlackP() || chPiece.isKingP()) {
	  if ((!((nDX > 0) && (nDY < 0)) && directedCapture (chPiece,  1, -1)) || 
	      (!((nDX < 0) && (nDY < 0)) && directedCapture (chPiece, -1, -1)) )
	    return true;
	}
        if (!chPiece.isBlackP() || chPiece.isKingP()) {
	  if ((!((nDX > 0) && (nDY > 0)) && directedCapture (chPiece,  1,  1)) || 
	      (!((nDX < 0) && (nDY > 0)) && directedCapture (chPiece, -1,  1)) )
	    return true;
	}
        
        return false;
      }


  /**
   *
   *@param   out     the output stream
   *@return  writes the checker board to a stream, sends the location of all checkers
   *
   */
    public void MarshallSelf(DataOutput out) throws IOException
      {
        //System.out.println ("DEBUG:  CheckerBoard::MarshallSelf");
	
        for (int ndx = 0; ndx < 24; ndx++)
	  (pckPieces [ndx]).Marshall (out);
      }
  
  /**
   *
   *@param   in      the input stream
   *@return  reads the checker board from a stream, primary work is 
   *         receiving the checkers
   *
   */
  
  public void UnmarshallSelf(DataInput in) throws IOException
    {
      //System.out.println ("DEBUG:  CheckerBoard::UnmarshallSelf");
      
      if (pckPieces == null) {
	pckPieces = new Checker [24];
      }
      
      //  Read the checkers
      for (int ndx = 0; ndx < 24; ndx++) {
	pckPieces [ndx] = (Checker) Marshalling.Unmarshall (in);
      }
    }
  
  //  END of CheckerBoard
}


