/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */

package Checkers;
import java.awt.*;
import ncsa.habanero.*;
import ncsa.habanero.granted.*;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;




/**
 * A game of Checkers
 *
 *@version     1.1, March 11, 1996
 */
public class CheckerGame extends java.applet.Applet 
    implements ncsa.habanero.Wrapped  {
  
  //  Internal Variables
  CheckerBoard board;         //  The current checker board
  Key kyColor = Key.NoKey;    //  Do we have a key
  Key kyMoving = Key.NoKey;   //  The key to move a piece
  Checker chMoving;           //  The original checker location 
  Checker chCurrent;          //  The current checker position
  MirrorFrame frame;          //  The frame we are working with
  boolean bBlackUp;           //  The black value

  boolean bMyTurn;            //  Is it my turn
  boolean bTurnSet;           //  Do we know whose turn it is yet?

  boolean bBufferReady;       //  Is the double buffer ready?
  Image offScreenImage;       //  The offscreen drawing buffer
  Graphics offScreenGraphic;  //  The graphic used to draw in the offscreen buffer

  Label lblMessages;        //  The area for messages

  static int nStrangeWindowWidth = 10;
  static int nStrangeWindowHeight = 30;
  static int nMessageHeight = 20;


  /**
   *
   *@returns   a new checker game object, initializes the board and the pieces
   *
   **/
  
  public CheckerGame () 
    {
      bBufferReady = false;
      board = new CheckerBoard ();
      bTurnSet = false;
      bBlackUp = true;
      bMyTurn = false;
    }

  /**  
   *
   *@param  f  the frame
   *@return   starts the applet rolling
   *
   **/
  public void startInFrame(MirrorFrame f)
      {
      //System.out.println ("CheckerGame::startInFrame");
      
      frame = f;
      
      Panel workArea = new Panel ();
      workArea.setLayout (new BorderLayout ());
      lblMessages = new Label ("Choose a side by selecting a checker", 
        Label.CENTER);
      
      f.add("Center", workArea);
      f.setTitle("Checkers");

      workArea.add("Center", this);
      workArea.add ("South", lblMessages);
      
      //  the +10 and +30 are a hack to get the right window size on
      //  a twm window manager window - the AWT doesn't take the extra
      //  window stuff into account
      
      f.resize(board.requiredWidth()  + nStrangeWindowWidth, 
               board.requiredHeight() + nStrangeWindowHeight + nMessageHeight);
      f.show();
    }

  /**
   *
   * Allows the user to try to select a side
   *
   *@param  nX   the x location of the checker you would like to be
   *@param  nY   the y location of the checker you would like to be
   *@return attempts to request the side that you want to play
   *
   **/ 
  private boolean selectSide (int nX, int nY) 
    {
      //  Set the limit on the number of players
      Arbitrator arbitrator = (frame.getCollobject()).arbitrator();
      Key kyPlayerLimit = arbitrator.getLock (new LimitPlayerLock (2), Key.NoKey);

      //  Tell that we aren't moving anything
      chMoving = null;
      chCurrent = null;

      //  Find the checker we have selected
      int nXPos = board.transformX(nX, bBlackUp);
      int nYPos = board.transformY(nY, bBlackUp);
        
      Checker chSelection = (Checker) board.findHit (nXPos, nYPos);
      if (chSelection == null)
          return true;

      //  Determine the side
      boolean bSide = chSelection.isBlackP();

      //  Ask if we can play that side
      kyColor = arbitrator.getLock (new ColorLock (bSide), Key.NoKey);
      
      //  Determine success
      if (!kyColor.equals (Key.NoKey)) {
	if (!bSide) {
          lblMessages.setText ("Red Player");
          bMyTurn = true;
          bTurnSet = true;
          bBlackUp = true;
          }
        else { 
          lblMessages.setText ("Black Player");
          bBlackUp = false;
          if (!bTurnSet) {
              bMyTurn = !bSide;
              bTurnSet = true;
              }
          }

        repaint ();
      }
      else {
          if (!bTurnSet)
              lblMessages.setText ("Sorry - try again");
          }
      return true;      
    }  

  /**
   *
   * forces the person to be an observer
   *
   *@return sets up the local variables as if the person was an observer
   *
   **/ 
  private void becomeObserver ()
    {
    kyColor = Key.NoKey;
    bBlackUp = true;
    lblMessages.setText ("Observer");
    bMyTurn = false;
    bTurnSet = true;
    }  

  /**
   *  
   *
   *@param  g  the graphics object
   *@return draws the next board
   *
   **/
  
  public void update (Graphics g) 
    {
      if (!bBufferReady) {
        bBufferReady = true;
        offScreenImage = createImage (board.requiredWidth(), 
                                      board.requiredHeight());
        offScreenGraphic = offScreenImage.getGraphics();
        }

      offScreenGraphic.setColor ((bMyTurn ? Color.yellow : Color.darkGray));
      offScreenGraphic.fillRect (0,0, 
                                 board.requiredWidth(), 
                                 board.requiredHeight());

      board.paint (offScreenGraphic, bBlackUp);

      g.drawImage (offScreenImage, 0, 0, this);
    }
  
  
  /**
   *  
   *
   *@param  g  the graphics object
   *@return draws the next board
   *
   **/
  
  public void paint(Graphics g) 
    {
      update (g);
    }
  
  
  /**
   *
   *@param  evt  the event that has been broadcast to all objects
   *@param  arg  the event's argument - probably a marshallable object
   *@return handles the incoming object and performs the necessary update
   *     to the screen
   *
   **/
  
  public boolean doEvent(Event evt, Object arg)
    {
      if ((arg.getClass().getName()).equals ("Checkers.CheckerBoard")) {
        try {
          board = (CheckerBoard) arg;
          repaint();
        }
        catch (java.lang.ClassCastException cce) {
          System.out.println("arg is not a CheckerBoard");
          System.out.println(arg.getClass().getName());
        }
      }
      else if ((arg.getClass().getName()).equals ("Checkers.MoveChecker")) {
        try {
          MoveChecker action = (MoveChecker) arg;
          board.moveChecker (action.startChecker(), action.endChecker());

          if (!kyColor.equals(Key.NoKey) &&
              !(board.moreCaptures (action.startChecker(), action.endChecker()) && 
               (Math.abs (action.endChecker().getXDist(action.startChecker())) > 1))
             ) {
              bMyTurn = !bMyTurn;
              }
          repaint();
        }
        catch (java.lang.ClassCastException cce) {
          System.out.println("arg is not a MoveChecker");
          System.out.println(arg.getClass().getName());
        }
      }
      else if ((arg.getClass().getName()).equals ("Checkers.RemoveChecker")) {
        try {
          RemoveChecker action = (RemoveChecker) arg;
          board.captureChecker (action.deadChecker());

          checkForWinner ();

          repaint();
        }
        catch (java.lang.ClassCastException cce) {
          System.out.println("arg is not a RemoveChecker");
          System.out.println(arg.getClass().getName());
        }
      }
      else if (arg.equals ("Observer")) {
          if (!bTurnSet)
              becomeObserver();
      }
      else {
        System.out.println("arg is not something I know how to deal with  ");
        System.out.println(arg.getClass().getName());            
      }
      return true;
    }

  /**
   *
   * checks to see if we have a winner
   *
   *@return deterines if someone has won the game, and updates the message board
   *   appropriately
   *
   **/

  private void checkForWinner ()
    {
    //  Determine if we have a winner
    if (board.winnerP (!bBlackUp)) {
        if (kyColor.equals(Key.NoKey)) {
            lblMessages.setText ("Game Over");
            }
        else {
            lblMessages.setText ("Game Over -- You win!!");
            }
        }
    if (board.winnerP (bBlackUp)) {
        if (kyColor.equals(Key.NoKey)) {
            lblMessages.setText ("Game Over");
            }
        else {
            lblMessages.setText ("Game Over -- You lost.");
            }
        }
    }
  

  /**
   *
   *@param  e  the event to be handled
   *@returns intercepts events at the local level and performs the necessary processing
   *
   **/  
  
  public boolean handleEvent(Event e) {
    switch (e.id) {
        case Event.MOUSE_DOWN:
            if (bTurnSet)
                return selectChecker (e.x, e.y);
            else
                return selectSide (e.x, e.y);
      
        case Event.MOUSE_DRAG:
            return localMoveChecker (e.x, e.y);
      
        case Event.MOUSE_UP:
            return broadcastMoveChecker (e.x, e.y);
      
    case Event.ACTION_EVENT:
      return false;
      
    default:
      return super.handleEvent(e);
    }
  }


  /**
   * 
   *@param   nX  the x coordinate of the mouse
   *@param   nY  the y coordinate of the mouse
   *@return to select a checker to be moved
   *
   **/

  private boolean selectChecker (int nX, int nY)
      {
      if (kyColor.equals (Key.NoKey)) {
          //System.out.println ("CheckerGame::handleEvent::MOUSE_DOWN  not playing");
          return false;
          }
        
      int nXPos = board.transformX(nX, bBlackUp);
      int nYPos = board.transformY(nY, bBlackUp);
        
      chCurrent = (Checker) board.findHit (nXPos, nYPos);
      if (chCurrent != null) {
          
          //  Clone the checker
          chMoving = (Checker) chCurrent.clone();
          
	  Arbitrator arbitrator = (frame.getCollobject()).arbitrator();
          kyMoving = arbitrator.getLock (new CheckerMoveLock(chMoving), 
                                         kyColor);
          if (kyMoving.equals(Key.NoKey)) {
              chMoving = null;
              chCurrent = null;
              }
          else {
              //  Correct the border since we know it is now our turn
              bMyTurn = true;
              }
          }
      else {
          chMoving = null;
          chCurrent = null;
          }
      return true;
      }
  
  
  /**
   * 
   *@param   nX  the x coordinate of the mouse
   *@param   nY  the y coordinate of the mouse
   *@return to move the selected checker locally
   *
   **/

  private boolean localMoveChecker (int nX, int nY)
      {
      if (kyColor.equals (Key.NoKey)) {
        //System.out.println ("CheckerGame::handleEvent::MOUSE_DOWN  not playing");
        return false;
        }
      
      if (chMoving != null) {
        int nXPos = board.transformX(nX, bBlackUp);
        int nYPos = board.transformY(nY, bBlackUp);
        
        if (chCurrent.moveTo (nXPos, nYPos))
            repaint ();
        }

      return true;
      }

  /**
   * 
   *@param   nX  the x coordinate of the mouse
   *@param   nY  the y coordinate of the mouse
   *@return to move the selected checker on all checker boards
   *
   **/

  private boolean broadcastMoveChecker (int nX, int nY)
      {
      if (kyColor.equals (Key.NoKey)) {
          //System.out.println ("CheckerGame::handleEvent::MOUSE_DOWN  not playing");
          return false;
          }
      if ((chMoving == null) || (chCurrent == null))
          return false;
        
      int nXPos = board.transformX(nX, bBlackUp);
      int nYPos = board.transformY(nY, bBlackUp);
        
      chCurrent.moveTo (nXPos, nYPos);
      Checker chFinalPosition = (Checker) chCurrent.clone ();
      chCurrent.copy (chMoving);
        
      Event ac = new Event(this, Event.ACTION_EVENT, 
                           new MoveChecker (chMoving, chFinalPosition));
      KeyedActionRequest req = new KeyedActionRequest(new AppletAction(ac), 
                                                      frame.getCollobject(),
                                                      kyMoving);
      if (!req.requestGranted()) {
          frame.getCollobject().arbitrator().releaseLock(kyMoving);
          repaint();
          }

      return true;
      }

  /**
   *
   *  Writes the current state of the checker game to the output stream
   *
   *@param  out  the output stream
   *@returns saves the state of the checker game to the output stream
   *
   */
  
  public void MarshallSelf(DataOutput out) throws IOException
    {
      //System.out.println ("DEBUG:  CheckerGame::MarshallSelf");
      
      board.Marshall (out);
      out.writeBoolean (bMyTurn);
    }
  

  /**
   *
   *  Reads the state of a checker game from the stream
   *
   *@param  in  the input stream
   *@returns loads the state of a checker game
   *
   **/
  
  public void UnmarshallSelf(DataInput in) throws IOException
    {
      //System.out.println ("DEBUG:  CheckerGame::UnmarshallSelf");
      
      if (board == null) {
        board = new CheckerBoard ();
      }
      
      board = (CheckerBoard) ncsa.habanero.Marshalling.Unmarshall (in);
      bMyTurn = !(in.readBoolean ());
    }
  
  //  End of CheckersGame
}
















