/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package Checkers;
import java.awt.*;
import ncsa.habanero.*;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;

/***
 ***  PlayerLock - a limitation on the lock
 ***/

public class ColorLock extends ncsa.habanero.PlayerLock {

  //  Internal Variables
  boolean isBlack = false;

  /**
   *
   *@param   black - is this a block lock
   *@return  Creates a lock
   *
   */
  
  public ColorLock (boolean black)
    {
      super(black ? "BlackPlayer" : "RedPlayer");
      //System.out.println ("ColorLock::ColorLock");
      isBlack = black;
      
      //  Set the ID of each person
      if (black)
	setID (2);
      else
	setID (1);
    }
  
  /**
   *
   *@return  Creates a ColorLock - should only be used by Marshall
   *
   */
  
  public ColorLock ()
    {
      this (false);
    }
  
  /**
   *
   *@param   out - the output stream
   *@return  writes the lock to the output stream
   *
   */
  
  public void MarshallSelf(DataOutput out) throws IOException
    {
      //System.out.println ("ColorLock::MarshallSelf");
      super.MarshallSelf(out);
      out.writeBoolean(isBlack);
    }
  
  /**
   *
   *@param   in - the input stream
   *@return  reads a lock from an input stream
   *
   */
  
  public void UnmarshallSelf(DataInput in) throws IOException
    {
      //System.out.println ("ColorLock::UnmarshallSelf");
      super.UnmarshallSelf(in);
      isBlack = in.readBoolean();
    }


  /**
   *
   *@return true if the color requested is black
   *
   */
   public boolean blackColor() 
     {
       //System.out.println ("ColorLock::blackColor");
       return isBlack;
     }

  /** 
   * 
   *  The user has the key to the current lock, is the lock still legal.
   *  Returning true allows a single player to play against themselves.
   *  
   *@param     ldRequest  the request that the user is making
   *@return    always returns true
   *
   */  
  public boolean keyWorksWith (Lock ldRequest) 
    {
      //System.out.println ("ColorLock::keyWorksWith");
      return true;  
    }


//  END of ColorLock
}


