/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */

package CollabCards;

import java.awt.Event;
import java.awt.Component;
import java.applet.Applet;
import ncsa.habanero.MirrorFrame;
import ncsa.habanero.Wrapped;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;


/**
 * CollabCards is a subclass of Sun's CardTest applet which extends it to 
 * provide collaborative functionality.  To do so, it implements the 
 * ncsa.habanero.Wrapped interface, and overrides the action method to return 
 * false, ensuring that action events are handled by the collaboration 
 * mechanism.
 */
public class CollabCards extends CardTest implements Wrapped {
  /**
   * The constructor just calls the parent's constructor.
   */
  public CollabCards() {
    super();
  }

  
  /**
   * Adds this object as a subComponent of the Frame f, sets its size
   * and shows the Frame.  Implements 
   * the ncsa.habanero.Wrapped interface
   */
  public void startInFrame(MirrorFrame f) {
    init();
    start();

    f.setTitle("Cards");

    f.add("Center", this);
    f.resize(300, 300);
    f.show();
  }

  /**
   * Responds to the action event by extracting the string and calling
   * the superclass' action method.  If the argument is a String, arg
   * will be incorrect, and argAsString must be called.  Since this 
   * method is called after arbitration and serialization have occured,
   * we can pass the event on to the superclass.  Implements 
   * the ncsa.habanero.Wrapped interface
   */ 
  public boolean doEvent(Event evt, Object arg) {
//    String val = ((HabaneroEvent)evt).argAsString();
      return super.action(evt, arg);
  }

  /**
   * Any collaborative application must not handle action events directly
   * and must return false from this routine.
   */
  public boolean action(Event evt, Object arg) {
    return false;
  }

  public void MarshallSelf(DataOutput out) throws IOException {
    out.writeInt(getCard());
  }

  public void UnmarshallSelf(DataInput stream) throws IOException {
    setCard(stream.readInt());
  }

  int getCard() {
    int ncomponents = cards.countComponents();
    for (int i = 0 ; i < ncomponents ; i++) {
      Component comp = cards.getComponent(i);
      if (comp.isVisible()) {
	return i;
      }
    }
    return 0;
  }

  void setCard(int n) {
    Component next = cards.getComponent(n);
    if ((next != null) && !next.isVisible()){
      int ncomponents =  cards.countComponents();
      for (int i = 0 ; i < ncomponents ; i++) {
	Component comp =  cards.getComponent(i);
	if (comp.isVisible()) {
	  comp.hide();
	  break;
	}
      }
      next.show();
      cards.validate();
    }
  }

}
