/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */

package CollabDraw;

import java.awt.*;

import java.util.Vector;
import java.util.Enumeration;
import ncsa.habanero.MirrorFrame;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;

/**
 * CollabDraw is a subclass of Sun's DrawTest applet which extends it to 
 * provide collaborative functionality.  To do so, it implements the 
 * ncsa.habanero.Wrapped interface, and also extends the class DrawPanel to 
 * implement the drawing methods as actions.
 */
public class CollabDraw extends DrawTest implements ncsa.habanero.Wrapped {
  CollabDrawPanel dp;  // Where the drawing is actually done

  /**
   * The constructor creates the underlying parts and positions them
   */
  public CollabDraw() {
    dp = new CollabDrawPanel();
    setLayout(new BorderLayout());
    add("Center", dp);
    add("South",new DrawControls(dp));
  }  

  /**
   * Forwards to the CollabDrawPanel, which does the work. Implements 
   * the ncsa.habanero.Wrapped interface
   */
  public boolean doEvent(Event evt, Object arg) {
    dp.doEvent(evt, arg);
    return true;
  }

  /**
   * Adds this object as a subComponent of the Frame f, sets its size
   * and shows the Frame.  Implements 
   * the ncsa.habanero.Wrapped interface
   */
  public void startInFrame(MirrorFrame f) {
    start();

    f.add("Center", this);
    f.setTitle("Whiteboard");
    f.resize(300, 300);
    f.show();
  }

  /**
   * Forwards to the CollabDrawPanel, which does the work. Implements 
   * the ncsa.habanero.Wrapped interface
   */
  public void MarshallSelf(DataOutput out) throws IOException {
    dp.MarshallSelf(out);
  }

  /**
   * Forwards to the CollabDrawPanel, which does the work. Implements 
   * the ncsa.habanero.Wrapped interface
   */
  public void UnmarshallSelf(DataInput in) throws IOException {
    dp.UnmarshallSelf(in);
  }

}


/** 
 * The CollabDrawPanel extends the class DrawPanel in several important
 * ways.  It adds two vectors which act to hold information while the user is
 * drawing, thus reducing the amount of information that needs to be sent over
 * a network.  It also uses a Java Event to change state, thus allowing 
 * state-changing information to be shared.  Finally, it implements parts of 
 * the ncsa.habanero.Wrapped interface, which are forwarded from CollabDraw.
 */
class CollabDrawPanel extends DrawPanel {
  /*
   * These variables are written to while the user is drawing, and then copied
   * into the permanent variables.
   */
  Vector tmpLines = new Vector();
  Vector tmpColors = new Vector();

  public CollabDrawPanel() {
    super();
  }

  /**
   * This routine is where an application responds to an action.  In this
   * case, the variables representing the recently drawn figure are copied
   * into the permanent buffers, and the window is repainted.
   */
  public void doEvent(Event ev, Object arg) {
    try {
      DrawTestState state = (DrawTestState) arg;
      Enumeration e = state.points.elements();
      lines.ensureCapacity(lines.size()+state.points.size());
      colors.ensureCapacity(lines.capacity());
      while (e.hasMoreElements()) {
	lines.addElement(e.nextElement());
	colors.addElement(state.color);
      }
      repaint();
      }
      catch (java.lang.ClassCastException cce) {
	System.out.println("arg is not a DrawTestState");
	System.out.println(arg.getClass().getName());
      }
    }

  /**
   * The application responds to events here.  This is modified from the
   * superclass' version to use the temporary variables.  Note that only
   * mouse up events are treated as collaborative events - for mouse ups
   * we create a new action event which incorporates the temporary buffers;
   * this is then posted, which has the ultimate effect of being sent to the
   * Arbitrator.  Other events are handled locally, so that drawing is done 
   * locally and only the finished figure is handled collaboratively.
   */
  public boolean handleEvent(Event e) {
    switch (e.id) {
    case Event.MOUSE_DOWN:
      switch (mode) {
      case LINES:
	return super.handleEvent(e);
      case POINTS:
      default:
	tmpColors.addElement(getForeground());
	tmpLines.addElement(new Rectangle(e.x, e.y, -1, -1));
	x1 = e.x;
	y1 = e.y;
	repaint();
	return true;
      }
    case Event.MOUSE_UP:
      switch (mode) {
      case LINES:
	tmpColors.addElement(getForeground());
	tmpLines.addElement(new Rectangle(x1, y1, e.x, e.y));
	x2 = xl = -1;
	break;
      case POINTS:
      default:
	break;
      }
      Event ac = 
	new Event(this, Event.ACTION_EVENT, 
		  new DrawTestState(tmpLines, 
				    (Color)tmpColors.firstElement()));
      postEvent(ac);
      tmpColors.removeAllElements();
      tmpLines.removeAllElements();
      return true;
    case Event.MOUSE_DRAG:
      switch (mode) {
      case LINES:
	return super.handleEvent(e);
      case POINTS:
      default:
	tmpColors.addElement(getForeground());
	tmpLines.addElement(new Rectangle(x1, y1, e.x, e.y));
	x1 = e.x;
	y1 = e.y;
	repaint();
	return true;
      }
    case Event.ACTION_EVENT:
      return false;
    default:
      return super.handleEvent(e);
    }
  }
  
  /** 
   * First call the superclass' method to paint the permanent buffer,
   * then paint the temporary buffer.
   */
  public void paint(Graphics g) {
    super.paint(g);
    int np = tmpLines.size();
    for (int i=0; i < np; i++) {
      Rectangle p = (Rectangle)tmpLines.elementAt(i);
      g.setColor((Color)tmpColors.elementAt(i));
      if (p.width != -1) {
	g.drawLine(p.x, p.y, p.width, p.height);
      } else {
	g.drawLine(p.x, p.y, p.x, p.y);
      }
    }
  }
  
  /**
   * This routine dumps the "important state" onto an output stream.  The 
   * important state is the state which is meant to be shared.  In this case
   * it is the buffers containing the lines and the colors.  They are simply
   * written to output.
   */
  public void MarshallSelf(DataOutput out) throws IOException {
    out.writeInt(lines.size());
    Enumeration e = lines.elements();
    while (e.hasMoreElements()) {
      Rectangle r = (Rectangle) e.nextElement();
      out.writeInt(r.x);
      out.writeInt(r.y);
      out.writeInt(r.width);
      out.writeInt(r.height);
    }
    e = colors.elements();
    while (e.hasMoreElements()) {
      Color c = (Color) e.nextElement();
      out.writeInt(c.getRGB());
    }
  }
  
  /**
   * Here the data which is written in MarshallSelf is read into the object.
   * After this, the state of this object should be identical to the object
   * which was marshalled.
   */
  public void UnmarshallSelf(DataInput in) throws IOException {
    int numLines = in.readInt();
    lines.ensureCapacity(numLines);
    colors.ensureCapacity(numLines);
    for (int i = 0; i < numLines; ++i) {
      lines.addElement(new Rectangle(in.readInt(),
				     in.readInt(),
				     in.readInt(),
				     in.readInt()));
    }
    for (int i = 0; i < numLines; ++i) {
      colors.addElement(new Color(in.readInt()));
    }
  }
  
}



	
