/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *
 * info@habanero.ncsa.uiuc.edu
 *
 * Copyright 1995-1996, Board of Trustees of the University of Illinois,
 * All Rights Reserved
 *
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 *
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 */
package ncsa.awt.draw;

import java.awt.*;
import java.util.*;


public class PaletteBox extends Panel {

	private RedSpread red = null;
	private GreenSpread green = null;
	private BlueSpread blue = null;
	private Canvas canvas = null;
	private Palette palette = null;

	/**
	 * Constructs a color palette box
	 */
	public PaletteBox() {

		setLayout(new FlowLayout());
		setBackground(Color.white);

		Panel spread = new Panel();
		spread.setLayout(new BorderLayout());
		spread.add("North", red = new RedSpread(this));
		spread.add("Center", green = new GreenSpread(this));
		spread.add("South", blue = new BlueSpread(this));

		canvas = new Canvas();
		canvas.resize(red.getHeight()*3, red.getHeight()*3);

		add(palette = new Palette(this));
		add(canvas);
		add(spread);

		updateColors();
	}

	public void update(Graphics g) {
		paint(g);
	}

	/**
	 * Return the currently selected color
	 * @return the selected Color
	 */
	public Color getSelectedColor() {
		return new Color(red.getValue(),
					green.getValue(),
					blue.getValue());
	}

	public void updateColors() {
		Color newColor = new Color(red.getValue(),
					green.getValue(),
					blue.getValue());

		canvas.setBackground(newColor);
		canvas.repaint();
		palette.setColor(newColor);
		repaint();
	}
		
	public void updateColor(Color color) {
		canvas.setBackground(color);
		canvas.repaint();
		
		red.setValue(color.getRed());
		green.setValue(color.getGreen());
		blue.setValue(color.getBlue());
		repaint();
	}
}

abstract class ColorSpread extends Canvas {
	private int width = 256;
	private int height = 20;
	private int pheight = 4;
	private int colorValue = 0;
	private int newValue = 0;
	private PaletteBox paletteBox = null;

	ColorSpread(PaletteBox paletteBox) {
		super();
		resize(width, height);
		setBackground(Color.white);
		this.paletteBox = paletteBox;
	}

	public void drawMarker(Graphics g) {
		g.drawLine(colorValue-pheight, 0, colorValue+pheight,0);
		g.drawLine(colorValue-pheight, 0, colorValue, pheight);
		g.drawLine(colorValue, pheight,   colorValue+pheight,0);

		g.drawLine(colorValue-pheight, height-1, colorValue, height-pheight-1);
		g.drawLine(colorValue, height-pheight-1, colorValue+pheight, height-1);
		g.drawLine(colorValue-pheight, height-1, colorValue+pheight, height-1);
	}

	public abstract void increment(Graphics g, int i);
		
	public void paint (Graphics g) {
		int index = 0;
		g.setColor(Color.black);
		drawMarker(g);
		while (index < width) {
			increment(g, index);
			g.drawLine(index,pheight+2,index,height-pheight-2);
			index++;
		}
	}

	public void update(Graphics g) {
		g.setColor(this.getBackground());
		drawMarker(g);
		if (newValue > 255)
			colorValue = 255;
		else if (newValue < 0)
			colorValue = 0;
		else
			colorValue = newValue;
		paint(g);
	}

	public final int getHeight() {
		return height;
	}

	public boolean mouseDown(Event e, int x, int y) {
		newValue = x;
		repaint();
		paletteBox.updateColors();

		return true;
	}

	public boolean mouseDrag(Event e, int x, int y) {
		newValue = x;
		repaint();
		paletteBox.updateColors();

		return true;
	}

	public final int getValue() {
		return colorValue;
	}

	public void setValue(int value) {
		newValue = value;
		repaint();
	}
}

class RedSpread extends ColorSpread {

	RedSpread(PaletteBox paletteBox) {
		super(paletteBox);
	}

	public void increment(Graphics g, int i) {
		g.setColor(new Color(i, 0, 0)); 
	}
		
}

class GreenSpread extends ColorSpread {

	GreenSpread(PaletteBox paletteBox) {
		super(paletteBox);
	}

	public void increment(Graphics g, int i) {
		g.setColor(new Color(0, i, 0)); 
	}
		
}

class BlueSpread extends ColorSpread {

	BlueSpread(PaletteBox paletteBox) {
		super(paletteBox);
	}

	public void increment(Graphics g, int i) {
		g.setColor(new Color(0, 0, i)); 
	}
		
}


class Palette extends Canvas {

	int columns = 6;
	int rows = 2;
	int boxsize = 16;
	Color color[][] = new Color[rows][columns];
	Color picked = null;
	PaletteBox paletteBox = null;

	/**
	 * Constructs a color palette
	 */
	Palette(PaletteBox paletteBox) {
		super();

		this.paletteBox = paletteBox;

		color[0][0] = Color.black;
		color[1][0] = Color.white;
		color[0][1] = Color.gray;
		color[1][1] = Color.darkGray;
		color[0][2] = Color.blue;
		color[1][2] = Color.cyan;
		color[0][3] = Color.magenta;
		color[1][3] = Color.orange;
		color[0][4] = Color.pink;
		color[1][4] = Color.red;
		color[0][5] = Color.yellow;
		color[1][5] = Color.green;

		picked = color[0][0];
	}

	public void paint(Graphics g) {
		Color cur = null;
		for (int i = 0; i < rows; i++) {
			for (int j = 0; j < columns; j++) {
				cur = color[i][j];
				g.setColor(color[i][j]);
				g.fillRect(j*boxsize,i*boxsize,boxsize,boxsize);
				g.setColor(Color.black);
				g.drawRect(j*boxsize,i*boxsize,boxsize-1, boxsize-1);
				if (cur == picked) {
					g.drawRect(j*boxsize+1,i*boxsize+1,boxsize-2, boxsize-2);
				}
			}
		}
	}

	public void update(Graphics g) {
		paint(g);
	}

	public Dimension preferredSize() {
		Dimension dim = new Dimension();

		dim.height = boxsize*rows;
		dim.width =  boxsize*columns;

		return dim;
	}
		
	public boolean mouseDown(Event e, int x, int y) {
		int column = (int)x/boxsize;
		int row = (int)y/boxsize;
		if ((column >= columns) || (row >= rows))
			return true;
		picked = color[row][column];
		repaint();
		paletteBox.updateColor(picked);
		return true;
	}

	public final Color getSelectedColor() {
		return picked;
	}

	public void setColor(Color newColor) {
		for (int i = 0; i < rows; i++) {
			for (int j = 0; j < columns; j++) {
				if (picked == color[i][j]) {
					color[i][j] = newColor;
					picked = newColor;
					repaint();
					return;
				}
			}
		}
		return;
	}

	public boolean handleEvent(Event e) {
		return super.handleEvent(e);
	}
}
