/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *
 * info@habanero.ncsa.uiuc.edu
 *
 * Copyright 1995-1996, Board of Trustees of the University of Illinois,
 * All Rights Reserved
 *
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 *
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 */

package ncsa.awt.draw;

import java.awt.*;
import java.util.*;

/**
 * This class produces a pointer that can be used the draw on components
 *
 * @version 0.1, 3/26/96
 * @author	 NCSA
 */
public class Pointer extends DrawingPrimitive {

	private int width = 12;
	private int thickness = 2;
	private int half_width = (width/2);

	/**
	 * Constructs a drawing Pointer
	 */
	public Pointer() {
		super();
	}

	/**
	 * Constructs a drawing Pointer at a specific location
	 * @param x the x coordinate
	 * @param y the y coordinate
	 */
	public Pointer(int x, int y) {
		points = new Vector();
		points.addElement(new Point(x,y)); /* 0 */
		bounding_box = new Rectangle(x-half_width, y-half_width, width, width);
        }

	/**
	 * Resets a drawing Pointer to a specific location
	 * @param x the x coordinate
	 * @param y the y coordinate
	 */
	public void resetPoint(int x, int y) {
		points.setElementAt(new Point(x, y), 0);
		bounding_box = new Rectangle(x-half_width, y-half_width, width, width);
	}
	
	/**
	 * Sets the bounding box values for this Pointer
	 * @param pnts the Vector containing the Pointer value
	 */
	public void setBoundingBox(Vector pnts) {
		Point p0 = (Point)pnts.elementAt(0);
		int x = p0.x;
		int y = p0.y;
		bounding_box = new Rectangle(x-half_width, y-half_width, width, width);
	}
		

	/**
	 * Returns the Point at which the Pointer currently resides
	 * @param x the x coordinate
	 * @param y the y coordinate
	 */
	public final Point getPoint() {
		return (Point)points.elementAt(0);
	}

	/**
	 * Sets the width of the pointer
	 * @param width the width of the pointer in pixels
	 */
	public void setWidth(int width) {
		this.width = width;
		this.half_width = (width/2);
	}

	/**
	 * Sets the thickness of the pointer's outline
	 * @param width the thickness in pixels
	 */
	public void setThickness(int thickness) {
		this.thickness = thickness;
	}

	/**
	 * Performs a mouseDown for a Pointer
	 */
	public void mouseDownAction(int x, int y) {
		points.addElement(new Point(x,y)); /* point 0 */
		bounding_box = new Rectangle(x-half_width, y-half_width, width, width);
	}

	/**
	 * Performs a mouseDrag for a Pointer
	 */
	public void mouseDragAction(int x, int y) {
		Point p0 = (Point)points.elementAt(0);
		points.setElementAt(new Point(x,y),0);

		Rectangle newArea = new Rectangle(x-half_width, y-half_width, width, width);

		Rectangle wholeArea = newArea.union(bounding_box);

		canvas.repaint(wholeArea.x, wholeArea.y, wholeArea.width+1, wholeArea.height+1);
		bounding_box = newArea;
        }

	/**
	 * Performs a mouseUp for a Pointer
	 */
        public void mouseUpAction(int x, int y) {
		mouseDragAction(x, y);
	}

        /**
         * Move this primitive from the current location to a new (x,y)
         * @param start the reference Point at which this move started
         * @param x the x coordinate
         * @param y the y coordinate
         */
	public void move(Point start, int x, int y) {
		mouseDragAction(x,y);
	}

	/**
	 * Draw the Pointer in the given graphics context
	 * @param the graphics context
	 */
        public void drawPoints(Graphics g) {
		Point p0 = (Point)points.elementAt(0);
		int x = p0.x;
		int y = p0.y;
		
		g.fillRect(x-half_width, y-(thickness/2), width, thickness);
		g.fillRect(x-(thickness/2), y-half_width, thickness, width);
        }

	/*
	 * Grabs this primitive
	 * @param x the x coordinate where the primitive was grabbed
	 * @param y the y coordinate where the primitive was grabbed
         * <p>
         * This differs from the super's grab() method in that it
	 * causes the center of the pointer to float to the mouse pointer
	 * on the grab.
	 */
	public void grab(int x, int y) {
		Point p0 = (Point)points.elementAt(0);
		redraw(p0.x-half_width, p0.y-half_width, 
			p0.x+half_width, p0.y+half_width);
		grabbedAt = new Point(x, y);
		points.setElementAt(grabbedAt, 0);
		bounding_box = new Rectangle(x-half_width, y-half_width, width, width);
		grabbed = true;
	}	
}

