/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *
 * info@habanero.ncsa.uiuc.edu
 *
 * Copyright 1995-1996, Board of Trustees of the University of Illinois,
 * All Rights Reserved
 *
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 *
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 */
package ncsa.awt.draw;

import java.awt.*;
import java.util.*;
import java.io.*;
import ncsa.habanero.*;

/**
 * A class that implements a TextInput string DrawingPrimitive 
 *
 * @version     1.0 05/25/96
 * @author      NCSA SDG
 */
public class TextInput extends DrawingPrimitive {

	protected String text = null;

	public TextInput() {
		super();
	}

        /**
         * Draws the TextInput string in the given graphics context
         * @param g the graphics context
         */
	public void drawPoints(Graphics g) {
		Point at = (Point)points.elementAt(0);
		if (text == null)
			return;
		g.drawString(text, at.x, at.y);
		setBoundingBox(g, points);
	}

	public void setBoundingBox(Vector pnts) {
		// Set within the drawPoints() method, because we
		// need a graphics context to compute the area
		// yeech....
	}

	/**
	 * Set the bounding box for this primitive.  The Graphics context
	 * is required to obtain the FontMetrics for this DrawingPrimitive.
	 * @param g the graphics context
	 * @param pnts the vector containing the DrawingPrimitive location
	 */ 
	public void setBoundingBox(Graphics g, Vector pnts) {
		Point at = (Point)points.elementAt(0);
		FontMetrics fm = g.getFontMetrics();
		int height = fm.getHeight();
		int width = fm.stringWidth(text);
		bounding_box = new Rectangle(at.x, at.y-height, width+1, height+1);
	}

	/** 
	 * Initializes the data structures for this primitive on mouse down
	 * @param x the x coordinate
	 * @param y the y coordinate
	 */
	public void mouseDownAction(int x, int y) {
                points.addElement(new Point(x,y)); /* point 0 */
		text = null;
	}

	/** 
	 * Updates the data structures for this primitive during a mouse drag
	 * @param x the x coordinate
	 * @param y the y coordinate
	 */
	public void mouseDragAction(int x, int y) {
		return;	
	}

	/** 
	 * Updates the data structures for this primitive during a mouse up
	 * @param x the x coordinate
	 * @param y the y coordinate
	 */
	public void mouseUpAction(int x, int y) {
		return;
	}

        /**
         * Move this primitive from the current location to a new (x,y)
         * @param start the reference Point at which this move started
         * @param x the x coordinate
         * @param y the y coordinate
         */
        public void move(Point start, int x, int y) {
                if (bounding_box == null)
                        return;

                canvas.repaint();
 
                int adjX = x - start.x;
                int adjY = y - start.y;
 
		Point pnt = (Point)points.elementAt(0);

		pnt.x = pnt.x + adjX;
		pnt.y = pnt.y + adjY;

                start.x = x;
                start.y = y;
 
                bounding_box.x = bounding_box.x + adjX;
                bounding_box.y = bounding_box.y + adjY;
        }

	/**
	 * Add this key stroke to the text string
	 */
        public void keyDownAction(int key) {
                if (key == 10) // enter key
                        enteringText = false;
 
                if (enteringText == false)
                        return;
 
                if ((key == 8) || (key == 127)) { /* bs or del */
                        if (text.length() > 0)
                                text = text.substring(0, text.length()-1);
                } else if (text == null)
			text = String.valueOf((char)key);
		else
                        text = text+(char)key;
                canvas.repaint();
        }

        /**
         * Marshalls this DrawingPrimitive on the given DataOutput
         * @param out the DataOutput
         */
	public void MarshallSelf(DataOutput out) throws IOException {
		super.MarshallSelf(out);
		out.writeUTF(text);
	}

        /**
         * UnMarshalls this DrawingPrimitive on the given DataInput
         * @param out the DataInput
         */
	public void UnmarshallSelf(DataInput in) throws IOException {
		super.UnmarshallSelf(in);
		text = in.readUTF();
	}
}
