/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package ncsa.habanero;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Observable;
import java.io.*;
import ncsa.habanero.granted.ActionGranted;
import ncsa.habanero.granted.Serializer;
import ncsa.habanero.Router;

/**
 * This router sits on a server and sends data to each of its
 * list of participants.
 */
public class CentralRouter extends Router {
  protected Vector connections = new Vector();
  
  /**
   *@param owningSharable
   *@return
   * Construct this object with the object which owns it
   */
  public CentralRouter(Sharable owningSharable) {
    super(owningSharable);
  }
  
  /**
   *@param request the action that all collobjects should perform
   *@return
   * The send_to_everyone method asks the ActionGranted to 
   * marshall itself onto an output stream, which is then
   * copied to each participant.  
   */
  protected void send_to_everyone(ActionGranted request) {
    ByteArrayOutputStream bytes = new ByteArrayOutputStream(1024);
    DataOutputStream data = new DataOutputStream(bytes);
    try {
      request.Marshall(data);
    }
    catch (IOException e) {
      System.out.println("Couldn't marshall ActionGranted");
    }

    ConnectionRecord c = null;
    for (Enumeration clients = participants(); clients.hasMoreElements();) { 
      try {
	c = (ConnectionRecord)clients.nextElement();
	bytes.writeTo((OutputStream)c.out);
      }
      catch (IOException e) {
	connections.removeElement(c);
      }
    } 
  }


  /**
   *@param o the ParticipantManager for the session
   *@param arg the SessionParticipant whose status has changed.
   *@return This method is called by o, which is a ParticipantManager, when arg,
   * which is a SessionParticipant is added to or removed from the Session.
  */
  public void update(Observable o, Object arg) {
    handleConnection((ParticipantManager)o, (TCPSessionParticipant) arg);
  }

  /**
   *@param pm
   *@param s
   *@return
   * Add the given output to my list if it's not already there.
   */
  protected void handleConnection(ParticipantManager pm,
				  TCPSessionParticipant s) {
//      This next line asks the participant to return the output corresponding
//      to my owner - there could be more than one output associated with
//      a given participant
    DataOutput out = s.getOutput(owner.MakeSpecifier(SessionManager.manager()));
    ConnectionRecord rec = new ConnectionRecord(s, out);
    if (connections.contains(rec)) {
      if (!pm.contains(s)) {
//	System.out.println(this + " removing connection " + s);
	connections.removeElement(rec);
     }
    }
    else if (out != null) {
//      System.out.println(this + " adding connection " + s);
      connections.addElement(rec);
    }
   if (out != null) {   
      thawInput();
    }
  }

  /**
   *@return
   * An enumeration of the participants.
   */
  protected Enumeration participants() {
    return connections.elements();
  }

}


class ConnectionRecord extends Object {
  SessionParticipant participant;
  DataOutput out;

  /**
   *@param p the participant
   *@param o the output stream
   *@return creates the connection record
   */
  public ConnectionRecord (SessionParticipant p, DataOutput o) 
    {
      participant = p;
      out = o;
    }

  /**
   *@param o
   *@return true if the connection record is the same
   */
  public boolean equals(Object o) 
    {
      if (o instanceof ConnectionRecord) {
	ConnectionRecord c = (ConnectionRecord) o;
	return ((c.participant != null) && (c.participant.equals(participant))) ||
	  ((c.out != null) && (c.out.equals(out)));
      }
      else
	return false;
    }
}


