/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package ncsa.habanero;
import ncsa.habanero.granted.Arbitrator;
import java.awt.Frame;
import java.awt.Event;
import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.Vector;
import java.util.Enumeration;

/**
 * A Collobject generally wraps a previously invented application, 
 * which exists in a frame.  Since the Collobject is a Sharable, it
 * has its own Arbitrator. Generally, the Collobject creates a frame
 * to display the wrapped object, although if the object is already
 * a frame, the Collobject doesn't bother.  We consider the wrapped
 * object to be a view on the Collobject, and it is possible for there
 * to be more than one (with different display attributes, for example)
 */
public class Collobject extends Sharable {
  protected Vector views = new Vector();
  protected HabaneroFrame frame;
  protected SessionParticipant originator = null;

  /**
   *@param parent
   *@param aClass
   *@return
   * In this case we instantiate the object 
   */
  public Collobject(Sharable parent, Class aClass) {
    super(parent);
    try {
      views.addElement(aClass.newInstance());
    }
    catch (InstantiationException e) {
    }
    catch (IllegalAccessException e) {
    }
  }
  
  /**
   *@param parent
   *@param toWrap
   *@param arb the arbitrator
   *@return
   * In this case the object was instantiated elsewhere.  The constructor
   * takes my supermodel, the wrapped object, and my arbitrator as arguments.
   * If a frame (window) needs to be created, that's called here.
   */
  public Collobject(Sharable parent, Wrapped toWrap, 
		    Arbitrator arb, String origin) {
    super(parent, arb);
    originator = getSession().getParticipant(origin);
    if (toWrap != null) {
      views.addElement(toWrap);
      setFrame();
    }
    getSession().addTool(this);
  }

  /**
   *@return
   * This is the first view set by the constructor.
   */
  protected Wrapped initialView() {
    if ((views != null) && (views.size() > 0))
      return (Wrapped) views.firstElement();
    else
      return null;
  }
  
  /**
   *@return
   * Sets my frame variable by either creating a new frame or using
   * the wrapped object.  If the wrapped object is a frame, it must 
   * implement the HabaneroFrame interface, which allows it to send actions
   * to other applications.
   */
  protected void setFrame() {
    Wrapped iView = initialView();
    if (iView instanceof HabaneroFrame) {
      frame = (HabaneroFrame) iView;
      frame.setCollobject(this);
    }
    else {
      frame = new MirrorFrame(this);
      iView.startInFrame((MirrorFrame)frame);
    }
    if (iView instanceof Initializable && originator() != null)
      if (SessionManager.manager().owner().equals(originator()))
        ((Initializable)iView).queryForParameters(this);
  }
   
  /**
   *@return creates a Collobject which will then be marshalled
   */
  public Collobject() {
    super(null);
  }
  
  /**
   *@return the frame
   */
  public HabaneroFrame frame() {
    return frame;
  }

  /**
   *@return the originator
   */
  public SessionParticipant originator() {
    return originator;
  }

  public void performAction(Action action) {
    super.performAction(action);
    if (frame !=null)
      frame.performedAction(action);
  }

  /**
   *@param event the event
   *@return
   * Deliver an AWT Event to all of the views until one of them accepts it.
   * returns true if it is accepted
   */
  public boolean deliverEvent(Event event) {
    for (Enumeration e = views.elements(); e.hasMoreElements();) {
      if (((Wrapped)e.nextElement()).doEvent(event, event.arg))
	return true;
    }
    return false;
  }

  /**
   *@param out the output stream
   *@return
   * Marshall myself by marshalling my arbitrator and my wrapped object(s).
   * Then call to my superclass
   *@exception IOException is thrown when it is unable to write the object
   */
  public void MarshallSelf(DataOutput out) throws IOException {
    out.writeUTF(originator.unique());
    arbi.Marshall(out);
    out.writeInt(views.size());
    for (Enumeration e = views.elements(); e.hasMoreElements();) {
      Marshalling.marshallSomething((Wrapped)e.nextElement(), out);
    }
    super.MarshallSelf(out);
  }

  /**
   *@param in the input stream
   *@param parent the object's parent
   *@return
   * Unarshall myself by unmarshalling my arbitrator and my wrapped object(s).
   * Then call to my superclass, and finally create my window.  This is called
   * when a participant joins a new session, and creates a duplicate of 
   * an existing Collobject.
   *@exception IOException is thrown if it cannot read the object
   */
  public void UnmarshallSelf(DataInput in, ModelObject parent) 
    throws IOException {
      SetSuperModel(parent);
      String originatorString = in.readUTF();
      arbi = (Arbitrator) Unmarshall(in); 
      arbi.set_owner(this);
      int numViews = in.readInt();
      while (numViews-- > 0)
	views.addElement(Unmarshall(in));
      super.UnmarshallSelf(in);
      setFrame();
      getSession().addTool(this);
      originator = getSession().getParticipant(originatorString);
    }

  /**
   *@return
   * A string representation, for debugging.
   */
  public String toString() {
    if (views == null)
      return "uninitialized Collobject " + hashCode();

    if (views.size() == 0)
      return "Collobject wrapping nothing " + hashCode();
    String result =  "Collobject wrapping " + 
      initialView().getClass().toString();
    if (views.size() > 1)
      result += " and " + (views.size()-1) + " others";
    return result;
  }

  /**
   *@return
   * return the first (and presumably only) view of this object.
   */
  public Wrapped getView() {
      return getView(0);
    }
  
  /**
   *@return
   * return a view of this object.
   */
  public Wrapped getView(int whichOne) {
    return (Wrapped) views.elementAt(whichOne);
  }
  
  /**
   *@param inObject the new view of this object
   *@return
   * Add a view of this object
   */
  public void addView(Wrapped inObject) {
    views.addElement(inObject);
  }
  
  /**
   *@param inObject the old view
   *@return
   * Remove a view of this object
   */
  public void removeView(Wrapped inObject) {
    views.removeElement(inObject);
  }

  /**
   *@return eliminates all the views of the object
   */
  public void shutDown() {
    Shutter s = new Shutter(views.elements());
    s.each();
    SubShutter d = new SubShutter(GetSubModels());
    d.each();
    if (frame != null)
      ((Frame)frame).dispose();
    arbi.stop();
    GetSuperModel().RemoveSubModel(this);
    views.removeAllElements();
    frame = null;
    arbi = null;
    super.shutDown();
  }

  public boolean isInitialized() {
    if (initialView() instanceof Initializable)
      return ((Initializable)initialView()).isInitialized();
    else
      return true;
  }
}


class Shutter extends Foreach {
  /**
   *@param e the enumeration to save
   *@return creates a shutter object to shut down things
   */
  public Shutter(Enumeration e) {
    super(e);
  }  
  
  /**
   *@param o the next object to shutdown
   *@return true if it can be shutdown, and it has been
   */
  public boolean doIt(Object o) {
    try {
      if (o instanceof Shutdownable)
	((Shutdownable)o).shutDown();
      return true;
    }
    catch (Exception e) {
      return false;
    }
  }
}

class SubShutter extends Foreach {
  /**
   *@param e the enumeration e
   *@return creates a SubShutter to shut down targetable objects
   */
  public SubShutter(Enumeration e) {
    super(e);
  }  

  /**
   *@param o the object to shutdown
   *@return shuts down the targetable object
   */
  public boolean doIt(Object o) {
    try {
      if (o instanceof Targetable)
	((Targetable)o).shutDown();
      return true;
    }
    catch (Exception e) {
      return false;
    }
  }
}

