/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package ncsa.habanero;
import java.net.*;
import java.util.*;
import java.io.*;
import ncsa.habanero.granted.*;


/** 
 * This is called a router, although what it really does is listen.  It
 * takes a DataInput (which can be gotten from DistributedArbitrator)
 * and spawns a thread to sit and listen for ActionGranted requests, which
 * are unmarshalled and handed off to the serializer.
 */
class DistributedRouter extends Router implements Runnable {
  DataInput input = null;
  Thread thread = null;
  
  /**
   * Construct this object with its owner and the DataInput
   */
  public DistributedRouter(Sharable owner, DataInput input) {
    super(owner);
    this.input = input;
    start();
  }
  
  /**
   * Start me in a new thread
   */
  public void start () {
    if (thread == null) {
      thread = new Thread(this);
      thread.start();
    }
  }
  
  /**
   * Listen on input and handle it.
   */
  public void run() {
    while (handleInput(input));  /* do this until handleInput returns false */
    if (!stopped) {
      Session s = owner.getSession();
      s.getPacSession().DisplayWarning("The server for " + s.id() + " has unexpectedly failed",
				     "You may continue the Session locally");
      s.makeLocal();
      stopped = true;
    }
  }

  void makeLocal() {
    try {
      ((DeferredIO)input).stop();
    }
    catch (ClassCastException e) {
    }
    stopped = true;
  }

  /**
   * This only receives.  It doesn't have participants.
   */
  protected Enumeration participants() {
    return null;
  }

  /**
   * This only receives.  It doesn't send to anyone.
   */
  protected void send_to_everyone(ActionGranted request) {
  }

  public void update(Observable o, Object arg) {
    if (arg == null) {
      System.out.println("disconnecting router " + this);
      makeLocal();
    }
  }

}

