/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package ncsa.habanero;
import java.util.StringTokenizer;
import java.awt.Event;
import java.awt.Component;
import java.awt.Point;

/**
 * This extends Event to allow it to be initialized from a string. The
 * tokens String is a list of tokens to pass to a StringTokenizer.  The
 * arg instance variable is handled as an object, not as part of the String,
 * allowing it to be marshalled.
 */
class HabaneroEvent extends Event {
  private static String tokens = new String("[],="); 
  public final static int QUIT_APP = 9901;
  public final static int QUIT_SESSION = 9902;
  /**
   * Return the list of tokens
   */
  public static String tokens() {
    return tokens;
  }

  /**
   * Create a new Event object by parsing the String representation of the
   * Event.  Try to create a new instance of the target; if that fails, leave
   * it as a String object.
   */
  public HabaneroEvent(String eventRep, Object arg) {
    super(null, 0, 
	  (arg instanceof ArgWrapper) ? ((ArgWrapper)arg).results() : arg);
    StringTokenizer tk = new StringTokenizer(eventRep, tokens());
    while (tk.hasMoreTokens()) {
      String token = tk.nextToken();
      if (token.equals("id"))
	id = Integer.parseInt(tk.nextToken());
      else if (token.equals("x"))
	x = Integer.parseInt(tk.nextToken());
      else if (token.equals("y"))
	y = Integer.parseInt(tk.nextToken());
      else if (token.equals("key"))
	key = Integer.parseInt(tk.nextToken());
      else if (token.equals("target")) {
	String name = tk.nextToken();
	try {
	  target = Class.forName(name).newInstance();
	}
 	catch (Throwable e) {
	  target = name + tk.nextToken("");
	}
      }
    }
  }
  
  /**
   * Since event coordinates get translated to the Frame's coordinate
   * system, this method translates them back.  This is very inefficient.
   * Really, it is.
   */
  public void toLocal(Component target) {
    while ((target != null) && !(target instanceof java.awt.Frame)) {
      Point loc = target.location();
      translate(-loc.x, -loc.y);
      target = target.getParent();
    }
  }


}
