/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package ncsa.habanero;
import java.awt.Frame;
import java.awt.Event;
import java.util.Hashtable;

/**
 * This implements HabaneroFrame, and can be added to a Component making
 * it able to send events across the network.
 */
public class HabaneroFrameImpl implements HabaneroFrame {
  static protected Integer val = new Integer(0);
  protected Collobject mirror;
  protected Hashtable sendThese = new Hashtable();
  protected Session session = null;  // We need to cache this now
  /** 
   * Construct this with the Collobject for which this is a 
   * visual representation.  As a default, set it to send ACTION_EVENTS
   * only.
   */
  public HabaneroFrameImpl(Collobject mom) {
    mirror = mom;
    addEventCode(Event.ACTION_EVENT);
  }

  /**
   * If the event is one that should be sent, create a new 
   * StandardActionRequest to do so.
   */
  public boolean action(Event evt, Object arg) {
    switch (evt.id) {
    case Event.WINDOW_DESTROY:
    case HabaneroEvent.QUIT_APP:
      new StandardActionRequest(new QuittingAction(), getCollobject());
      return true;
    case HabaneroEvent.QUIT_SESSION:
      new StandardActionRequest(new QuittingAction(), getSession());
      return true;
    default:
      if (shouldSend(evt.id)) {
	new StandardActionRequest(new AppletAction(evt), getCollobject());
	return true;
      }
      else
	return false;
    }
  }

  /**
   * Add the given event id (listed in Event.java) to the list of Event
   * types which should be sent across the network.
   */
  public void addEventCode(int code) {
    sendThese.put(new Integer(code), val);
  }

  /**
   * Remove the given event id (listed in Event.java) from the list of Event
   * types which should be sent across the network.
   */
  public void removeEventCode(int code) {
    sendThese.remove(new Integer(code));
  }

  /**
   * Returns true if the code is in the list
   */
  public boolean shouldSend(int code) {
    return sendThese.containsKey(new Integer(code));
  }

  /**
   * Call to the action method, which sends the Event over the network
   */
  public boolean handleEvent(Event e) {
    return action(e, null);
  }

  /**
   * Set the Collobject for which this window is a visual representation
   */
  public void setCollobject(Collobject c) {
    mirror = c;
  }

  /**
   * Get the Collobject for which this window is a visual representation
   */
  public Collobject getCollobject() {
    return mirror;
  }

  public Session getSession() {
    if (session == null)
      session = getCollobject().getSession();

    return session;
  }

  public void performedAction(Action action) {
    // Annie put your code here
    System.out.println("originator is " + action.originator(getSession()));
  }


}
