/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 */

package ncsa.habanero;
import java.awt.*;
import ncsa.habanero.MirrorFrame;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;

/***
 ***  Lock - a limitation on the Locker
 ***/

abstract public class Lock extends Marshalling {

  //  Internal Variables
  String szName = null;

  /**
   *
   *@param   name   the name of the descriptor
   *@return  Creates a lock
   *
   */
  
  public Lock (String name)
    {
      super();
      szName = name;
    }
  
  /**
   *
   *@return  Creates a lock, should only be used by Marshall
   *
   */
  
  public Lock ()
    {
      super();
    }


  /**
   *
   *@param   another   any type of object
   *@return returns true if the locks are equal
   *
   */
  
  public boolean equals (Object another) {
    if (another instanceof Locker)
      return another.equals(this);
    else
      return super.equals(another);
  }
  
  /**
   *
   *@return  returns the type of lock (the name of the lock)
   *
   */
  
  public String getType ()
    {
      return szName;
    }
  

  /**
   *
   *@param   out  the output stream
   *@return  writes the lock to the output stream
   *
   */
  
  public void MarshallSelf(DataOutput out) throws IOException
    {
      //System.out.println ("DEBUG:  Lock::MarshallSelf");
      
      out.writeUTF (szName);
    }
  
  /**
   *
   *@param   in  the input stream
   *@return  reads a lock from an input stream
   *
   */
  
  public void UnmarshallSelf(DataInput in) throws IOException
    {
      //System.out.println ("DEBUG:  Lock::UnmarshallSelf");
      
      szName = in.readUTF();
    }

  /**
   *
   *This method is executed when you attempt an Action using the key
   *to this lock.
   *
   *@param     act the action to be performed
   *@return    returns the true if the lock allows the action to be performed
   *
   */
  abstract public boolean allows(Action act);

  /**
   *
   *This method is executed when the application attempts an action
   *without a lock.
   *
   *@param   act the action that is being attempted without a lock
   *@return  returns true if the action is not allowed by the lock,
   *          our assumption is that a lock doesn't block anything 
   *          unless you say it does
   *
   */
  
  public boolean blocks (Action act) {
    return false;
  }

  /**
   *
   *This method is executed when an application is requesting a new
   *lock, but does not have the key to this lock.
   *
   *@param   ldRequest  the lock request
   *@return  returns false if the request conflicts with the lock,
   *              this is used when the user does not have the key
   *
   */

  abstract public boolean consistent(Lock lkdRequest);

  /**
   *
   *This method is executed when an application is requesting a new
   *lock, and the application has the key to this lock.
   *
   *@param   ldRequest  the lock request
   *@return  returns false if the request conflicts with the lock,
   *              this is used when the user has the key
   *
   */
  
  public boolean keyWorksWith (Lock ldRequest) {
    return true;
  }

  /**
   *
   *This method returns true if the lock should only be used
   *once and then automatically released, and this method returns
   *false if the application is responsible for releasing
   *the lock.
   *
   *@return  returns true if the lock may only be used once
   *
   */
  
  public boolean releaseAfterUsing() {
    return false;
  }
//  END of Lock
}


