/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 */

package ncsa.habanero;
import java.awt.*;
import ncsa.habanero.granted.ActionRequest;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;

/***
 ***  Locker - a limitation on a collaboration session
 ***/

public class Locker {
  //  Local Variables
  Lock ldItem;		 //  The description of the limit
  protected Key code;	 //  A code number set on creation
  private static KeyGen keyGenerator = new KeyGen ();  //  Creates the keys

    /**
     *
     *@param   desc - the descriptor
     *@param   kInit - the initial code
     *@return  Creates a Locker
     *
     */

    public Locker (Lock desc) {
      super();
      ldItem = desc;
      code = keyGenerator.nextKey();
    }

  /**
   *
   *@return  Creates a Locker - should only be used by Marshall
   *
   */

  public Locker ()
    {
      super();
    }

  /**
   *
   *  allows the user to get the lock out
   *
   *@return the lock associated with this locker
   *
   */
  public Lock getLock () 
    {
      return ldItem;
    }
  
  
  /**
   *
   *@param   ldAnother - a lock descriptor
   *@return  compares the lock descriptor, and returns true if they are the same
   *@return  compares the locks, and returns true if they are the same
   *
   */
  public boolean equals (Object another) {
      if (another instanceof Lock) {
	return (ldItem.equals(another));
        }
      else if (another instanceof Locker) {
	return (code.equals(((Locker)another).code));
        }
      else if (another instanceof Key) {
	return openedBy((Key)another);
        }
      else 
	return false;
    }
  
  
  /**
   *
   *@return  returns true if the lock is still valid
   *
   */
  
  public boolean valid() {
    return code.valid();
  }

  /**
   *
   *  onlyOnce
   *
   *@return  returns true if this lock can only be used once, 
   *				a lock derived from this class has no limit
   *				to the number of times it can be used.
   *
   */
  
  public boolean onlyOnce ()
    {
      return ldItem.releaseAfterUsing();
    }
  
  /**
   *
   *@return  uses the Locker
   *
   */
  
  public boolean use (ActionRequest req) {
    if (code.equals(req.getKey())) {
      if (onlyOnce()) {
	code.invalidate();
	//System.out.println("invalidating myself");
      }
      if (ldItem.allows(req.action()))
	return true;
    }
    return false;
  }

  /**
   *
   *@param   ldRequest - the key descriptor
   *@param   oldKey - the old key which gives us permission to
   *                   get the request
   *@return  returns true if the old Locker and the lock descriptor
   *                would be allowing people do to things that they
   *                weren't supposed to
   *
   */
  
  public boolean conflict(Lock ldRequest, Key oldKey) 
    {
      //System.out.println ("Locker::conflicts");
      if ((oldKey !=  null) && (code.equals (oldKey))) {
	//System.out.println ("Locker::conflicts --- we have this key");
	return !ldItem.keyWorksWith (ldRequest);
      }
      else {
	//System.out.println ("Locker::conflicts --- we have another" + ldItem);
	return !ldItem.consistent (ldRequest);
      }
    }

  /**
   *
   *@param   act the action that is being attempted without a lock
   *@return  returns true if the action is not allowed by the lock,
   *          our assumption is that a lock doesn't block anything 
   *          unless you say it does
   *
   */
  
  public boolean blocks (Action act) {
    return ldItem.blocks (act);
  }

  /**
   *
   *@return  returns the key
   *
   */
  
  public Key getKey ()
    {
      return code;
    }
  
  /**
   *
   *@param   key - a key
   *@return  returns true if the key opens the Locker
   *
   */
  
  public boolean openedBy(Key key) {
    return code.equals(key);
  }

  //  END of Locker
}


