/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package ncsa.habanero;
import java.awt.Frame;
import java.awt.Panel;
import java.awt.Label;
import java.awt.Event;
import java.util.Hashtable;

import ncsa.env.control.PacSession;
import ncsa.env.presentation.UiCollObject;

/**
 * This is a window which knows how to send events over the Internet.  The
 * idea is that events travel up the component hierarchy until they arrive
 * here, and are then checked to see if they should be sent.  You can specify
 * the type of Event to be sent - the default is that only ACTION_EVENTS are
 * sent.  It is Habanero's default implementation of HabaneroFrame, and 
 * delegates the event-sending methods to a HabaneroFrameImpl object.  
 * Habanero creates these internally - as a rule, someone writing a 
 * collaborative application does not have to make any.
 */
public class MirrorFrame extends Frame implements HabaneroFrame {
  protected HabaneroFrameImpl delegate;
  protected UiCollObject UiCo;
  protected boolean disposed = false;

  /**
   * Initialize the object with the window name and the Collobject
   * this object represents.
   */
  public MirrorFrame(String title, Collobject rep) {
    super(title);
    delegate = new HabaneroFrameImpl(rep);

    // if no originator do not write the name of the participant
    // if the session is anonymous no name printed
	

    UiCo = new UiCollObject(((rep.originator()== null) ||
			     (rep.getSession().GetInfoSession().getAnonymous()))
			    ? " " :
			     rep.originator().getName());
    add("South", UiCo);
  }

  /**
   * Initialize the object with the Collobject it represents.
   */
  public MirrorFrame(Collobject rep) {
    this(null, rep);
  }

  /**
   * Delegate this method to the HabaneroFrameImpl.
   */
  public boolean action(Event evt, Object arg) {
    if (disposed)
      return false;

    return delegate.action(evt, arg);
  }

  /**
   * Add an event type which should be sent over the net.
   * Delegate this method to the HabaneroFrameImpl.
   */
  public void addEventCode(int code) {
    delegate.addEventCode(code);
  }

  /**
   * Remove an event type which should be sent over the net.
   * Delegate this method to the HabaneroFrameImpl.
   */
  public void removeEventCode(int code) {
    delegate.removeEventCode(code);
  }

  /**
   * Return true if the event type should be sent over the net.
   * Delegate this method to the HabaneroFrameImpl.
   */
  public boolean shouldSend(int code) {
    return delegate.shouldSend(code);
  }

  /**
   * Delegate this method to the HabaneroFrameImpl.
   */
  public boolean handleEvent(Event e) {
    if (disposed)
      return false;

    if (e.id == Event.KEY_ACTION )
      {
	keyaction( e) ;
	return(true);
      }

    return delegate.handleEvent(e);
  }

  /**
   * Set the Collobject which this Frame visually represents.
   * Delegate this method to the HabaneroFrameImpl.
   */
  public void setCollobject(Collobject c) {
    delegate.setCollobject(c);
  }

  /**
   * Get the Collobject which this Frame visually represents.
   * Delegate this method to the HabaneroFrameImpl.
   */
  public Collobject getCollobject() {
    return delegate.getCollobject();
  }


  public synchronized void show() {
   if (getCollobject().isInitialized())
      super.show();
  }

  public synchronized void dispose() {
    super.dispose();
    disposed = true;
  }

  public void performedAction(Action action) {
    SessionParticipant originator = action.originator(delegate.getSession());
    // if no originator do not write the name of the participant
    // if the session is anonymous no name printed
    if ((originator == null) ||
	(getCollobject().getSession().GetInfoSession().getAnonymous()))
	{
	  UiCo.setName(" ");
	}
    else
	UiCo.setName(originator.getName());
  }

   /* add by annie to handle double click 
     */
    
  public boolean keyaction (Event e) {
    if( e.key ==  Event.HOME) {
      PacSession PacS = getCollobject().getSession().getPacSession();
      if (PacS != null)
	PacS.ShowSC();
    } // kbdaction()

    return(true);
  }

}
