/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package ncsa.habanero;
import java.awt.Component;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;
import java.awt.Event;
import java.awt.Frame;

/**
 * A subclass of ModelObject which 
 * "mirrors" another object.  The mirrored object must implement
 * the Mirrored interface, allowing it to marshall and unmarshall and
 * to have its ModelObjectMirror set.  This is useful when using 
 * model objects with existing class hierarchies.  Note: 
 * existing objects which are the "primary" objects for an application
 * (often these inherit from Applet) should be mirrored by the Collobject
 * object rather than this one.
 */

public class ModelObjectMirror extends Targetable {
  protected Mirrored mirrored;
  
  
  /**
   * Default Constructor.
   */ 
  public ModelObjectMirror() {
    super();
  }
  
  /**
   * Constructs a new ModelObjectMirror with a given 
   * supermodel and the object which it is to mirror.  
   * The supermodel may be null; this would represent a root 
   * object.
   */
  public ModelObjectMirror(ModelObject inSuperModel,
			   Mirrored inRepresentative) {
    super(inSuperModel);
    mirrored = inRepresentative;
  }
  
  /**
   * return the object that this ModelObject mirrors.
   */
  public Mirrored GetRepresentative() {
    return mirrored;
  }
  
  /**
   * Set the object that this ModelObject mirrors.
   */
  public void SetRepresentative(Mirrored inObject) {
    mirrored = inObject;
  }
  
  /**
   * Sends the given event on to the Mirrored object, if there is one.
   */
  public boolean deliverEvent(Event event) {
    if (mirrored != null)
      return mirrored.doEvent(event, event.arg);
    else
      return false;
  }

  /**
   * Marshall this object by marshalling the Mirrored object, then 
   * passing control to the superclass.
   */
  public void MarshallSelf(DataOutput out) throws IOException {
    Marshalling.marshallSomething(mirrored, out);
    super.MarshallSelf(out);
  }

  /**
   * Unmarshall this object by unmarshalling the Mirrored object, setting
   * its mirror to be this object, then passing control to the superclass.
   */
  public void UnmarshallSelf(DataInput in) throws IOException {
    mirrored = (Mirrored) Unmarshall(in); 
    mirrored.setMirror(this);
    super.UnmarshallSelf(in);
  }

  /**
   * This static method finds the ModelObjectMirror whose mirrored
   * object is the given parameter.
   */
  static public ModelObjectMirror MatchTarget(Object awtTarget) {
    return (ModelObjectMirror) mDefaultTopObject.Match(awtTarget, 
						       new TargetMatcher());
  }

  public void shutDown() {
    if (mirrored instanceof Shutdownable)
      ((Shutdownable)mirrored).shutDown();

    if (mirrored instanceof Frame)
      ((Frame)mirrored).dispose();

    mirrored = null;
  }
}

/**
 * This class is a subclass of Matcher whose Matches method returns true
 * if the mirrored object of the second parameter is the first parameter.
 */
class TargetMatcher implements Matcher {
  public boolean Matches(Object awtTarget, Object mdlObject) {
    if (mdlObject instanceof ModelObjectMirror)
      return ((ModelObjectMirror)mdlObject).GetRepresentative().equals(awtTarget);
    else
      return false;
  }
}


