/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package ncsa.habanero;
import ncsa.habanero.granted.Arbitrator;
import java.util.Enumeration;
import java.net.Socket;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;
import java.io.DataOutputStream;
import java.util.Date;
import java.util.Vector;
import java.util.Observable;
import java.util.Observer;

import ncsa.env.control.PacSession;
import ncsa.env.abstraction.InfoSession;
import ncsa.env.abstraction.InfoPart;

class Connections extends Observable {
  public Connections() {
    super();
  }

  public synchronized void notifyObservers(Object arg) {
    setChanged();
    super.notifyObservers(arg);
  }
}
		       
/**
 * The Session class represents a set of Collobjects
 * which are being used by (and thus available to) a
 * set of participants.  In particular, all participants
 * see all of the Collobjects, and the Session is the 
 * ancestor (probably the parent in most cases) of all of
 * the Collobjects.  Since Sessions are Sharables, they 
 * have arbitrators.  They are also required to have a
 * serializer and a router, which are the defaults for all
 * of its descendant Collobjects.
*/
public class Session extends Sharable {
  protected ParticipantManager pm;
  protected String sessionID;
  protected SessionParticipant owner;

  protected PacSession pacS = null;
  protected InfoSession infoS = null;
  protected Connections arbitratorConnections = new Connections();
  protected int recordingInProgress = 0;

  /**
   * First initialize my superclass.  Then create my
   * arbitrator, which is required to be a special 
   * SessionArbitrator, and which wraps the Arbitrator I
   * am passed as an argument.  I also store my id and my
   * owner.
  */
  public Session(String id, Arbitrator inner, 
		 SessionParticipant who, InfoSession i) {
    super(null);
    pm = new ParticipantManager(this, who);
    sessionID = id;
    owner = who;
    arbi = new SessionArbitrator(inner, this);
    infoS = i;

   /* call the env Remote Control
     */
    if (SessionManager.manager().useGraphicalEnvironment()) {
      pacS = new PacSession(this);
    }

  } 

  /**
   * This a default constructor for marshalling
   */
  public Session() {
    super(null);
    pm = new ParticipantManager(this, null);
   /* call the env Remote Control
     */
  } 

  /**
   * I am marshalled when a new Participant wants to join me.  I write
   * out my id, my owner, and my arbitrator.  Then I marshall each 
   * Participant.  Finally, I write out Specifiers for each of my children,
   * so that when I unmarshall I can ask them to marshall themselves one
   * at a time.
   */
  public void MarshallSelf(DataOutput out) throws IOException {
    out.writeUTF(sessionID);
    owner.Marshall(out);
    arbi.Marshall(out);
    if (infoS != null)
      Marshalling.marshallSomething(infoS, out);
    else
      out.writeUTF("null");
    out.writeInt(pm.countParticipants());
    Participator pp = new Participator(pm.participants(), out);
    pp.each();
    writeTools(out);
  }

  /**
   * First I read in my id, my owner, and my arbitrator.  Then I unmarshall
   * my participant list.  Finally, I ask the
   * session owner's SessionManager to marshall each child, and I unmarshall
   * them and add them as my own children.
   */
  public void UnmarshallSelf(DataInput in) throws IOException {
    try {
      sessionID = in.readUTF();
      SessionManager.manager().addSession(this);
      owner = (TCPSessionParticipant) Unmarshall(in); 
      Arbitrator inner = (Arbitrator) Unmarshall(in); 
      arbi = new SessionArbitrator(inner, this);
      Object o = Unmarshall(in);
      if (!"null".equals(o))
	infoS = (InfoSession) o;
      int numParticipants = in.readInt();
      while (numParticipants-- > 0) {
	SessionParticipant nextOne = (SessionParticipant) Unmarshall(in);
	addParticipant(nextOne);
      }
      int numChildren = in.readInt();
      if (numChildren > 0) {
	while (numChildren-- > 0) {
	  DeferredIO connection  = owner().sessionManagerConnection();
	  connection.writeUTF("Dump");
	  Marshallable m = Unmarshall(in);
	  marshallSomething(m, connection);

	  ModelObject child = (ModelObject) Unmarshall(connection, this);
	  connection.stop();
	}
      }
      if (SessionManager.manager().useGraphicalEnvironment())
	pacS = new PacSession(this);

      arbi.router().thawInput();
    }
    catch (IOException e) {
      System.out.println("Session.UnmarshallSelf : Couldn't read something from the stream");
    }
  }
  

  /**
   * Adding a participant requires telling the ParticipantManager
   * to add it.
   */
  public synchronized boolean addParticipant(SessionParticipant client) {
    pm.addParticipant(client);
    if (pacS != null)
      pacS.AddParticipant(client);
    return true;
  }

  /**
   * Get the participant with the given id.
   */
  public synchronized SessionParticipant getParticipant(String id) {
    return pm.get_participant(id);
  }

  /**
   * Removing a participant requires telling the ParticipantManager
   * to remove it.
  */
  public synchronized boolean removeParticipant(SessionParticipant client) {
    pm.remove_participant(client);
    if (pacS != null)
      pacS.RemoveParticipant(client);
    return true;
  }

  /**
   * Return my identifier.
   */
  public String id() {
    return sessionID;
  }

  /**
   * Register a router with the ParticipantManager.  
   */
  public void register(Observer toRegister) {
    pm.addObserver(toRegister);
  }

  /**
   * Register a router with the ParticipantManager.  
   */
  public void unregister(Observer toRegister) {
    pm.deleteObserver(toRegister);
  }

  /**
   * My identifier is my key in my parent's table of children.
   */
  public Object key() {
    return sessionID;
  }
  
  /**
   * Return the participant who owns me.
   */
  public SessionParticipant owner() {
    return owner;
  }

  /**
   * Return an Enumeration representing the participants in this Session.
   */
  public Enumeration participants() {
    return pm.participants();
  }

  /**
   * Ask my arbitrator to write relevant information to the output object.
   */
  public void writeInfo(DataOutput out) {
    arbitrator().writeInfo(out);
  }

  /** 
   * This is called when a participant's arbitrator makes a new connection 
   * to any of my children's arbitrators.  It tries to find the 
   * SessionParticipant object representing that participant, and adds the
   * connection information to it.
   */
  synchronized public boolean newConnection(String participantId,
			       Sharable owner,
			       DataOutput connection) {
    Specifier spec = owner.MakeSpecifier(SessionManager.manager());
    TCPSessionParticipant from = 
      (TCPSessionParticipant)getParticipant(participantId);

    if (from == null) {
      System.out.println("New connection is from an unknown participant " +
				participantId);
//       int ind = participantId.indexOf(':');
//       if (ind >= 0) {
// 	String participantName = participantId.substring(0, ind);
// 	String newTry = connection.getInetAddress().getHostName();
// 	System.out.println("Trying" + newTry);
// 	if (newTry.startsWith(participantName)) {
// 	  from = (TCPSessionParticipant)get_participant(participantName + participantId.substring(ind));
// 	}
//       }
//       if (from == null)
 	return false;
    }

    
    from.addConnection(spec, connection);

    /* HACK - this is a stupid way to determine when we are ready to thaw */
    if (owner == this) {
      ((CentralArbitrator)((DynamicArbitrator)arbi).delegate).thawInput();
    }

    pm.notifyObservers(from);
    return true;
  }
  
  public void stopListening() {
    ((CentralArbitrator)((DynamicArbitrator)arbi).delegate).freezeInput();
  }

  public void shutDown() {
    Enumeration e = GetListTool();
    while (e.hasMoreElements()) {
      Collobject c = (Collobject) e.nextElement();
      c.shutDown();
    }
    GetSuperModel().RemoveSubModel(this);
    arbi.stop();
    pm.clear();
    pm = null;
    if (pacS != null)
      pacS.QuitRCFwCb();
    super.shutDown();    
  }

  public void setOwner(SessionParticipant p) {
    System.out.println("setting owner " + p);
    owner = p;
  }

  synchronized public void RemoveSubModel(ModelObject inObject) {
    if (getPacSession() != null)
      getPacSession().RemoveTool((Collobject)inObject);
    super.RemoveSubModel(inObject);
  }

  synchronized protected void AddSubModel(ModelObject inObject, Object key) {
    super.AddSubModel(inObject, key);
  }
 
  public void addTool(Collobject c) {
    if (getPacSession() != null) {
      getPacSession().AddTool(c);
    }
  }

 // react to PacS calls
  public PacSession getPacSession() {
    return (pacS);
  }

 // return InfoSession
 public InfoSession GetInfoSession () {
   return(infoS);
 }

 // return list of InfoParticipant
  public Enumeration GetListParticipant() {
    return pm.participants();
  }


  /* For Ed  return vector of String representing the collobject 
     => should return a vector of collobject
     */
  public Enumeration GetListTool() {
    return toolList().elements();
  }
  
  protected Vector toolList() {
    Vector tools = new Vector();
    for (Enumeration subM = GetSubModels(); subM.hasMoreElements(); ) {
      Object o = subM.nextElement();
      if (o instanceof Collobject)
	tools.addElement(o);
    }
    return tools;
  }

  protected void writeTools(DataOutput out) throws IOException {
     Vector tools = toolList();
     out.writeInt(tools.size());
     SubMarshaller sm = new SubMarshaller(tools.elements(), this, out);
     sm.each();
   }


  public void changeInfoSession(InfoSession is)
    {
      infoS = is;
      if (pacS != null)
	pacS.ChangeInfoSessionFwCb();
    }
  


 public void leave(boolean quitSession) {
   if (quitSession)
     new StandardActionRequest((SessionManager.manager().owner().makeLeavingAction(this)), 
			       this);
   else
     makeLocal();
   }


  public void stop() {
    new StandardActionRequest(new QuittingAction(), this);
  }

  public boolean isShared() {
    return arbi.isDistributed();
  }

  public boolean invite(SessionParticipant p) {
    if (getParticipant(p.unique()) != null)
      return true;

    DeferredIO connection = p.sessionManagerConnection();
    try {
      connection.writeUTF("Invite");
      connection.writeUTF(infoS.getAddrServer());
      connection.writeInt(Integer.parseInt(infoS.getAddrPort().trim()));
      connection.writeUTF(infoS.getName());
      connection.writeUTF("null");
//      String reply = connection.readUTF();
      return true;
    }
    catch (IOException e) {
      System.out.println("failed to connect properly to " + connection);
      System.out.println("error read: " + e.getMessage());
      return false;
    }
  }

  /**
   * Really only remove other participants but myself
   */
  protected void removeAllParticipants() {
    SessionParticipant me = SessionManager.manager().owner();
    owner = me;
    Remover rm = new Remover(participants(), this, me);
    rm.each();
  }

  protected void makeLocal() {
    if (isShared()) {
      removeAllParticipants();
      pm.notifyObservers(null);
      arbitratorConnections.notifyObservers(this);
    }
  }

  public void addArbitratorConnection(Observer conn) {
    arbitratorConnections.addObserver(conn);
  }

  public void removeArbitratorConnection(Observer conn) {
    arbitratorConnections.deleteObserver(conn);
  }

  public void startRecording() {
    recordingInProgress++;
  }

  public void stopRecording() {
    recordingInProgress--;
  }

  public boolean isRecording() {
    return (recordingInProgress > 0);
  }
}

/** 
 * This subclass of Foreach writes the specifier of each object in the
 * given Enumeration to the output object.
 */
class SubMarshaller extends Foreach {
  DataOutput out;
  Session s;

  public SubMarshaller(Enumeration e, Session s, DataOutput out) {
    super(e);
    this.out = out;
    this.s = s;
  }
  
  public boolean doIt(Object o) {
    ((ModelObject)o).MakeSpecifier(out, SessionManager.manager());
    return true;
  }



}

/** 
 * This subclass of Foreach marshalls each object in the
 * given Enumeration to the output object.
 */
class Participator extends Foreach {
  DataOutput out;

  public Participator(Enumeration e, DataOutput out) {
    super(e);
    this.out = out;
  }
  
  public boolean doIt(Object o) {
    try {
      ((Marshalling)o).Marshall(out);
      return true;
    }
    catch (IOException e) {
      return false;
    }
  }
}

class Remover extends Foreach {
  Session s;
  SessionParticipant me;

  public Remover(Enumeration e, Session session, SessionParticipant p) {
    super(e);
    s = session;
    me = p;
  }

  public boolean doIt(Object o) {
    if (!me.equals(o))
      s.removeParticipant((SessionParticipant)o);
    return true;
  }
}
// class SessionShutter extends Foreach {
//   Session session;

//   SessionShutter(Session s, Enumeration e) {
//     super(e);
//     session = s;
//   }  

//   public boolean doIt(Object o) {
//     try {
//       Collobject c = (Collobject)o;
//       c.shutDown();
//       return true;
//     }
//     catch (Exception e) {
//       return false;
//     }
//   }
// }
