/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package ncsa.habanero;
import java.util.Hashtable;
import java.lang.IllegalArgumentException;
import java.net.*;
import java.io.*;
import ncsa.env.abstraction.Const;
import ncsa.env.abstraction.InfoSession;
import ncsa.env.control.PacSessionManager;
import ncsa.env.presentation.MsgWindow;
import ncsa.habanero.granted.Serializer;
import java.io.File;
import java.applet.AppletContext;
import java.net.URL;

/** 
 *  The SessionManager is intended to be a high-level construct 
 *  that keeps track of a client's Sessions and presents an interface to 
 *  other SessionManagers and to Neighborhoods.  It keeps a list of 
 *  sessions in a dictionary, keyed by the session Identifier.  There is
 *  only one SessionManager per invocation of Habanero.  It also responds
 *  to server requests, like "create a new session" which logically cannot 
 *  be sent to objects lower in the hierarchy and to requests from other
 *  SessionManagers such as "dump a session"
 */
public class SessionManager extends ModelObject implements Runnable {
  private static SessionManager manager;
  protected int listenOn;
  protected SessionParticipant owner;
  protected String id;
  protected String accepting = null;
  protected ServerSocket server = null;
  static protected Hashtable addressCache = new Hashtable();
  protected boolean useEnvironment = true;
  protected boolean autoAccept = false;
  protected String latestHood = null;
  protected static AppletContext context = null;
  protected java.awt.Image clipboard = null;
     
  /**
   *  Private constructor.  Creates my owner by querying the local machine
   *  for its Internet address.  Starts a thread to listen for requests.
   *@param listeningPort where to listen for requests
  */
  private SessionManager(int listeningPort, boolean useEnv) {
    super(null, null);
    listenOn = listeningPort;
    useEnvironment = useEnv;


    try {
      InetAddress myself = InetAddress.getLocalHost();
      System.out.println("I am " + myself);

//       if (useEnv) {
// 	String myHostName = myself.getHostName();
// 	if (myHostName.index('.', 0) < 0) {
// 	  MsgWindow P_Warning = new MsgWindow(null,
// 					      "Warning",
// 					      MsgWindow.CLOSE);

// 	  P_Warning.show("Your computer is reporting its hostname as " + myHostName + " rather than a full name",
// 			      "This is likely to cause you problems.  Please ask your sysadmin to do something about it.");
// 	}
//       }
	  
    try {
       server = new ServerSocket(listenOn);
       System.out.println("SessionManager listening on port " + listenOn);
     } catch (IOException e) {
       System.out.println("SessionManager Couldn't create socket on port "+listenOn);
       try {
	 server = new ServerSocket(0);
       } catch (IOException ee) {
	 System.out.println("SessionManager Couldn't create any socket");     
       }
     }

      if (server != null) {
	owner = new TCPSessionParticipant(myself, server.getLocalPort());
	String fileName = Const.getFile("file.myidf");
       
	try {
	  owner.parsePartFile(fileName);
	}
	catch(Exception e) {
	  System.out.println("could not parse file " + fileName);
	} 
      }
	
    }
    catch (UnknownHostException e) {
      System.out.println("Could not determine hostname: unknown host");
      System.exit(0);
    }
    catch (Exception e) {
      System.out.println("Unknown exception: " + e);
      System.exit(0);
    }

    Thread thread = new Thread(this);
    thread.start();
   }

  /** 
   * Listen until someone makes a connection.  Read the request, call a 
   * a routine to execute it.  Close the connection.
   */
  public void run() {
    while (true) { /* run forever */
      try {
	Socket sock = server.accept();
	System.out.println("SessionManager got a connection");
// 	System.out.println(sock.getInetAddress());
// 	System.out.println(sock.getInetAddress().getHostName());
	// HACK - to cache address to workaround bug in getInetAddress()
	// Now we are really really caching this bugger evry way we can manage.
	try {
	  InetAddress ina = sock.getInetAddress();
	  if (ina == null)
	    System.out.println("Can't even get an InetAddress; might as well give up now");
	  String hashKey = 
	    TCPSessionParticipant.makeKeyFromUnique(ina.toString());
	  System.out.println("caching " + ina + " as: " + hashKey);
	  addressCache.put(hashKey, ina);
	  InetAddress.getByName(sock.getInetAddress().getHostName());
	  System.out.println("SessionManager.run() is caching " + sock.getInetAddress());
	}
	catch (UnknownHostException e) {
	  System.out.println("SessionManager.run() could not determine hostname: " + e.getMessage()); 
	}
	DataInputStream in = new DataInputStream(sock.getInputStream());
	DataOutputStream out = new DataOutputStream(sock.getOutputStream());
	String instruction = in.readUTF();
	System.out.println("SessionManager got the instruction: " + instruction);
	if ("Join Session".equals(instruction)) {
	  if (!joinSessionRequested(in, out)) {
	    out.writeUTF("Join Session Failed");
	  }
	}
	else if ("New Session".equals(instruction)) {
	  if (newSessionRequested(in, out) == null) {
	    out.writeInt(-1);
	  }
	}
	else if ("Invite".equals(instruction)) {
	  invited(in, out);
	}
	else if ("Dump".equals(instruction)) {
	  if (!dumpSharable(in, out)) {
	    out.writeUTF("Dump Failed");
	  }
	}
	accepting = null;
	sock.close();
	System.out.println("SessionManager finshed the instruction: " + instruction);
      } catch (IOException e) {
	System.out.println("Accept failed in SessionManager.run()!");
	/* er, what do we do now? */
      }
    }
  }
  

  /**
   *  Static method called to create a TCP participant
   * annie modif
  */
  public static SessionParticipant makeParticipant(String host, int port)
    {
    return (new TCPSessionParticipant(host, port));
    }
  /**
   *  Static method called to get the one instance of this class.
   *  Throws an exception if there is no instance.
  */
  public static SessionManager manager() throws IllegalArgumentException {
    if (manager == null)
      throw new IllegalArgumentException("No Session Manager exists");
    else
      return manager;
  }

  /**
   *  Static method called to make the one instance of this class.
   *  Throws an exception if there is already an instance.
   *@param listeningPort where to listen for requests
  */
//   public synchronized static boolean makeManager(int listeningPort,
// 						 boolean useEnv) throws IllegalArgumentException {
//       if (manager != null)
// 	throw new IllegalArgumentException("Session Manager already exists");
//       else {
// 	manager =  new SessionManager(listeningPort, useEnv);
// 	return manager.good();
//       }
//     }

  public static boolean makeManager(int listeningPort) throws IllegalArgumentException {
    return makeManager(listeningPort, true, false, null, null);
  }

  public static boolean makeManager(int listeningPort,
				    boolean useEnv) throws IllegalArgumentException
  {
    return makeManager(listeningPort, useEnv, false, null, null);
  }



  public static boolean makeManager(int listeningPort, boolean useEnv, boolean url, String location) throws IllegalArgumentException {
    return makeManager(listeningPort, useEnv, url, location, null );
  }

  public static boolean makeManager(int listeningPort, boolean url, String location) throws IllegalArgumentException {
    return makeManager(listeningPort, true, url, location, null );
  }

  public static boolean makeManager(int listeningPort, boolean url, String location, String lang) throws IllegalArgumentException {
    return makeManager(listeningPort, true, url, location, lang );
  }

  public synchronized static boolean makeManager(int listeningPort, boolean useEnv,boolean url, String location, String lang) throws IllegalArgumentException {
    if (manager != null)
      throw new IllegalArgumentException("Session Manager already exists");
    else {
      // read properties
      Const.initProperties(url, location, lang);
      manager =  new SessionManager(listeningPort, useEnv);
      return manager.good();
    }
  }


  /**
   *  Add the given session to my list.  Will be called when a user creates
   *  a new session as well as when a user joins a remote session.
   *@param toAdd the new session
   *@param owner who has rights to determine access to the session
  */
  public void addSession(Session toAdd) {
    toAdd.SetSuperModel(this);
  }

  /**
   *  Add a client to the given session
   *@param sessionId identifier of the new session
   *@param client who to add
  */
//   public boolean addParticipant(String sessionId, SessionParticipant client) {
//     if (can_add_to_session(sessionId)) {
//       Session aSession = session_from_id(sessionId);
//       aSession.addParticipant(client);
//       return true;
//     }
//     else
//       return false;
//   }

  /**
   *  Remove a client from the given session
   *@param sessionId Specifier of the session
   *@param client who to remove
  */
//   public boolean remove_participant(String sessionId, 
// 				    SessionParticipant client) {
//     if (can_remove_from_session(sessionId)) {
//       Session aSession = session_from_id(sessionId);
//       aSession.removeParticipant(client);
//       return true;
//     }
//     else
//       return false;
//   }

//   /**
//    *  Returns true if I have rights to add participants to the session
//    *@param sessionId Specifier of the session
//   */
//   protected boolean can_add_to_session(String sessionId) {
//     return owner().equals(session_owner(sessionId));
//   }

//   /**
//    *  Returns true if I have rights to remove participants from the session
//    *@param sessionId Specifier of the session
//   */
//   protected boolean can_remove_from_session(String sessionId) {
//     return owner().equals(session_owner(sessionId));
//   }

//   /**
//    *  Returns the owner of the given session
//    *@param sessionId Specifier of the session
//    */
//   protected SessionParticipant session_owner(String sessionId) {
//     return session_from_id(sessionId).owner();
//   }

  /**
   *  Returns a Session object given an Identifier
   *@param sessionId Specifier of the session
   */
  protected Session session_from_id(String sessionId) {
    return (Session) GetSubModelByKey(sessionId);
  }

  /**
   *  Returns the owner of this object.
   */
  public SessionParticipant owner() {
    return owner;
  }

  /**
   * Returns the id of this object, which is the hostname + listening port.
   */
  public String id() {
    return owner().unique();
  }

  /*
   * This overrides the ModelObject method to store Sessions 
   * in a Hashtable with their names as keys.
   */
  protected SubModels subModels() {
    if (mSubModels == null)
      mSubModels = new SubModelsByHashtable();

    return mSubModels;
  }

  /**
   * Return a new Session with the given name and owner.
   */
  protected Session makeSession(String sessionName, 
				SessionParticipant sessionOwner) {
    CentralRouter r = new CentralRouter(null);
    CentralArbitrator darby =  new CentralArbitrator(null, r);
    Session session = new Session(sessionName, darby, sessionOwner, null);
    addSession(session);
    return session;
  }

  /**
   * This is a server-side method which is called when a client requests
   * a new session be created.  It reads the session's name from the input
   * object, and if the name is not already being used, makes a new session
   * of that name with the owner it reads from the input object.
   */
  protected Session newSessionRequested(DataInput in, DataOutput out) {
    try {
      if (!checkVersion(in))
	return null;
      String sessionName = in.readUTF();
      Session session = session_from_id(sessionName);
      if (session == null) {
	SessionParticipant sessionOwner = 
	  (SessionParticipant) Marshalling.Unmarshall(in);
	session = makeSession(sessionName, sessionOwner);
	session.writeInfo(out);
	return session;
      }
      else {
	System.out.println("newSessionRequested: session " + sessionName + " already exists");
      }
    }
    catch (IOException ioe) {
      System.out.println("newSessionRequested:Couldn't read from input stream");
    }    
    return null;
  }

  public boolean invited(DataInput in, DataOutput out) {
    try {
      String serverName = in.readUTF();
      int serverPort = in.readInt();
      String sessionName = in.readUTF();
      String neighborhood = in.readUTF();
      System.out.println("invited to " + sessionName + " on " + serverName + " on port " + serverPort);
      if (acceptAutomatically() || askToAccept(sessionName, serverName))
	if (joinSession(sessionName, serverName, serverPort) != null) {
	  if ((context != null) && !"null".equals(neighborhood)) {
	    try {
	      URL url = new URL(neighborhood);
	      context.showDocument(url);
	    } catch (Exception e) {
	      System.out.println("failed to send URL: " + e);
	    }
	  }
	  return true;
	}
    }
    catch (IOException e) {
      System.out.println("SessionManager.invited: Could not read info");
    }
    return false;
  }

  /**
   * Returns true if the SessionManager should accept invitations without 
   * asking the user.
   */
  public boolean acceptAutomatically() {
    return autoAccept;
  }

  public void setAcceptAutomatically(boolean b) {
    autoAccept = b;
  }

  /**
   * Put up a dialog box asking the user whether to accept an invitation.  
   * Return the result.
   */
  public boolean askToAccept(String sessionName, String serverName) {
//    System.out.println("There should be a dialog box here");
    if ( PacSessionManager.manager() != null)
      return (PacSessionManager.manager().Invit(sessionName, serverName));
    else
      return false;
  }

  protected boolean checkVersion(DataInput in) {
    int majorV = -1;
    int minorV = -1;
    try {
      majorV = in.readInt();
      minorV = in.readInt();
      if (majorV != majorVersionNumber())
	throw new IOException();
      else if (minorV != minorVersionNumber())
	System.out.println("minor versions don't match: Mine is " + minorVersionNumber() + " Client's is " + minorV);
      return true;
    }
    catch (IOException e) {
      System.out.print("major versions don't match: Mine is " + majorVersionNumber() + " Client's is ");
      if (majorV == -1)  
	System.out.println("unknown");
      else 
	System.out.println(majorV);
      return false;
    }
  }

  /**
   * When a client asks to join an existing session, this routine writes
   * back the session's owner object, which is where the client goes to get
   * the session's data.  It also adds the client to the session's
   * list of participants.  This is a server-side method.
   */
  public synchronized boolean joinSessionRequested(DataInput in, DataOutput out) {
    // This needs some work.  The router and arbitrator types should
    // come from some sort of config file.  Also, the permission 
    // stuff does not make any sense, but some sort of participant
    // info should go in, so I am leaving it for now.
    try {
      if (!checkVersion(in))
	return false;
      String sessionName = in.readUTF();
      Session session = session_from_id(sessionName);
      
      if (session == null) {
	System.out.println("joinSessionRequested:No such session as " + sessionName);
	return false;
      }
      /* HACK - we need to handle this in a better way. */
//      session.stopListening();
      session.startRecording();
      SessionParticipant joiner = 
	(SessionParticipant) Marshalling.Unmarshall(in);
      session.addParticipant(joiner);
      session.owner().dumpConnectionInfo(out);
      return true;
    }
    catch (IOException ioe) {
      System.out.println("joinSessionRequested:Couldn't read from input stream");
      return false;
    }
  }

  /**
   * This is called by a client to start a session of the given name on a 
   * given address/port.  It makes a connection to the server, writes out
   * the new session request, and gets back the port of the session arbitrator.
   * Then it creates a new session object as its child.
   */
  public Session initiateSession(InfoSession is) {
    String serverName = is.getAddrServer();
    int sessionPort = 1;
    String realSessionName = is.getName();
    try {
      if (!serverName.equals("local")) {
	Socket establish = new Socket(serverName, 
				      Integer.parseInt(is.getAddrPort().trim()));
	// HACK - to cache address to workaround bug in getInetAddress()
	InetAddress.getByName(serverName);

	// Might as well cache this too.
	InetAddress ina = establish.getInetAddress();
	String hashKey = 
	  TCPSessionParticipant.makeKeyFromUnique(ina.toString());
	System.out.println("caching " + ina + " as: " + hashKey);
	addressCache.put(hashKey, ina);
	serverName = ina.toString();
	if (serverName.indexOf('/') > 0)
	  serverName = serverName.substring(serverName.indexOf('/')+1);

	realSessionName = realSessionName + " at " + 
	  serverName + " on port " + is.getAddrPort().trim();
      
	DataOutputStream out = new DataOutputStream(establish.getOutputStream());
	out.writeUTF("New Session");
	out.writeInt(majorVersionNumber());
	out.writeInt(minorVersionNumber());
	out.writeUTF(realSessionName);
	owner.Marshall(out);
	DataInputStream in = new DataInputStream(establish.getInputStream());
	sessionPort = in.readInt();
      }
      else {
	if (session_from_id(realSessionName) != null)
	  sessionPort = 0;
      }

      if (sessionPort > 0) {
	DistributedArbitrator darby = 
	  new DistributedArbitrator(null, serverName, sessionPort, true);
	Session aSession = new Session(realSessionName, darby, owner(), is);

	addSession(aSession);
	doNotify();
	notifyNeighborhood(is.getName(), is.getAddrServer(), is.getAddrPort());
	return aSession;
      }
      else {
      System.out.println("initiateSession:New session " + is.getName() + "not allowed: " + sessionPort);
      }
    }
    catch (UnknownHostException uhe) {
      System.out.println("initiateSession:Couldn't establish a connection with " + serverName);
    }
    catch (IOException ioe) {
      System.out.println("initiateSession:Couldn't read or write something");
    }
    doNotify();
    return null;
  }

  /**
   * This is called by a client to join a session of the given name on a 
   * given address/port.  It makes a connection to the server, writes out
   * the join session request, and gets back the object referring to the 
   * owner of that session.  It then contacts that owner's SessionManager 
   * and asks it to dump the state of the session.  It unmarshalls and
   * returns the session.
   */
  public Session joinSession(String sessionName, 
			     String serverName, int serverPort) {
    try {
      Socket establish = new Socket(serverName, serverPort);
      // HACK - to cache address to workaround bug in getInetAddress()
      InetAddress.getByName(serverName);

      serverName = establish.getInetAddress().toString();
      if (serverName.indexOf('/') > 0)
	serverName = serverName.substring(serverName.indexOf('/')+1);

      String realSessionName = 
	sessionName + " at " + serverName + " on port " + serverPort;
      if (session_from_id(realSessionName) != null) {
	System.out.println("joinSession: already belong to " + sessionName);
	return null;
      }

      Serializer.startRecording();
      DataOutputStream out = new DataOutputStream(establish.getOutputStream());
      out.writeUTF("Join Session");
      out.writeInt(majorVersionNumber());
      out.writeInt(minorVersionNumber());
      out.writeUTF(realSessionName);
      owner.Marshall(out);
      DataInputStream in = new DataInputStream(establish.getInputStream());
      
      DeferredIO connection = owner().connectFromDump(in);
      if (connection != null) {
	connection.writeUTF("Dump");
	Specifier spec = new Specifier();
	spec.addElement(realSessionName);
	spec.Marshall(connection);
	Session aSession = (Session) Marshalling.Unmarshall(connection);
	connection.stop();
	new StandardActionRequest(new ParticipantAddingAction(owner), aSession);
	return aSession;
      }
    }
    catch (UnknownHostException uhe) {
      System.out.println("joinSession:Couldn't establish a connection with " + serverName);
    }
    catch (IOException ioe) {
      System.out.println("joinSession:Couldn't read or write something");
    }
    return null;
  }

  /**
   * This method is called when a SessionManager is asked to dump the state
   * of either a Session or a Collobject.
   */
  public synchronized boolean dumpSharable(DataInput in, DataOutput out) {
    try {
      Specifier shar = (Specifier) Marshalling.Unmarshall(in);
      Sharable toDump = (Sharable) FindSpecifiedObject(shar);
      if ((toDump == null) && (CountSubModels() == 0)) { 
	// try to wait until session loading is over
	System.out.println("Can't find " + shar);
	doWait();
	toDump = (Sharable) FindSpecifiedObject(shar);
      }
      
      if (toDump != null) {
	toDump.arbitrator().router().freezeInput();
	toDump.Marshall(out);
	toDump.arbitrator().router().thawInput();
	return true;
      }
    }
    catch (IOException e) {
      System.out.println("Couldn't  dump " + e);
    }
    return false;
  }

  public boolean good() {
    return (server != null);
  }

  public boolean useGraphicalEnvironment() {
    return useEnvironment;
  }

  synchronized void doWait() {
    try {
      wait();
    }
    catch (InterruptedException e) {
    }
    System.out.println("Woke up");
  }

   synchronized void doNotify() {
    notifyAll();
  }

  static protected int minorVersionNumber() {
    return 1;
  }

  static protected int majorVersionNumber() {
    return 2;
  }

  public static void setAppletContext(AppletContext ac) {
    context = ac;
  }

  public static AppletContext getAppletContext() {
    return context;
  }

  static public InetAddress findAddress(String key) {
    return (InetAddress) addressCache.get(key);
  }

  public void setLatestNeighborhood(String hood) {
    latestHood = hood;
  }

  public String getLatestNeighborhood() {
    return latestHood;
  }

  protected void notifyNeighborhood(String sessionName, 
				    String server, String portNum) {
    if ((latestHood != null) && (context != null)){
      StringBuffer sb = new StringBuffer(latestHood);
//       sb.append("&Start=Start&name=");
//       sun.hotjava.forms.FormPanel.encodeValue(sb, sessionName);
//       sb.append("&port=");
//       sun.hotjava.forms.FormPanel.encodeValue(sb, ":" + owner().port());
//       sb.append("&server=");
//       sun.hotjava.forms.FormPanel.encodeValue(sb, server + ":" + portNum);
      sb.append("&Start=Start&name=" + sessionName.replace(' ', '+'));
      sb.append("&port=%3a" + owner().port());
      sb.append("&server=%3a" + portNum);
      try {
	URL url = new URL(sb.toString());
	System.out.println("notify: " + url);
	context.showDocument(url);
      } catch (Exception e) {
	System.out.println("notifying the neighborhood: " + e);
	System.out.println(e.getMessage());
	e.printStackTrace();
      }
    }
  }
  public void setClipboard(java.awt.Image clipboard) {
    this.clipboard = clipboard;
  }
 
  public java.awt.Image getClipboard() {
    return clipboard;
  }
}

