/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package ncsa.habanero;

import java.net.*;
import java.util.*;
import java.io.*;
import java.awt.Image;
import ncsa.env.abstraction.Const;
import ncsa.util.CollabImage;
import ncsa.util.ImageHandler;

/**
 *  The SessionParticipant object identifies a particular client within the
 *  collaboration.  
*/
abstract public class SessionParticipant extends Marshalling implements ncsa.env.abstraction.InfoPart {
  private String Name = null ;
  private String Cie= null ;    // Company Information
  private URL Url= null ;
  private String Email= null ;
  private String Phone= null ;
  private String Fax = null ;
  private String City = null;
  private String State = null;
  private String Country = null ;
  private String Status = null;
  private String Interest = null ;
  private String Picture = null ;
  private int X = 10;
  private int Y = 10;
  private Image PictureIm = null;
  private int PictureImWidth = 0;
  private int PictureImHeight = 0;
  private CollabImage PictureCollab = null; 
  /**
   * return a unique identifier for this participant.
   */
  abstract public String unique(); 
  abstract public String hostName(); 
  abstract public String port(); 
  abstract public String key();

  /**
   * Dump info about how to connect to my SessionManager to out.  
   */
  abstract public void dumpConnectionInfo(DataOutput out);

  abstract public DeferredIO connectFromDump(DataInput in) 
    throws IOException, UnknownHostException;

  /**
   * Given a SessionManager's connection info, return an IO Object connected
   * to it.  This routine has nothing to do with this particular object, but
   * it needs to be virtual to accomodate different kinds of connection
   * algorithms.  The assumption is that all SessionParticipants will be of 
   * the same subclass.
   */
  abstract public DeferredIO sessionManagerConnection();
  public int hashCode() {
    return unique().hashCode();
  }

  /**
   *  SessionParticipants are equal if their unique ids are
   *@param obj
  */
  public boolean equals(Object obj) {
    try {
      return unique().equals(((SessionParticipant)obj).unique());
    }
    catch (Exception e) {
      return false;
    }
  }

  public SessionParticipant() {
  }

  abstract public boolean isValidAddress();
  
  public String getName()
    {
      return (Name);
    }

  public String getCie ()
    {
      return(Cie);
    }

  public URL getUrl ()
    {
      return(Url);
    }

 public String getEmail()
   {
     return(Email);
   }

  public String getPhone()
    {
      return(Phone);
    }

  public String getFax()
    {
      return( Fax);
    }

  public String getStatus ()
    {
      return(Status);
    }
  
  public String getInterest ()
    {
      return(Interest);
    }

  public String getCity()
    {
      return(City);
    }

  public String getState()
    {
      return( State);
    }

  public String getCountry()
    {
      return(Country);
    }

  public String getPicture()
    {
      return (Picture);
    }

  public int getX()
    {
      return(X);
    }

  public int getY()
    {
      return(Y);
    }

  public Image  getPictureIm()
    {
      return (PictureIm);
    }

  public int  getPictureImWidth()
    {
      return (PictureImWidth);
    }

  public int  getPictureImHeight()
    {
      return (PictureImHeight);
    }

  public CollabImage  getPictureCollab()
    {
      return (PictureCollab);
    }

  public void setName(String s)
    {
      Name= s;
    }

  public void setCie (String s)
    {
      Cie= s;
    }

  public void setUrl (URL u)
    {
      Url= u;
    }

 public void setEmail(String s)
   {
     Email= s;
   }

  public void setPhone(String s)
    {
      Phone= s;
    }

  public void setFax(String s)
    {
       Fax= s;
    }

  public void setStatus (String s)
    {
      Status= s;
    }
  
  public void setInterest (String s)
    {
      Interest= s;
    }

  public void setCity(String s)
    {
      	City= s;
    }

  public void setState(String s)
    {
       State= s;
    }

  public void setCountry(String s)
    {
      Country= s;
    }

  public void setPicture(String s)
    {
      
      if (s == null)
	{
	  Picture = s;
	  return;
	}
      File file = new File(s);
      if ( ! file.exists())
	  {
	    System.out.println("the participant picture file does not exist");
	    Picture = null;
	    return;
	  }
	  
      Picture= s;
    }

  public void setX(int x)
    {
      X = x;
    }

  public void setY(int y)
    {
      Y = y;
    }
  
  public void setPictureIm(Image im)
  {
//    System.out.println("picture im for " + this + " is: " + im);
    PictureIm = im;
  }

  public void setPictureImWidth (int i)
  {
    PictureImWidth = i;
  }

  public void setPictureImHeight(int i)
  {
    PictureImHeight = i;
  }

  public void MarshallSelf(DataOutput out) throws IOException
    {

    out.writeUTF ( (Name == null ) ? "" : Name );
    out.writeUTF ((Cie == null ) ? "" : Cie );
    if ( Url == null)
      out.writeUTF("");
    else
      out.writeUTF (Url.toString());
    out.writeUTF ((Email == null ) ? "" : Email );
    out.writeUTF ((Phone == null ) ? "" : Phone);
    out.writeUTF ((Fax == null ) ? "" : Fax );
    out.writeUTF ((Status == null ) ? "" : Status );
    out.writeUTF ((Interest == null ) ? "" : Interest );
    out.writeUTF ((City == null ) ? "" : City );
    out.writeUTF ((State == null ) ? "" : State);
    out.writeUTF ((Country == null ) ? "" : Country);

    out.writeUTF(Integer.toString(X));
    out.writeUTF(Integer.toString(Y));
    // marshall the picture
    if (PictureIm != null) {
      ImageHandler ih = new ImageHandler(PictureIm, 
					 PictureImWidth,
					 PictureImHeight);
      ih.readPixels();
      
      CollabImage ci = new CollabImage(ih);
      Marshalling.marshallSomething(ci, out) ;
    }
    else {
//	  System.out.println("Error in marshalling the participant Image for " + this);	  
      if (PictureCollab != null) {
//	    System.out.println("Marshalling PictureCollab instead");
	Marshalling.marshallSomething(PictureCollab, out) ;
      }
      else {
//	    System.out.println("Making a new CollabImage instead" +
//			       PictureImWidth + "  " +
//			      PictureImHeight );
	    
	out.writeUTF("null");
      }
    }
  }


  public void UnmarshallSelf(DataInput in) throws IOException {
    Name = in.readUTF();
    Cie =in.readUTF();
    String str =  in.readUTF();
    try {
      Url  = new URL (str) ;
    }
    catch ( MalformedURLException e)
      {
	Url = null;
      }
    
    Email = in.readUTF();
    Phone = in.readUTF();
    Fax = in.readUTF();
    Status = in.readUTF();
    Interest= in.readUTF();
    City = in.readUTF();
    State = in.readUTF();
    Country = in.readUTF();
    X = Integer.parseInt(in.readUTF());
    Y = Integer.parseInt(in.readUTF());

    // unmarshall the image
    PictureCollab =  (CollabImage) Marshalling.Unmarshall(in);
    if ((PictureCollab == null) && 
	SessionManager.manager().useGraphicalEnvironment()) {
      CollabImage ci = new CollabImage(PictureImWidth, PictureImHeight);
    }
  }


  /* Function that reads the info of a participant through a file
     each property is in a new line
     */
   
  public void parsePartFile (String s) throws IOException
    {
      String line;
      FileInputStream in = null;
      DataInputStream dataIn = null;
      
      URLConnection connection;
      URL url ;

      if (Const.ACCESS_HABANERO_DIR == Const.LOCAL)
	{
	  try
	    {
	      in = new FileInputStream(s);
	      dataIn = new DataInputStream(in);
	    }
	  catch(Throwable e)
	    {
	      System.out.println("Error in opening file " + s);
	      return;
	    }
	}
      else
	{ // url
	  try
	    {
	      url = new URL(s);
	      connection = url.openConnection();
	      dataIn = new DataInputStream(connection.getInputStream());
	    }
	  catch (MalformedURLException e) 
	    {
	      System.out.println("Error in opening url file " + s);
	      return;
	    }
	}

      try
	{
	  parsePartContent(dataIn, in );
	}
      catch(IOException e) 
	{
	  throw e;
	}
    }

  public void parsePartContent(DataInputStream dataIn,  FileInputStream in ) throws IOException
  {
    String line;
    try
	{
	  if ((line = dataIn.readLine()) != null)
	    Name= line;
	  
	  if ((line = dataIn.readLine()) != null)
	    Cie= line;
	  
	  if ((line = dataIn.readLine()) != null)
	    {
	      try {
		Url  = new URL (line) ;
	      }
	      catch ( MalformedURLException e)
		{
		  Url = null;
		}
	    }
	  
	  if ((line = dataIn.readLine()) != null)
	    Email= line;
	  
	  if ((line = dataIn.readLine()) != null)
	    Phone= line;
	  
	  if ((line = dataIn.readLine()) != null)
	    Fax= line;
	  
	  if ((line = dataIn.readLine()) != null)
	    Status = line;
	  
	  if ((line = dataIn.readLine()) != null)
	    Interest = line;
	  
	  if ((line = dataIn.readLine()) != null)
	    City = line;
    
	  if ((line = dataIn.readLine()) != null)
	    State = line;
	  
	  if ((line = dataIn.readLine()) != null)
	    Country = line;
	  
	  if ((line = dataIn.readLine()) != null)
	    Picture= line;
	  
	  if ((line = dataIn.readLine()) != null)
	    X = Integer.parseInt(line);
	  
	  if ((line = dataIn.readLine()) != null)
	    Y = Integer.parseInt(line);
	  if (in != null)
	    in.close();
	}
    catch(IOException e) 
      {
	if (in != null)
	  in.close();
	if ( Name == null)
	  {
	    System.out.println("Error in reading file ");
	    throw e;
	  }
	
      }
  }
  
  /* Function that writes the info of a participant through a file
     each property is in a new line
     if url file => impossible
     */
  public void writePartFile (String s) throws IOException
    {
//       File f = null ;
//       try 
// 	{
// 	  f = new File (s);
// 	}
//       catch (NullPointerException ne)
// 	{
// 	  System.out.println(" the path is  null " + s);
// 	}

//       if ( !f.exists())
// 	{
// 	  System.out.println (" file does not exist ");
// 	}


      if (Const.ACCESS_HABANERO_DIR == Const.URL)
	return;

      FileOutputStream out = null;
      PrintStream ps = null;


      try
	{
	  out = new FileOutputStream(s);
	  ps = new PrintStream(out);
	  
	}
      catch (Throwable e)
	{
	  System.out.println("File does not exist => create it " + s);
	  
	}


      try
	{
	  writePartContent(ps);

	  out.close();
	}
      catch(IOException e) 
	{
	  System.out.println("Error in writing to file " + s );
	  throw e; 
	}


      
    }


  public void writePartContent (PrintStream ps )
    {
  	  ps.println ( Name == null ? "" : Name);
	  ps.println (Cie == null ? "" : Cie );
	  if ( Url == null)
	    ps.println ("");
	  else 
	    ps.println ( Url.toString());
	  ps.println ( Email== null ? "" : Email);
	  ps.println ( Phone== null ? "" : Phone);
	  ps.println ( Fax== null ? "" : Fax);
	  ps.println ( Status== null ? "" : Status);
	  ps.println ( Interest== null ? "" : Interest);
	  ps.println ( City== null ? "" : City);
	  ps.println ( State== null ? "" : State);
	  ps.println ( Country== null ? "" : Country);
	  ps.println ( Picture== null ? "" : Picture);
	  ps.println( X);
	  ps.println(Y);
	}
  /* debug fonction
   */
   public void Print ()
     {
       System.out.println ("----" + Name
			   + "\n----" + Cie
			   + "\n----" + Url.toString() 
			   + "\n----" + Email
			   + "\n----" + City + "," + State + " " + Country
			   + "\n----" + Phone
			   + "\n----" + Fax
			   + "\n----" + Status
			   + "\n----" + Interest
			   + "\n----" + Picture
			   + "\n----" + X
			   + "\n----" + Y
			   );
     }

  public Action makeLeavingAction(Session s) {
    if (equals(s.owner())) {
      SessionParticipant newOwner = this;
      for (Enumeration e = s.participants(); e.hasMoreElements();) {
	newOwner = (SessionParticipant) e.nextElement();
	if (!equals(newOwner)) 
	  break;
      }
      if (equals(newOwner)) 
	return new QuittingAction();
      else
	return new ParticipantLeavingAction(this, newOwner);
    }
    else
      return new ParticipantLeavingAction(this);
 }

}

