/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package ncsa.habanero;
import ncsa.habanero.granted.Arbitrator;
import java.io.*;
/**
 * Basically, the Sharable object represents things that
 * Habanero can share among paricipants.  At the moment,
 * that is Collobjects and Sessions.  Sharables are ModelObjects,
 * so every one knows its parent and its children.
 * Each Sharable also has an arbitrator. 
*/
abstract public class Sharable extends Targetable {
  protected Arbitrator arbi = null;

  /** 
   * Construct a Sharable with no Arbitrator.  Presumably it
   * will be set by a subclass.
  */
  protected Sharable(ModelObject parent) {
    super(parent);
  }
  
 /** 
   * Construct a Sharable with an Arbitrator.  Set the Arbitrator's
   * oowner to be me.
  */
  public Sharable(ModelObject parent, Arbitrator arb) {
    this(parent);
    arbi = arb;
    arbi.set_owner(this);
  }

  /**
   * Add a child Sharable.  
   */
  protected void add_child(Sharable collaborator) {
    collaborator.SetSuperModel(this);
  }
  
  /**
   * Remove a child Sharable.  
   */
  protected void remove_child(Sharable collaborator) {
    collaborator.SetSuperModel(null);
  }
  
  /**
   * Make a new child of the type represented
   * by the Class argument
   *@param aClass class object representing the type of the new child
   */
//   protected Sharable instantiate(Class aClass) {
//     Sharable child = new Collobject(this, aClass);
//     return child;
//   }
  
  /**
   * Make a new child of the type represented
   * by the String argument
   *@param aClass string object representing the type of the new child
   */
  protected synchronized void instantiate(String className,
					  Arbitrator arb,
					  String originator) {
    try {
      Class theClass = Class.forName(className);
      Wrapped newObject = (Wrapped)theClass.newInstance();
      new Collobject(this, newObject, arb, originator);
    } 
    catch (ClassCastException e) {
      System.out.println("ClassCastException in Sharable.instantiate ");
      System.out.print("Tried to cast a ");
      System.out.println(className);
    }
    catch (ClassNotFoundException e) {
      System.out.println("ClassNotFoundException in Sharable.instantiate ");
      System.out.print("Tried to find a ");
      System.out.println(className);
    }
    catch (NoSuchMethodError e) {
      System.out.println("NoSuchMethod in Sharable.instantiate ");
      System.out.print("Failed to create a new instance of ");
      System.out.println(className);
    }
    catch(InstantiationException e) {
      System.out.println("InstantiationException in Sharable.instantiate ");
    }
    catch(IllegalAccessException e) {
      System.out.println("IllegalAccessException in Sharable.instantiate ");
    }
  }
  
  /**
   * Return the arbitrator for this object 
   */	
  public Arbitrator arbitrator() {
    return arbi;
  }

  public void shutDown() {
    arbi = null;
    super.shutDown();
  }

}
  
