/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package ncsa.habanero;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.net.Socket;
import java.util.Hashtable;
import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.Enumeration;
import ncsa.habanero.granted.Arbitrator;

/**
 *  The TCPSessionParticipant object identifies a particular client within the
 *  collaboration when the collaboration is based on TCP, ie there is an IP
 *  address and a port number.
*/
public class TCPSessionParticipant extends SessionParticipant {
  int smgr_port;
  InetAddress address = null;
  Socket mgrSocket = null;
  Hashtable ports = new Hashtable();
  String key = null;
  String hostString = null;
  boolean attempted = false;

  /**
   * To Marshall myself, I write out the port number of my SessionManager and
   * my IP address.  Together, these identify me uniquely.
   */
  public void MarshallSelf(DataOutput outStream) throws IOException {
    super.MarshallSelf(outStream);
    outStream.writeInt(smgr_port);
    if (address == null)
      outStream.writeUTF(hostString);
    else
      outStream.writeUTF(hostName());
  }

  public String uncheckedHostname() {
    return hostString;
  }

  protected InetAddress addressFromString(String choices) {
    StringTokenizer tok = new StringTokenizer(choices, "/");
    while (tok.hasMoreTokens()) {
      try {
	return InetAddress.getByName(tok.nextToken());
      }
      catch (UnknownHostException e) {
      }
    }
    return SessionManager.findAddress(makeKeyFromUnique(choices));
  }
    
  /**
   * Read back my SessionManager's port number and my IP address.
   */
  public void UnmarshallSelf(DataInput inStream) throws IOException {
    super.UnmarshallSelf(inStream);
    try {
      smgr_port = inStream.readInt();
      hostString = inStream.readUTF();
    }
    catch (IOException e) {
      System.out.println("TCPSessionParticipant.UnmarshallSelf : Couldn't read something from the stream");
    }
  } 

  /**
   * The default constructor, for marshalling.
   */
  public TCPSessionParticipant() {
  }

  /**
   * The constructor takes an IP address (as a string)
   * and constructs an InetAddress.  It also takes the port number of my
   * SessionManager to use as a unique identifier.
   *@param hostname
   *@param port
  */
  public TCPSessionParticipant(String hostname, int port) {
    smgr_port = port;
    hostString = hostname;
  }

  /**
   * The constructor takes an InetAddress.  It also takes the port number 
   * of my SessionManager to use as a unique identifier.
   *@param addr
   *@param port
  */
  public TCPSessionParticipant(InetAddress addr, int port) {
    smgr_port = port;
    address = addr;
    hostString = addr.toString();
  }

  /**
   * A single TCPSessionParticipant could have different ports
   * for each collobject.  This adds a port number to my list.
  */
  synchronized public void addConnection(Specifier collobjId, DataOutput connection) {
    ports.put(collobjId, connection);
  }
  
  synchronized public void lostConnection(Specifier collobjId, 
					  Arbitrator arbi) {
    ports.remove(collobjId);
    System.out.println("lost connection to " + this);
    // Check that participant has not already been removed from session
    if (ports.isEmpty() && 
	(arbi.session().getParticipant(unique()) != null)) {
      System.out.println("no more connections to " + this);
      Action a;
      Session s = arbi.session();
      new StandardActionRequest(makeLeavingAction(s), s);
    }
  }


  /**
   *  Given a Collobject identifier, return an Output
   *@param collobjId identifies the collobject for which we find the port
  */
  public DataOutput getOutput(Specifier collobjId) {
//    System.out.println("getting output port for " + collobjId);
    return (DataOutput) ports.get(collobjId);
  }

  /**
   *  Given a Collobject identifier, return an Output
   *@param collobjId identifies the collobject for which we find the port
  */
//   public DataOutput getOutput() {
//      return getOutput(new Specifier());
//   }

  /**
   * Returns a string uniquely identifying me by appending my SessionManager's
   * port number to my IP address.
   */
  public String unique() {
    return hostName() + ":" + smgr_port;
  }

  /**
   * Returns my unique identifier as my printable String.
   */
  public String toString() {
    return unique();
  }

  /**
   * Just returns my IP address as a String.
   */
  public String hostName() {
    if (address() != null) {
      return address().toString();
    }
    else {    
      return "unknown IP address ";
    }
  }


  /**
   * Just returns my IP address as a String.
   */
  public String port() {
    return Integer.toString(smgr_port);
  }

  /**
   * Return an output object connected to my SessionManager.  Open a socket
   * if necessary.
   */
  public DeferredIO sessionManagerConnection() {
      return new DeferredIO(address().toString(), smgr_port, false);
  }

  /**
   * Dump info about how to connect to my SessionManager to out.  
   */
  public void dumpConnectionInfo(DataOutput out) {
    System.out.println("dumping " + hostString + ":" + smgr_port);
    try {
      if (address == null)
	out.writeUTF(hostString);
      else
	out.writeUTF(hostName());
      out.writeInt(smgr_port);
    } catch (IOException e) {
      System.out.println("Could not dumpConnectionInfo to " + address);
    }
  }
 
  /**
   * Given a SessionManager's connection info, return an IO Object connected
   * to it.  This routine has nothing to do with this particular object, but
   * it needs to be virtual to accomodate different kinds of connection
   * algorithms.  The assumption is that all SessionParticipants will be of 
   * the same subclass.
   */
  public DeferredIO connectFromDump(DataInput in) {
    try {
      return new DeferredIO(in.readUTF(), in.readInt());
    }
    catch (IOException e) {
      System.out.println("Could not connectFromDump");
      return null;
    }  
  }

  public boolean isValidAddress() {
    return address() != null;
  }

  public String key() {
    if (key == null) {
      key = makeKeyFromUnique(unique());
    }
    return key;
  }

  static public String makeKeyFromUnique(String unique) {
    int slashIndex = unique.indexOf('/', 0);
    if (slashIndex < 0)
      return unique;
    else
      return unique.substring(slashIndex + 1);
  }
    
  protected InetAddress address() {
    if ((address == null) && !attempted) {
      attempted = true;
      address = addressFromString(hostString);
    }
    return address;
  }
}
