/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */

package ncsa.habanero;

import java.awt.*;
import ncsa.habanero.*;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;

/**
 *   <B>TurnLock</B> - a limitation on the types of action that can be performed
 *
 *@version 1.0  Created on April 24, 1996
 *
 */

public class TurnLock extends ncsa.habanero.Lock {

  /**
   *  The player's name or code
   */
  private String szName;

  /**
   *  The player's ID number
   */
  private int nID;

  /**
   *
   *  requests a turn with the participant's name
   *
   *@param szInitName  the name or identifier of the player
   *@return a lock to request a turn
   *
   */
  
  public TurnLock (String szInitName)
    {
      super("TurnLock");
      //System.out.println ("TurnLock::TurnLock");

      szName = szInitName;
      nID = PlayerLock.nUnknownID;
    }
      

  /**
   *
   *  requests a turn with the participant's ID number
   *
   *@param nNewID the person's ID number
   *@return a lock to request a turn
   *
   */
  
  public TurnLock (int nNewID)
    {
      super("TurnLock");
      //System.out.println ("TurnLock::TurnLock");

      szName = "Unknown";
      nID = nNewID;
    }


  /**
   * 
   *  returns the turn's ID number
   *
   *@return     returns the turn's ID number
   *
   */
  public int getID ()
    {
      //System.out.println ("TurnLock::getID");
      return nID;
    }


  /**
   * 
   *  returns the turn's name
   *
   *@return     returns the turn's ID name
   *
   */
  public String getName ()
    {
      //System.out.println ("TurnLock::getName");
      return szName;
    }
  
    
  /**
   *
   *@return a player lock, one should be used for each participant in a game
   *
   */
  
  public TurnLock ()
    {
      this ("Unknown");
    }
  
    
  /**
   *
   *  This method is called when the TurnLock is sent to the arbitrator.
   *
   *@param     out     the output stream
   *@return    writes the player lock to the output stream
   *@exception IOException generates an error if it can't write to the stream
   *
   */
  
  public void MarshallSelf(DataOutput out) throws IOException
    {
      //System.out.println ("TurnLock::MarshallSelf");
      super.MarshallSelf(out);

      out.writeUTF (szName);
      out.writeInt (nID);
    }

  
  /**
   *
   *  This method is called when the TurnLock is received by the arbitrator.
   *
   *@param      in     the input stream
   *@return     reads an Player lock from an input stream
   *@exception  IOException generates an error if unable to read the Player lock
   *
   */
  
  public void UnmarshallSelf(DataInput in) throws IOException
    {
      //System.out.println ("TurnLock::UnmarshallSelf");
      super.UnmarshallSelf(in);

      szName = in.readUTF();
      nID = in.readInt ();
    }


  /**
   * 
   *  This method is used when an action is being attempted.  It
   *  checks to see if the action is allowed.
   *
   *@param      act     the action that is being attempted
   *@return     always returns true, since this allows any action in turn
   *
   */
  public boolean allows(Action act) 
    {
      //System.out.println ("TurnLock::allows");
      return true;
    }
      

  /**
   *
   *@param   act the action that is being attempted without a lock
   *@return  returns true if the action is not allowed by the lock,
   *          we'll be nice and not allow actions while someone is
   *          taking their turn
   *
   */
  
  public boolean blocks (Action act) {
    return true;
  }

  /** 
   * 
   *  This routine is used to determine if this lock can coexist with this
   *  older lock.
   *
   *@param      lkdRequest     the other lock that is attempted
   *@return     true if no other turn locks exist
   *
   */
  public boolean consistent(Lock lkdRequest) 
    {
      //System.out.println ("TurnLock::consistent");

      //  See if someone is trying to move something
      try {
	TurnLock mlRequest = (TurnLock) lkdRequest;
	//System.out.println ("TurnLock::consistent  cast successfully");
	  
	//  Don't allow any other moves at the same time
	return false;
      }
      catch (ClassCastException e) {
	//System.out.println ("TurnLock::consistent  Not a turn lock subclass");
	return true;  // anything we don't know about is OK
      }
    }


  /** 
   * 
   *  The user has the key to the current lock, is the lock still legal.
   *  
   *@param     ldRequest  the request that the user is making
   *@return    always returns true, since if you have the key, 
   *           this lock doesn't interfere
   *
   */  
  public boolean keyWorksWith (Lock ldRequest) 
    {
      //System.out.println ("TurnLock::keyWorksWith");

      return true;
    }


  /**
   *
   *  Should this lock be released after it has been used only once
   *
   *@return     this lock is used to take a turn, only one turn
   *
   */
  public boolean releaseAfterUsing() 
    {
      return true;
    }


  /** 
   *
   *  Used to compare locks
   *
   *@param     another   the other lock
   *@return    true if they have the same ID, or if one is missing an ID, then
   *           the same name
   *
   */
  public boolean equals(Object another) 
    {
      //System.out.println ("TurnLock::equals");
      
      if (another instanceof Locker)
	return another.equals(this);
      else if (another instanceof TurnLock) {
	TurnLock plAnother = (TurnLock) another;
	  
	if ((plAnother.nID == PlayerLock.nUnknownID) || (nID == PlayerLock.nUnknownID)) {
	  //  Check the names
	  //System.out.println ("TurnLock::equals - Unknown ID, checking names");
	  return ((plAnother.szName).equals (szName));
	}
	else {
	  //  Otherwise check the ID
	  //System.out.println ("TurnLock::equals - Known IDs");
	  return (nID == plAnother.nID);
	}
      }
      else
	return super.equals(another);
    }
  
  //  END of TurnLock
}


