/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */

package ncsa.habanero;
import ncsa.habanero.*;
import ncsa.habanero.granted.*;
import ncsa.habanero.tickets.*;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;
import java.util.Vector;
import java.util.Enumeration;
import java.awt.Event;

/**
 * The TurnTakerArbitrator is when only one person may move at a time
 * and each person must take turns
 *
 *@version Alpha2  April 26, 1996
 */
public class TurnTakerArbitrator extends LimitArbitrator {
  
  //  Local Variables
  int nCurrentTurn = 1;             //  The person's whose turn it is
  int nCurrentPlayers = 0;          //  The last player
  
  /**
   * 
   *@param parent the parent arbitrator
   *@param r the router
   *@return a new arbitrator
   *
   */
  public TurnTakerArbitrator(Arbitrator parent, Router r) 
    {
      super(parent, r);
      //System.out.println ("TurnTakerArbitrator::TurnTakerArbitrator");
    }
  
  /**
   * 
   *This method is used to marshall the arbitrator
   *
   *@return a new arbitrator
   *
   */
  public TurnTakerArbitrator() 
    {
      super();
      //System.out.println ("TurnTakerArbitrator::TurnTakerArbitrator");
    }
  
  /**
   * The arbitrate routine simply passes the problem of arbitrating the
   * event on to the limit arbitrator.
   *
   *@param request the ActionRequest we want to grant
   *@return true if the action was granted and false if not
   *
   */
  public synchronized boolean arbitrate(ActionRequest request) 
    {
      //System.out.println ("TurnTakerArbitrator::arbitrate");
      boolean bResult = super.arbitrate(request);

      //  Next turn
      if (bResult) {
	nCurrentTurn++;
	if (nCurrentTurn > nCurrentPlayers) {
	  nCurrentTurn = 1;
	}
      }

      return bResult;
    }

  /**
   * 
   * returns a lock if it is OK to do so, it limits each participants to requesting
   * keys allowing actions during their turn.
   *
   *@param lpdRequest    what you would like to do
   *@param oldKey        the key that permits us to get the lock
   *@return checks to see if you are allowed to do that, and 
   *            returns a key if you can, and Key.NoKey if you can't
   *
   */
  public synchronized Key getLock(Lock ldRequest, Key oldKey) 
    {
      //System.out.println ("TurnTakerArbitrator::getLock");

      if (ldRequest instanceof TurnLock) {
	//System.out.println ("TurnTakerArbitrator::getLock  --  Requesting a TurnLock");

	PlayerLock plPlayer;

	TurnLock tlRequest = (TurnLock) ldRequest;

	//  Get the player lock
	try {
	  Locker locker1 = (Locker) 
	    validLockSet.elementAt (validLockSet.indexOf (oldKey));
	  plPlayer = (PlayerLock) locker1.getLock ();
	}
	catch (ClassCastException e) {
	  //  The bum tried to take a move without the right type of
	  //  key!!!!
	  return Key.NoKey;
	}

	//  Check that the player lock or the turn lock match the current turn
	if ((plPlayer.getID() == nCurrentTurn) || (tlRequest.getID() == nCurrentTurn)) {
	  //  First ask for the key
	  Key kyResult = super.getLock (ldRequest, oldKey);

	  //  No Key, so nothing to do
	  if (kyResult.equals (Key.NoKey)) {
	    return kyResult;
	  }

	  //  Return the Go Ahead Signal
	  return kyResult;
	}
	else {
	  return Key.NoKey;
	}
      }
      else if (ldRequest instanceof PlayerLock) {
	//System.out.println ("TurnTakerArbitrator::getLock  --  Request a PlayerLock");

	PlayerLock plRequest = (PlayerLock) ldRequest;
	
	//  Set the current ID
	if (plRequest.getID() == PlayerLock.nUnknownID) {
	  plRequest.setID (nCurrentPlayers + 1);
	}

	//  First ask for the key
	Key kyResult = super.getLock (ldRequest, oldKey);

	//  No Key, so nothing to do
	if (kyResult.equals (Key.NoKey))
	  return kyResult;

	nCurrentPlayers = Math.max (nCurrentPlayers, plRequest.getID());
	return kyResult;
      }

      //System.out.println ("TurnTakerArbitrator::getLock  --  Requesting a other");
      return super.getLock (ldRequest, oldKey);
    }


  /**
   * 
   *@return the number of active participants in the session
   *
   */
  public int getPlayers ()
    {
      return nCurrentPlayers;
    }


  /**
   * 
   *  Allows a subclass of this arbitrator to make it someone's turn
   *
   *@param nTurn whose turn should it be?
   *
   */
  protected void setTurn (int nTurn)
    {
      if (nTurn > nCurrentPlayers)
	nTurn = 1;
      nCurrentTurn = nTurn;
    }


  /**
   * 
   *  Allows a subclass of this arbitrator to make it someone's turn
   *
   *@return returns the integer whose turn it is
   *
   */
  protected int getTurn ()
    {
      return nCurrentTurn;
    }
}






