/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package ncsa.habanero.granted;
import ncsa.habanero.*;
import ncsa.habanero.tickets.*;
import java.io.*;

/**
 * The ActionGranted is a wrapper class which is created by the Arbitrator
 * after it has determined that an Action is allowable.  It is the class 
 * which is actually routed to the other clients.  It contains the Action
 * to be executed and the Ticket which determined when the Action can be 
 * executed
 */
public class ActionGranted extends Marshalling {
  protected Action action;
  protected Ticket ticket;
  protected Arbitrator arbi;
  /**
   * Create the ActionGranted.  Theoretically, only an Arbitrator can
   * do this.
   * @param req The Action is extracted from this ActionRequest
   */
  protected ActionGranted(ActionRequest req, Ticket tk, Arbitrator arb) {
    super();
    action = req.action();
    ticket = tk;
    arbi = arb;
  }

  /**
   * Create the ActionGranted.  Theoretically, only an Arbitrator can
   * do this.  This constructor is used when a CentralArbitrator is 
   * directly creating new Actions to be propagated and therefore there
   * is no request.
   * @param req The Action is extracted from this ActionRequest
   */
  protected ActionGranted(Action ac, Ticket tk, Arbitrator arb) {
    super();
    action = ac;
    ticket = tk;
    arbi = arb;
  }

  /**
   * The default constructor should only be called if the next call 
   * will be Unmarshall()
   */
  public ActionGranted() {
    super();
  }

  /**
   * The Serializer calls this method to send the action to the target
   */
  public void OK() {
    ticket().act(action);
  }
  
  /**
   * Return the ticket object
   */
  public Ticket ticket() {
    return ticket;
  }
  
  /** 
   * Return the serializer object.
   */
  public Serializer serializer() {
    return arbi.serializer();
  }
  
  /**
   * Unmarshall the Action and the Ticket, then find the Arbitrator from
   * its Specifier.
   */
  public void UnmarshallSelf(DataInput stream) throws java.io.IOException {
    action = (Action) Unmarshall(stream);
    ticket = (Ticket) Unmarshall(stream);
    arbi = (Arbitrator) SessionManager.manager().FindSpecified(stream);
  }
  
  /**
   * Write the Action and the Ticket, then the Arbitrator's Specifier.
   */
  public void MarshallSelf(DataOutput stream) throws java.io.IOException {
    action.Marshall(stream);
    ticket.Marshall(stream);
    arbi.MakeSpecifier(stream, SessionManager.manager());
  }
  
}























































