/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */

package Chat;

import ncsa.habanero.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.net.*;

/**
 * Chat - A simple text conferencing tool.
 */
public class Chat extends Panel implements ncsa.habanero.Wrapped {

	ChatNameField name = null;       /* participant name */
	TextArea chatArea = null;        /* text messages display area */
	ChatInputArea inputArea = null;  /* input field */

	/**
	 *
	 * Chat() - Layout the elements of the text conferencing tool.
	 * 
	 */ 
	public Chat() {
		setLayout(new BorderLayout());
		/*
		 * Create a left justified name input textfield at the
		 * top of the layout
		 */
		name = new ChatNameField("Name", 30);
		name.setParent(this);

		Panel p1 = new Panel();
		p1.setLayout(new BorderLayout());
		p1.add("West", name);

		add("North", p1);

		/*
		 * A place to read text in the middle
		 */
		chatArea = new TextArea(10, 80);
		chatArea.setEditable(false);
		add("Center", chatArea);

		/*
		 * An input area at the bottom.
		 */
		inputArea = new ChatInputArea("Input");
		inputArea.setParent(this);

		add("South", inputArea);
		show();
	}

	/**
	 * startInFrame() -  adds the Chat() object as a subcomponent of
	 * the Frame f.  Implements the montage.Wrapped interface for
	 * startInFrame().
	 */
	public void startInFrame(MirrorFrame f) {
		f.setTitle("Chat");
 
		f.add("Center", this);
		f.pack();
		f.resize(f.preferredSize());
		f.show();
		name.setName(SessionManager.manager().owner().getName());
	}


	/**
	 * let events pass through to Habanero subsystem
	 */
	public boolean handleEvent(Event e) {
		return false;
	}

	/**
	 * Handle events coming back from Habanero.
	 */
	public boolean doEvent(Event e, Object arg) {
		/*
		 * If this is a ChatInputArea event, we received a message
		 * that needs to be displayed.  Display it.
		 */
		if (e.id == Event.ACTION_EVENT) {
			String msg = (String)e.arg;
			if (msg.length() > 0) {
				try {
					chatArea.appendText(msg+"\n");
				} catch (Exception except) {
					System.out.println("Write exception");
				}
			}
			return true;
		}
		return false;
  	}

	/**
	 * MarshallSelf() - Marshall the text in the chatArea
	 * Implemented for montage.Wrapped interface.
	 */
	public void MarshallSelf(DataOutput out) throws IOException {
		String buf = chatArea.getText();
		out.writeInt(buf.length());
		out.writeBytes(buf);
	}

	/**
	 * UnmarshallSelf() - Unmarshall the text for the chatArea
	 * Implemented for montage.Wrapped interface.
	 */
	public void UnmarshallSelf(DataInput in) throws IOException {
		int bufsize = in.readInt();
		byte[] b = new byte[bufsize];
		in.readFully(b,0,bufsize);
		String buf = new String(b,0,0,bufsize);
		chatArea.setText(buf);
	}

	/**
	 * getChatArea() - expose ChatArea to others
	 */
	public TextArea getChatArea() {
		return chatArea;
	}

	/**
	 * getChatNameField() - expose ChatNameField to others
	 */
	public ChatNameField getChatNameField() {
		return name;
	}
}
