/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */

package Chat;
import java.awt.*;
import ncsa.awt.LabeledTextField;
import ncsa.habanero.*;


/**
 * ChatNameField - implements the name area that's used in the conferencing
 * session.  This events from this object are *NOT* shared.
 */
class ChatNameField extends LabeledTextField {
	/**
	 * Create a labeled text field 
	 */
	protected Chat parent;
	private String mouseJustEnteredName;

	public ChatNameField(String label, int size) {
		super(label, size);
	}

	/**
	 * handleEvent - Pressing the return key causes an event to occur.
	 * We don't need to pass the event along to the rest of the 
	 * since this is a local object.  We handle the event here, and
	 * ignore the fact that the key was pressed.  We get retrieve the
	 * name by calling the getName() method.
	 */
	public boolean handleEvent(Event e) {
				if (e.id == Event.MOUSE_ENTER)
				{	String nowName = getName();
					setMouseJustEnteredName(nowName);
					return true;
				}
				else
				if (e.id == Event.MOUSE_EXIT)
				{
					
					if (getName().equals(getMouseJustEnteredName()))
						return true;
					else
					{	e.id = Event.ACTION_EVENT;
						e.arg = "*** "+getMouseJustEnteredName()+" ("+SessionManager.manager().owner().getName() +") has a new alias: "+getName()+" ***";
						setMouseJustEnteredName(getName());
						return false;
					}
				}
				else
				if (e.id == Event.ACTION_EVENT) 
				{	e.arg = "*** "+getMouseJustEnteredName()+" ("+SessionManager.manager().owner().getName() +") has a new alias: "+getName()+" ***";
					setMouseJustEnteredName(getName());
					return false;
				}
                return super.handleEvent(e);
	}

	public void setParent(Chat parent) {
		this.parent = parent;
	}


	/**
	 * getName - Read what the user put in the name field.  If there
	 * is nothing in the field, return the generic name "Unknown".
	 */
	public String getName() {
		String name = getText();

		if (name.length() > 0)
			return name;
		return "Unknown";
	}
	
	public void setName(String name) {
		int index = name.indexOf(' ');
		if (index == -1) {
			setText(name);
			return;
		}
		setText(name.substring(0,index));
		
	}

	public void setMouseJustEnteredName(String name)
	{	mouseJustEnteredName = name;
	}

	public String getMouseJustEnteredName()
	{	return mouseJustEnteredName;
	}

	
}
