/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package VotingTool;

import ncsa.habanero.*;
import java.awt.*;
import java.io.*;
import java.util.*;
//import java.net.*;


/**
 * Vote - A Multiple Answer/Question Voting tool.
 */
public class MultiVoting extends ActiveGridPanel implements ncsa.habanero.Marshallable{
  
  Label AccessLabel, DisplayLabel, TopicLabel;
  Button Sbt,DCbt ;

  MultiBallot InfoB;
  String WhoIAm, SessionOwner;
  Panel PCenter;
  CheckboxGroupIndex cbg;
  Checkbox cbA[];
  CheckboxIndex cbiA[];
   int  Answer[];
  int nbrPossibleAnswers, BottomPosition;
  GridBagLayout gridbag = new GridBagLayout();
  GridPanel ActionPanel;

  public MultiVoting(){}
	 
  public MultiVoting( MultiBallot Info) {
    this.setBackground(Color.white);
    SetUp(Info);
}
  private void SetUp(MultiBallot Info){
    InfoB = Info;
    WhoIAm = SessionManager.manager().owner().unique();
    SessionOwner = Info.SessionOwnerName;
    setLayout(new BorderLayout());

    TextArea QuestionArea = new TextArea(Info.Question(), 4, 30);
    GridBorderPanel GBP = new GridBorderPanel("Vote Information");
    GBP.setLayout(gridbag);
    TopicLabel= new Label("QUESTION:");
    GBP.constrain(GBP, TopicLabel, 0,0,1,1);
    GBP.constrain(GBP, QuestionArea, 1,0,30,4);
    if (Info.Public()) 
      AccessLabel= new Label("Type of Vote : Public");
    else AccessLabel = new Label("Type of Vote : Private");
    GBP.constrain(GBP,AccessLabel, 0,5,2,1);
    if (Info.DisplayResult().equals("EO"))
	    DisplayLabel= new Label("Display Results to: Every One");
    else 
      DisplayLabel= new Label("Display Results to: Owner Only");
    GBP.constrain(GBP,DisplayLabel,0,6,2,1);
    add("North",GBP);

    ActionPanel = new GridPanel();
    ActionPanel.setLayout(gridbag);
    ActionPanel.constrain(ActionPanel, new Label("Your Voter ID: "+WhoIAm),0,8,7,1);  

    
    if (Info.MultipleAnswers()) 
      parseResponseArray(Info.ResponseArray,"ManyChoices");
    else 
      
        parseResponseArray(Info.ResponseArray, "OneChoice");
   add("Center",ActionPanel); 
    
     
  }
/* returns RAns: array of the non null selectionned answers */
  public int [] Parse_Answer(int Answer [] ){
    int leng = Answer.length;
    int tmp = 0;
    int NNS =0;
    int RAns [];

    for ( int i=0; i<leng; i++){
      if (Answer[i]>0) NNS++;
    }
    RAns = new int[NNS];

    for ( int i=0; i<leng; i++){
      if (Answer[i]>0) {
	RAns[tmp] = i;
	tmp++;
      }
		    }
    return RAns;
}

public void parseResponseArray(String Array [], String typeOutput){
nbrPossibleAnswers = Array.length;
  
if (typeOutput.equals("OneChoice")){
  ActionPanel.constrain(ActionPanel, new Label("Your Answer: "),0,9,2,1);
  cbiA = new CheckboxIndex[nbrPossibleAnswers];
    cbg = new CheckboxGroupIndex();
    Answer = new int[1];
    for (int i=0; i<nbrPossibleAnswers; i++){
      cbiA[i]= new CheckboxIndex(Array[i], cbg, false);
      ActionPanel.constrain(ActionPanel,cbiA[i], 2,2*i+9,2,1);
    }
  }else {
    ActionPanel.constrain(ActionPanel, new Label("Your Answers: "),0,9,2,1);
    cbA = new Checkbox[nbrPossibleAnswers];
      Answer = new int[nbrPossibleAnswers];
    for (int i=0; i<nbrPossibleAnswers; i++){
      cbA[i] = new Checkbox(Array[i]);
      ActionPanel.constrain(ActionPanel,cbA[i],2,2*i+9,2,1);
    }
    
}
BottomPosition = 2*nbrPossibleAnswers+10;
Sbt = new Button("Send");
DCbt = new Button("Don't Care");
Sbt.disable(); 
ActionPanel.constrain(ActionPanel, Sbt, 1,BottomPosition,1,1);
ActionPanel.constrain(ActionPanel, DCbt, 5,BottomPosition,1,1);

++BottomPosition;

}
	/**
	 * let events pass through to Habanero subsystem
	 */
	public boolean handleEvent(Event e) {
	  Frame fr; 
	  MultiEnv Vote=null;
	 
	  switch (e.id){
	  case Event.ACTION_EVENT:
	    if (e.target instanceof CheckboxIndex){
	      // One Answer only: Answer[] gets the index of the selected CheckboxIndex
	      Answer[0] = cbg.getSelected().getIndex();
	      //System.out.println("MultiVoting, handleEvent: Answer[0]= "+Answer[0]);
	     
	      Sbt.enable();
	      
	      return true;
	    }
	    if (e.target instanceof Checkbox){
	      // Multiple Answers: Answer[i] gets 1 if checkbox i selected, 0 if not;
	      for (int i=0; i<nbrPossibleAnswers; i++){
		if (cbA[i].getState()) {
		  //System.out.println("MultiVoting: handleEvent: "+cbA[i].getState());
		  Answer[i]=1;
		}
		else Answer[i] =0;
	      }
	      DCbt.enable();
	      Sbt.enable();
	      return true;
	    }
	    if (e.target instanceof Button){
	      String label=(String)e.arg;

	      if (label.equals("Send")) {
		 //System.out.println("MultiVoting: handleEvent: Send MultiEnv: nbrResponses "+InfoB.nbrResponses);
		if (InfoB.MultipleAnswers())
		  Vote = new MultiEnv(Parse_Answer(Answer),InfoB.MultipleAnswers, WhoIAm,this.InfoB.Question, false, InfoB.nbrResponses); 
		else 
		  Vote = new MultiEnv(Answer,InfoB.MultipleAnswers, WhoIAm,this.InfoB.Question, false, InfoB.nbrResponses);
		Vote.toDetail();
		Event ac = new Event(this, Event.ACTION_EVENT,Vote );
		postEvent (ac);
		Sbt.hide();
		DCbt.hide();
		
		return true;
	      }
	      
	      if (label.equals("Don't Care")) {
		Vote = new MultiEnv(WhoIAm,this.InfoB.Question, true, InfoB.nbrResponses );
		Event ac = new Event(this, Event.ACTION_EVENT,Vote );
		Vote.toDetail();
		postEvent (ac);
		Sbt.hide();
			DCbt.hide();
		
		return true;
	      }
	      
	    }
	  
	  return false;
	default:
	  return false;
	    
	  }
	}
	
	

	



public boolean doEvent(Event e, Object arg){
   MultiArbiResult TheResults = (MultiArbiResult) arg;
   MultiResultPanel MRF = new MultiResultPanel(InfoB, TheResults);  
   add("South",MRF);
   this.layout();
   BottomPosition = BottomPosition+ 7;

return true;	
	
      }


	/**
	 * MarshallSelf() - nothing to marshall, so this is empty.
	 * Implemented for montage.Wrapped interface.
	 */
	public void MarshallSelf(DataOutput out) throws IOException {
	 Marshalling.marshallSomething(InfoB, out);

	}

	/**
	 * UnmarshallSelf() - nothing to marshall, so this is empty.
	 * Implemented for montage.Wrapped interface.
	 */
	public void UnmarshallSelf(DataInput in) throws IOException {
 InfoB = (MultiBallot)Marshalling.Unmarshall(in);
	  SetUp(InfoB);
	}


}




