/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */
package VotingTool;

import java.util.*;
import java.awt.*;
import java.applet.Applet;

/**
 *
 * PieChartCanvas Class - Draws a 3D PieChart
 *
 * @version		1.02	4-Dec-1995
 * @author		Ciaran Treanor
 *
 * email		ciaran@broadcom.ie
 * phone		+353-1-604-6000
 * web			http://www.broadcom.ie/~ct
 *
 * Revision History:
 *
 * 1-Dec-1995 : Added buffering to do graphics offscreen

 modified: L. Feuillet: INRIA/NCSA
 */
class PieChartCanvas extends Canvas {
	final double aspectFudge = 2.5;
	int		radius,
			depth,
			called = 1,
			numSlices = 0;
	float	total = 0,
			value[] = new float[10];
	Color	color[] = new Color[10];
	Graphics	offGraphics;
	Image		gfxBuff;

	public PieChartCanvas(int radius, int depth) {
		this.value = value;
		this.color = color;
		this.radius = radius;
		this.depth = depth;
	}

	public void paint(Graphics g) {
		int startAngle;
		float angle;
		Dimension d = size();

		if(gfxBuff == null) {
			gfxBuff = createImage(d.width, d.height);
			offGraphics = gfxBuff.getGraphics();
			offGraphics.setColor(getBackground());
			offGraphics.fillRect(0, 0, d.width, d.height);
		}

// This is less than optimal, but we don't have a floodfill.
// Draw depth-1 ovals in a darker colour
		for(int x = depth; x >= 1; x--) {
			startAngle = -45;
			for(int i = 0; i < numSlices; i++) {
				offGraphics.setColor(color[i].darker());
				angle = Math.round(360 * (value[i] / total));
				offGraphics.fillArc(0, x, radius, (int)(radius / aspectFudge),
					startAngle, (int)angle);
				startAngle += angle;
			}
		}

// Now draw the final (top) oval in the undarkened color		
		startAngle = -45;
		for(int i = 0; i < numSlices; i++) {
			offGraphics.setColor(color[i]);
			angle = Math.round(360 * (value[i] / total));
			offGraphics.fillArc(0, 0, radius, (int)(radius / aspectFudge),
				startAngle, (int)angle);
			startAngle += angle;
		}
		g.drawImage(gfxBuff, 0, 0, null);
	}

	public void addSlice(float value, Color color) {
		this.value[numSlices] = value;
		this.color[numSlices++] = color;
		total += value;
	}

	public Dimension preferredSize() {
		return minimumSize();
	}

	public Dimension minimumSize() {
		return new Dimension(radius, (int)((radius / aspectFudge) + depth));
	}
}
