/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */

package VotingTool;

import java.awt.*;
import java.io.*;
import java.util.*;
import java.net.*;
import ncsa.habanero.*;

/**
 * Vote - A simple Voting tool: Yes or No.
 */
public class SimpleVoting extends ActiveGridPanel implements ncsa.habanero.Marshallable{

	//String pname= "laurent";
	CheckboxGroup cbg;
	Label AccessLabel, DisplayLabel, TopicLabel;
	Button Sbt,DCbt, More ;
	GridBagLayout gridbag;
	SimpleBallot InfoB;
	String WhoIAm, SessionOwner;


	public SimpleVoting(){}

	public SimpleVoting(SimpleBallot Info) {
	  SetUp(Info);
	}
	private void SetUp(  SimpleBallot Info) {
	  TextArea QuestionArea = new TextArea(Info.Question(), 4, 30);
	  GridBorderPanel GBP = new GridBorderPanel("Vote Information");
	  InfoB = Info;
	  SessionOwner = Info.SessionOwnerName;

	  gridbag = new GridBagLayout();
	  setLayout(new BorderLayout());
	  
	  
	  GBP.setLayout(gridbag);
	  TopicLabel= new Label("QUESTION:");
	  GBP.constrain(GBP, TopicLabel, 0,0,1,1);
	  GBP.constrain(GBP, QuestionArea, 1,0,30,4);
	  if (Info.Public()) 
	    AccessLabel= new Label("Type of Vote : Public");
	  else AccessLabel = new Label("Type of Vote : Private");
	  GBP.constrain(GBP,AccessLabel, 0,5,2,1);
	  if (Info.DisplayResult().equals("EO"))
	    DisplayLabel= new Label("Display Results to: Every One");
	  else 
	    DisplayLabel= new Label("Display Results to: Owner Only");
	  GBP.constrain(GBP,DisplayLabel,0,6,2,1);

	  add("North",GBP);

	  GridPanel ActionPanel = new GridPanel();
	  ActionPanel.setLayout(gridbag);
	  WhoIAm = SessionManager.manager().owner().unique();
	   ActionPanel.constrain(ActionPanel, new Label("Your Voter ID: "+WhoIAm),0,8,7,1);
	  ActionPanel.constrain(ActionPanel, new Label("Your answer: "), 0,9,1,1);
	  cbg = new CheckboxGroup();
	  Checkbox cb1= new Checkbox("Yes", cbg, false);	  
	  Checkbox cb2= new Checkbox("No", cbg, false);
	  ActionPanel.constrain(ActionPanel,cb1, 2,10,1,1);
	  ActionPanel.constrain(ActionPanel,cb2, 2,11,1,1);
	  Sbt = new Button("Send");
	  DCbt = new Button("Don't Care");
	  Sbt.disable();
	  ActionPanel.constrain(ActionPanel,Sbt, 1,13,1,1);
	  ActionPanel.constrain(ActionPanel,DCbt,5,13,1,1);
	  WhoIAm = SessionManager.manager().owner().unique();	
	  add("Center",ActionPanel);
	}

	/**
	 * let events pass through to Habanero subsystem
	 */
	public boolean handleEvent(Event e) {
	  Frame fr; 
	  SimpleEnv Vote;
	  switch (e.id){
	  case Event.ACTION_EVENT:
	    if (e.target instanceof Checkbox){
	      DCbt.enable();
	      Sbt.enable();
	      return true;
	    }
	    if (e.target instanceof Button){
	      String label=(String)e.arg;
	   
	      if (label.equals("Send")) {
		if (cbg.getCurrent().getLabel()!= null){
		  if (cbg.getCurrent().getLabel()=="Yes"){
		    
		    Vote = new SimpleEnv(true,WhoIAm,InfoB.Question(), false); 
		    Vote.toDetail();
		  }
		  else {
		    Vote = new SimpleEnv(false,WhoIAm,InfoB.Question(), false);
		    Vote.toDetail();
		  }
		  
		  
		  Event ac = new Event(this, Event.ACTION_EVENT,Vote );
		  postEvent (ac);
		  Sbt.hide();
		  DCbt.hide();
		  
		  return true;
		}
		else {
		  //System.out.println("No Answer given");
		  return true;
		}
	      }
	      if (label.equals("Don't Care")) {
		Vote = new SimpleEnv(WhoIAm,InfoB.Question, true);
		Vote.toDetail();
		Event ac = new Event(this, Event.ACTION_EVENT,Vote );
		postEvent (ac);
		Sbt.hide();
		DCbt.hide();
		
		return true;
	      }
	      
	    return false;
	}
	  default:
	    return false;

	  }
	}

	public boolean doEvent(Event e, Object arg){

	   SimpleArbiResult TheResults = (SimpleArbiResult) arg;

	   SimpleResultPanel SRF = new SimpleResultPanel(InfoB, TheResults);  

	   this.add("South",SRF);

	   this.layout();
	   return true;
	}
	/**
	 * MarshallSelf() - nothing to marshall, so this is empty.
	 * Implemented for montage.Wrapped interface.
	 */
	public void MarshallSelf(DataOutput out) throws IOException {
	  Marshalling.marshallSomething(InfoB, out);
	}

	/**
	 * UnmarshallSelf() - nothing to marshall, so this is empty.
	 * Implemented for montage.Wrapped interface.
	 */
	public void UnmarshallSelf(DataInput in) throws IOException {
	  InfoB = (SimpleBallot)Marshalling.Unmarshall(in);
	  SetUp(InfoB);
	}


}
////////////////////////////////////////////////////////////////////////////////////
