/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 *
 * ----
 * 
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * 
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 * 
 */

package VotingTool;

import ncsa.habanero.*;
import ncsa.habanero.granted.*;
//import ncsa.habanero.tickets.*;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;
import java.util.*;
//import ncsa.habanero.StandardActionRequest;
//import ncsa.habanero.NewCollobjectAction;
import java.awt.Event;
import VotingTool.*;

/**
 * The LocalArbitrator is used when there is no floor control at all.  
 * It immediately grants any ActionRequests it gets.  It's not clear
 * how useful this is, now that we have a centrlized TicketGranter 
 * anyway.
*/
public class VotingArbitrator extends CentralArbitrator {
  
  String BType;
  int NullVotes, NbVotes, NonVoters =0;
  int NbVoters=0;

  String Sresult = "incomplete";
  // Duration of the timer in ms
  long TimerDuration = 300000;
  //long TimerDuration = 30000;
  boolean WFVended = false;
  int YesVote, NoVote;
  Hashtable DatatoTransmit = new Hashtable();
  int [] Mresult;
  boolean MRinitialized = false;
  boolean TimerinAction = false;
  EnvCharact Ballot;
  Vector PropV = new Vector();
  int Presult;
  Timer VotingTimer;


  /**
   * The constructor sets the parent arbitrator
   * @param parent
   */
  public VotingArbitrator(Arbitrator parent, Router r) 
    {
      super(parent, r);
     //System.out.println ("VotingArbitrator::VotingArbitrator");
    }
  
  /**
   * The constructor sets the parent arbitrator
   * @param parent
   */
  public VotingArbitrator()     {
    
    super();
    
  }
  public void init(Arbitrator parent, Router r){
    super.init(parent, r);

    
  } 
 

  /**
   * The arbitrate routine simply grants the request and asks
   * the router to send it to everyone
   * @param request the ActionRequest we want to grant
   */
  public synchronized boolean arbitrate(ActionRequest request) 
    {
      //System.out.println ("VotingArbitrator::arbitrate");
      
      Action actThis = request.action();
      
      
      try {
	Ballot = (EnvCharact) ((AppletAction)actThis).getEvent().arg;
      }
      catch (java.lang.ClassCastException cce) {
	
	return super.arbitrate(request);
      }
      BType = Ballot.getClass().getName();
      if (!WFVended){
	NbVoters =0;
	Enumeration part = (this.session()).participants();
	while (part.hasMoreElements() ){
	  NbVoters++;
	  //System.out.println("Participant Number "+NbVoters+": "+part.nextElement());
	  part.nextElement();
	}
	AnalyseVote(Ballot);
	NbVotes ++;
	//System.out.println("Nb de Votes: "+NbVotes);
	if (!TimerinAction){
	  VotingTimer = new Timer(this, TimerDuration);
	  VotingTimer.start();
	  TimerinAction = true;
	}else {
	  VotingTimer.stop();
	   VotingTimer.start();
	  TimerinAction = true;
	}
	  
	if (VoteCompleted()){
	  System.out.println(" Arbitrator says: VOTE is CLOSED !");
	  printResult();
	  
	  return true;
	}
	}
      
      return false;
    }
  
  private void AnalyseVote(EnvCharact Ballot){
   
    //System.out.println("Arbitrator: Ballot.getClassName(): "+ BType);
    if (BType.equals("VotingTool.SimpleEnv")){
      	SimpleEnv SB = (SimpleEnv)Ballot;
      if (IsDC(SB)) {
	NullVotes ++;
	DatatoTransmit.put(SB.ParticipantName, "DC");
      }
      else{

	if (SB.answer) {
	  YesVote ++;
	  DatatoTransmit.put(SB.ParticipantName,"Yes");
	}
	else {
	  NoVote++;
	  DatatoTransmit.put(SB.ParticipantName,"No");
	}
      }
    }else if (BType.equals("VotingTool.PropEnv")){
       PropEnv PB = (PropEnv) Ballot;
     if (IsDC(PB)) {
       NullVotes ++;
       DatatoTransmit.put(PB.ParticipantName,"DC");
     }
     else 
       {
	DatatoTransmit.put(PB.ParticipantName,new Integer(PB.answer));
	PropV.addElement(new Integer(PB.answer));
       }
    } else if (BType.equals("VotingTool.MultiEnv")){
      MultiEnv MB = (MultiEnv) Ballot;
      if (!MRinitialized){ Mresult = new int[MB.nbPropAnsw];
			  //System.out.println("Arbitrator: AnalyseVote: Mresult.size() "+MB.nbPropAnsw);
			 MRinitialized = true;
			 }
     if (IsDC(MB)) {
       NullVotes ++;
       DatatoTransmit.put(MB.ParticipantName,"DC");
     }
     else {
     DatatoTransmit.put(MB.ParticipantName,MB.answers);
     parseAnswers(MB.answers);
   }
    }

   
}
  private void parseAnswers(int [] RA){
    
    int longueur = RA.length;

    for (int i=0;i<longueur;i++) {
      Mresult[RA[i]]= Mresult[RA[i]] +1;

    }
}

  private boolean IsDC(EnvCharact Ballot){
    return Ballot.DC;
}
  private boolean VoteCompleted(){
  if (NbVotes == NbVoters) {
    if (!WFVended) WFVended=true;
    return true;
  }
  return false;
}
  
  public  void  printResult(){
    Event ac ;

    //System.out.println("Arbitrator: Ballot Type: "+BType);
   if (BType.equals("VotingTool.SimpleEnv")){
    float calc;
    try {
      calc= 2 *((float)YesVote/(YesVote+NoVote));
      //System.out.println("VotingArbitrator: printResult() calc: "+calc);
      if (calc >1){
	Sresult = "Yes";
	//System.out.println("answer is YES: "+ YesVote+" against: "+NoVote);
      }else if  (calc <1){ 
	//System.out.println("answer is NO: "+NoVote+" against: "+YesVote);
	Sresult = "No";
	
	//System.out.println("adios");
      }else {
	//System.out.println("no answer: the pros equal the cons");
	Sresult = "the Pros equals the Cons";
      }
      if (NonVoters<1){
      ac = new Event(this, Event.ACTION_EVENT, new SimpleArbiResult(NbVotes, NbVoters, NullVotes, DatatoTransmit, Sresult,YesVote, NoVote,false ));
    }else{
       ac = new Event(this, Event.ACTION_EVENT, new SimpleArbiResult(NbVotes, NbVoters, NullVotes, DatatoTransmit, Sresult,YesVote, NoVote,true ));
      }
      route_to_everyone(grantNewAction(new AppletAction(ac), owner));
      //System.out.println("Voting Arbitrator: return Vote Information");
    }
    catch (ArithmeticException m){
    System.out.println("Invalid Vote: information missing");
    Sresult = "Missing Information";
    //java.lang.Runtime.exit(0);
  }
 }
   if (BType.equals("VotingTool.PropEnv")){
     Presult = meanPropV();
     //System.out.println("mean Answer is: "+Presult);
      if (NonVoters<1){
      ac = 
        new Event(this, Event.ACTION_EVENT, new PropArbiResult(NbVotes, NbVoters, NullVotes, DatatoTransmit, Presult, false));
    }else{
      ac = 
        new Event(this, Event.ACTION_EVENT, new PropArbiResult(NbVotes, NbVoters, NullVotes, DatatoTransmit, Presult, true));
    }
      route_to_everyone(grantNewAction(new AppletAction(ac), owner));
     //System.out.println("Voting Arbitrator: return Vote Information");
 }
   if (BType.equals("VotingTool.MultiEnv")){
      //Print_Mresult();
      if (NonVoters<1){
	ac = 
	  new Event(this, Event.ACTION_EVENT, new MultiArbiResult(NbVotes, NbVoters, NullVotes, DatatoTransmit, Mresult, false));
      }else{
	ac =  
	  new Event(this, Event.ACTION_EVENT, new MultiArbiResult(NbVotes, NbVoters, NullVotes, DatatoTransmit, Mresult, true));
      }
     
      route_to_everyone(grantNewAction(new AppletAction(ac), owner));

      //System.out.println("Voting Arbitrator: return Vote Information");
   }
    //System.out.println("Voting Arbitrator: NonVoters "+NonVoters);
}

   private void  Print_Mresult(){
     System.out.println("Voting Arb: Mresult");
     for (int i=0;i<Mresult.length;i++) 
       System.out.println("Mresult["+i+"]= "+Mresult[i]);
       
     }

private int meanPropV(){
  int size = PropV.size();
  int PartialSum =0;
// System.out.println("VotingArbitrator : mean Prop: size: "+size);
for (Enumeration e = PropV.elements() ; e.hasMoreElements() ;) {
  int temp = ((Integer)e.nextElement()).intValue();
  //System.out.println("VotingArbitrator : mean Prop: valeur: "+temp);
  PartialSum = temp + PartialSum ;
        }
  if (size == 0){
    //System.out.println("Arbitrator: Mean Calculation: Divide by 0");
    return 0;}
  else 
  return (PartialSum/size);
}
}
class Timer extends Thread implements  Runnable{
  VotingArbitrator VA = null;
  long Duree = 0;
  Thread thread = null;

  public Timer(VotingArbitrator VotArb, long Duration){
    VA= VotArb;
    Duree = Duration;
  }

   public void start () {
     if (thread == null) {
      thread = new Thread(this);
      thread.start();
    } 
}
  public void run() {
try {   
 thread.sleep(Duree);
 //System.out.println(" Arbitrator says: Timer OUT !");
 if (!VA.WFVended ){
   VA.NonVoters = VA.NbVoters - VA.NbVotes;
   VA.printResult();
   VA.WFVended = true;
  }
}
catch (java.lang.InterruptedException e) {
	System.out.println("Thread interrupted in wait state");
	System.out.println("Ignoring exception : we should still wait");
      }
}
}
