/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *
 * info@havefun.ncsa.uiuc.edu
 *
 * Copyright 1995-1996, Board of Trustees of the University of Illinois,
 * All Rights Reserved
 *
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 *
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 *
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 */

/**
 *
 *   @(#)ChooseBrowserDialog.java  1.01 23 May 1996 Jason Marshall
 *
 **/

/**
 * Dialog Box for the BrowserAgent class, used to configure
 * The Browser Agent
 **/

package WWWSession;

import java.awt.*;
import java.util.Properties;

public class ChooseBrowserDialog extends Dialog {

    Properties props;
    WWWSession ws;
    Button OK;
    Button Cancel;
    List browserList;
    Checkbox configure;

    public ChooseBrowserDialog(Frame f, WWWSession ws, Properties props) {
        super(f, "Browser Configuration", false);
        this.props = props;
        this.ws = ws;

        setBackground(Color.lightGray);
	setResizable(false);

        GridBagLayout gridBag = new GridBagLayout();
        GridBagConstraints constraints = new GridBagConstraints();
        setLayout(gridBag);

        constraints.weightx = 1.0;
        constraints.weighty = 1.0;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        constraints.gridwidth = GridBagConstraints.REMAINDER;
        constraints.ipadx = 100;
        constraints.ipady = 5;
        browserList = new List(5, false);
        browserList.setBackground(Color.white);
        gridBag.setConstraints(browserList, constraints);
        browserList.addItem("NCSA Mosaic (TCP/IP CCI)");
        browserList.addItem("None");
        browserList.select(0);
        add(browserList);

        configure = new Checkbox("Configure Browser");
        gridBag.setConstraints(configure, constraints);
        add(configure);

        constraints.gridwidth = GridBagConstraints.RELATIVE;
        constraints.weightx = 1.0;
        constraints.weighty = 0.0;
        constraints.ipadx = 10;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        OK = new Button("OK");
        gridBag.setConstraints(OK, constraints);
        add(OK);

        constraints.gridwidth = GridBagConstraints.REMAINDER;
        Cancel = new Button("Cancel");
        gridBag.setConstraints(Cancel, constraints);
        add(Cancel);
        pack();
        }

        public boolean handleEvent(Event e) {
            if (e.target == Cancel) {
                hide();
                return true;
                }
            if (e.target == OK) {
                hide();
                String selected = browserList.getSelectedItem();
                if (selected.equals("NCSA Mosaic (TCP/IP CCI)")) {
                    ws.setAgent("MosaicBrowserAgent", configure.getState());
                    }
                else
                    ws.setAgent("BrowserAgent", false);
                dispose();
                return true;
                }
            return false;
            }
    }
