/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *
 * info@havefun.ncsa.uiuc.edu
 *
 * Copyright 1995-1996, Board of Trustees of the University of Illinois,
 * All Rights Reserved
 *
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 *
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 *
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 */

/**
 *
 *   @(#)Message.java  1.0 16 April 1996 Jason Marshall
 *
 **/

/**
 *
 * Message Object to be passed between BrowserAgents in a WWWSession
 *
 **/

package WWWSession;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import ncsa.habanero.Marshallable;

public class Message implements Marshallable {

    // Legal values for the command

    public static String GETURL 	=	"GET URL";
    public static String FORWARD 	=	"FORWARD";
    public static String BACK 	    =	"BACK";
    public static String RELOAD 	=	"RELOAD";
    public static String FIRST      =   "FIRST";
    public static String LAST       =   "LAST";

    // The command the Message contains

    private String command;

    // argument to the command (as needed)

    private String argument;

    // A unique ID used to determine who originated the Message

    private String originator;

    public Message(String cmd, String arg, String orig) {
        command = cmd;
        argument = arg;
        originator = orig;
        }

    public Message(String command, String orig) {
        this(command, "", orig);
        }

    public Message() {
        }

    public String command() {
        return command;
        }

    public String argument() {
        return argument;
        }

    public String originator() {
        return originator;
        }

    // Determine if this Message matches another Message

    public boolean equals(Message msg) {
        if (msg.command().equals(command) && msg.argument().equals(argument))
            return true;
        return false;
        }

    public void MarshallSelf(DataOutput out) throws IOException {
        out.writeUTF(command);
        out.writeUTF(originator);
        out.writeUTF(argument);
        }
    public void UnmarshallSelf(DataInput in) throws IOException {
        command = in.readUTF();
        originator = in.readUTF();
        argument = in.readUTF();
        }
}
