/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *
 * info@havefun.ncsa.uiuc.edu
 *
 * Copyright 1995-1996, Board of Trustees of the University of Illinois,
 * All Rights Reserved
 *
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 *
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 *
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 */

/**
 *
 *   @(#)MosaicBrowserAgent.java  1.0 16 April 1996 Jason Marshall
 *
 **/

/**
 *  Browser Agent which talks to NCSA Mosaic, utilizing its TCP/IP CCI interface.
 **/

package WWWSession;

import java.awt.Event;
import java.io.*;
import java.net.Socket;
import java.util.Properties;

public class MosaicBrowserAgent extends BrowserAgent {

    Socket socket;
    DataOutputStream CCIOut;
    DataInputStream CCIIn;

    public void configure() {
        MosaicDialog dialog = new MosaicDialog(parent, props);
        }

    public boolean doEvent(Event e, Object arg) {
        if (e.target instanceof BrowserAgent) {
            Message msg;

            try {
                msg = (Message) arg;
                }
            catch(ClassCastException cce) {
                return false;
                }

            /* All messages besides Get URL messages of our own creation
               are of interest
             */

            if (e.id == Event.ACTION_EVENT) {
                String command = msg.command();
                if ((!command.equals(Message.GETURL)) ||
                    (!msg.originator().equals(myID))) {
                    try {
                        CCIOut.writeBytes("GET URL <" +
                                          parent.currentURL() +
                                          "> OUTPUT CURRENT\r\n");
                        }
                    catch (Exception except){
                        // Never happens
                        }
                    }
                }
            return true;
            }
        return false;
        }

    public void run() {
        String status = null;
        String address = props.getProperty("address", "localhost");
        String portString = props.getProperty("port", "1200");
        int port;

        try {
            port = Integer.parseInt(portString);
            }
        catch (NumberFormatException nfe) {
            port = 1200;
            }
        try {
            socket = new Socket(address, port);
            }
        catch(Exception ee) {
            }

        if (socket != null) {
            try {
                CCIOut = new DataOutputStream(socket.getOutputStream());
                CCIIn = new DataInputStream(socket.getInputStream());
                CCIOut.writeBytes("SEND ANCHOR BEFORE\r\n");
                if (parent.currentURL().length() != 0) {
                    CCIOut.writeBytes("GET URL <" +
                                       parent.currentURL() +
                                       "> OUTPUT CURRENT\r\n");
                    }
                }
            catch (IOException ee) {
                }
            }

        else {
            System.err.println("Unable to connect to Browser");
            socket = null;
            }

        while ((socket != null) && (thread != null)) {
            try {
                if (CCIIn.available() != 0) {
                    status = new String(CCIIn.readLine());
                    }
                else {
                    status = null;
                    }
                }
            catch (Exception e) {
                }

            if (status != null) {
                if (status.startsWith("301 ANCHOR <")) {
                // Handle any information received from the browser
                    String newURL = status.substring(12, status.lastIndexOf(">")).trim();
                    Message msg = null;
                    if (newURL.equals(parent.currentURL())) {
                        msg = new Message(Message.RELOAD, myID);
                        }
                    else {
                        msg = new Message(Message.GETURL, newURL, myID);
                        }
                    Event evt = new Event(parent, Event.ACTION_EVENT, msg);
                    postEvent(evt);
                    }
                }
            else {
                if (thread != null)
                    try {
                        thread.sleep(100);
                        }
                    catch(InterruptedException ie) {
                        }
                }
            }
        if (thread != null) {
            System.err.println("Connection to browser lost");
            }
        }

    public void stop() {
        if (socket != null) {
            try {
                synchronized(socket) {
                    try {
                        if (CCIOut != null) {
                            CCIOut.writeBytes("DISCONNECT \r\n");
                            CCIOut.flush();
                            }
                        socket.close();
                        socket = null;
                        }
                    catch(IOException ee) {
                        }
                    }
                }
            catch (NullPointerException npe) {
                }
            }
        super.stop();
        }
    }
