/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *
 * info@havefun.ncsa.uiuc.edu
 *
 * Copyright 1995-1996, Board of Trustees of the University of Illinois,
 * All Rights Reserved
 *
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 *
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 *
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 */

/**
 *
 *   @(#)WWWSession.java  0.82 11 March 1996 Jason Marshall
 *
 **/

/**
 *
 * A Collaborative tool to allow shared Web browsing sessions
 *
 **/

package WWWSession;


import java.awt.*;
import java.io.DataInput;
import java.io.DataOutput;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.EmptyStackException;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Stack;
import ncsa.habanero.MirrorFrame;
import ncsa.habanero.SessionManager;
import ncsa.habanero.Wrapped;


public class WWWSession extends Panel implements Wrapped {

    static String packageName = "WWWSession.";
    BrowserAgent agent;
    MirrorFrame frame;
    MenuBar menuBar;
    Properties props;
    WSMenuItem browserConfig;
    Stack backSessionHistory;
    Stack forwardSessionHistory;
    String myID = null;
    String currentURL;

    public WWWSession() {
        backSessionHistory = new Stack();
        forwardSessionHistory = new Stack();
        currentURL = "";
        }


    private void createButtons() {
        FlowLayout layout = new FlowLayout(FlowLayout.CENTER, 3, 3);
        setLayout(layout);

        Button button = new Button("First");
        add(button);
        button = new Button("Back");
        add(button);
        button = new Button("Forward");
        add(button);
        button = new Button("Last");
        add(button);
        }

    private void createMenu() {
        menuBar = new MenuBar();
        Menu menu = new Menu("Options");
        menuBar.add(menu);
        browserConfig = new WSMenuItem("Configure Browser", this);

        if (agent == null ) {
            browserConfig.disable();
            }
        else {
            browserConfig.enable();
            }

        menu.add(browserConfig);
        MenuItem item = new WSMenuItem("New Browser", this);
        menu.add(item);
        frame.setMenuBar(menuBar);
        }

    public String currentURL() {
        return currentURL;
        }

    synchronized public boolean doEvent(Event e, Object arg) {
        if (e.target instanceof WWWSession) {
            Message msg;
            try {
                msg = (Message) arg;
                }
            catch(ClassCastException cce) {
                System.err.println("Cast exception");
                return false;
                }

            if (e.id == Event.ACTION_EVENT) {
                String command = msg.command();
                String argument = msg.argument();

                if (command.equals(Message.GETURL)) {
                    // First FF the session to the last entry...
                    try {
                        while (!forwardSessionHistory.empty()) {
                            backSessionHistory.push(currentURL);
                            currentURL = (String) forwardSessionHistory.pop();
                            }
                        }
                    catch (EmptyStackException ee){
                        System.err.println("Empty Stack!!");
                        }
                    // Then push the new element on the stack
                    try {
                        if (currentURL.length() != 0)
                            backSessionHistory.push(currentURL);
                        forwardSessionHistory.removeAllElements();
                        currentURL = argument;
                        }
                    catch (EmptyStackException ee){
                        System.err.println("Empty Stack!!");
                        }
                    }
                else if (command.equals(Message.FORWARD)) {
                    try {
                        if (currentURL.length() != 0)
                            backSessionHistory.push(currentURL);
                        currentURL = (String) forwardSessionHistory.pop();
                        }
                    catch (EmptyStackException ee){
                        System.err.println("Empty Stack!!");
                        }
                    }
                else if (command.equals(Message.BACK)) {
                    try {
                        if (currentURL.length() != 0)
                            forwardSessionHistory.push(currentURL);
                            currentURL = (String) backSessionHistory.pop();
                        }
                    catch (EmptyStackException ee){
                        System.err.println("Empty Stack!!");
                        }
                    }
                else if (command.equals(Message.FIRST)) {
                    try {
                        while (!backSessionHistory.empty()) {
                            forwardSessionHistory.push(currentURL);
                            currentURL = (String) backSessionHistory.pop();
                            }
                        }
                    catch (EmptyStackException ee){
                        System.err.println("Empty Stack!!");
                        }
                    }
                else if (command.equals(Message.LAST)) {
                    try {
                        while (!forwardSessionHistory.empty()) {
                            backSessionHistory.push(currentURL);
                            currentURL = (String) forwardSessionHistory.pop();
                            }
                        }
                    catch (EmptyStackException ee){
                        System.err.println("Empty Stack!!");
                        }
                    }
                if (agent != null) {
                    e.target = agent;
                    agent.doEvent(e, arg);
                    }
                return true;
                }
            }
        return false;
        }

    synchronized public boolean handleEvent(Event e) {

        // Check for Button presses
        if (e.target instanceof Button) {
            Message msg = null;
            String label = ((Button) e.target).getLabel();

            if (label.equals("Back")) {
                if (!backSessionHistory.empty()) {
                    msg = new Message(Message.BACK, myID);
                    }
                }
            else if (label.equals("Forward")) {
                if (!forwardSessionHistory.empty()) {
                    msg = new Message(Message.FORWARD, myID);
                    }
                }
            else if (label.equals("First")) {
                if (currentURL.length() != 0) {
                    msg = new Message(Message.FIRST, myID);
                    }
                }
            else if (label.equals("Last")) {
                if (currentURL.length() != 0) {
                    msg = new Message(Message.LAST, myID);
                    }
                }

            if (msg == null) {
                }
            else {
                Event evt = new Event(this, Event.ACTION_EVENT, msg);
                postEvent(evt);
                }
            return true;
            }
        else if (e.target instanceof MenuItem) {
            MenuItem item = (MenuItem) e.target;
            if (item.isEnabled()) {
                if (item.getLabel().equals("Configure Browser")) {
                    agent.configure();
                    }
                if (item.getLabel().equals("New Browser")) {
                    Dialog dialog = new ChooseBrowserDialog(frame, this, props);
		    dialog.show();
                    }
                }
            return true;
            }
        return false;
        }

    public void MarshallSelf(DataOutput out) throws IOException {
        out.writeBytes(currentURL + "\r");
        backSessionHistory.trimToSize();
        out.writeInt(backSessionHistory.size());
        Enumeration e = backSessionHistory.elements();
        while (e.hasMoreElements()) {
            out.writeBytes(e.nextElement() + "\r");
            }

        forwardSessionHistory.trimToSize();
        out.writeInt(forwardSessionHistory.size());
        e = forwardSessionHistory.elements();
        while (e.hasMoreElements()) {
            out.writeBytes(e.nextElement() + "\r");
            }
        }

    public boolean setAgent(String agentName) {
        return setAgent(agentName, false);
        }

    public boolean setAgent(String agentName, boolean configure) {
        BrowserAgent newAgent;
        String name;
        if (agentName.length() == 0) {
            agent = null;
            return true;
            }
        else
            name = agentName;

        try {
            newAgent = (BrowserAgent) Class.forName(packageName + name).newInstance();
            }
        catch (Exception e) {
            newAgent = null;
            }

        if (newAgent != null) {
            if (!name.equals("")) {
                props.put("agent", name);
                browserConfig.enable();
                }
            else {
                if (browserConfig != null) {
                    browserConfig.disable();
                    }
                }
            agent = newAgent;
            agent.initClass(this, props, myID);

            // The agent has several options at this point.  It can configure
            // itself, or add new Components to this Panel.  If we're
            // configuring, call the configure method.  It will call setAgent
            // again, with useful values.  Otherwise, it starts.  Make sure
            // our startInFrame has already been called...
            if (frame != null) {
                if (configure)
                    agent.configure();
                else {
                    agent.startInFrame(frame);
                    frame.pack();
                    }
                }
            return true;
            }
        return false;
        }

    public void startInFrame(MirrorFrame f) {
        frame = f;
        frame.setTitle("WWW Browser Session");
        f.add("Center", this);
        f.setResizable(false);

        myID = SessionManager.manager().id();

        f.setBackground(Color.lightGray);

        createButtons();
        createMenu();

    	// TODO: Clean up serialization -JDM

        String config = System.getProperty("user.home", ".");
        String sep = System.getProperty("file.separator", "/");

        config += sep + ".habanero" + sep + "WWWSession"
                  + sep + "properties";

        //System.out.println(config);

        props = new Properties();

        FileInputStream in;
        try {
            in = new FileInputStream(config);
            }
        catch(FileNotFoundException fnfe) {
            System.out.println("Unable to locate configuration file.");
            // OK.. we just don't have a pre-existing config file...
            in = null;
            }
        if (in != null) {
            try {
                props.load(in);
                }
             catch (IOException ioe) {
                System.out.println("Error reading configuration file");
                // Bail out, with no properties.  We'll overwrite the file
                // on exit, hopefully correcting the problem.
                props = new Properties();
                }
            }

        setAgent(props.getProperty("agent", ""));

        f.pack();
        f.show();
        f.pack();
        show();
        if (agent != null) {
            agent.show();
            }
        }

    public void UnmarshallSelf(DataInput in) throws IOException {
        currentURL = in.readLine();
        int elements = in.readInt();
        for (int i = 0; i < elements; i++) {
            backSessionHistory.push(in.readLine());
            }
        elements = in.readInt();
        for (int i = 0; i < elements; i++) {
            forwardSessionHistory.push(in.readLine());
            }
        }
    }


