/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *
 * info@habanero.ncsa.uiuc.edu
 *
 * Copyright 1995-1996, Board of Trustees of the University of Illinois,
 * All Rights Reserved
 *
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 *
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 */
package WhiteBoard;
import ncsa.util.*;
import ncsa.habanero.*;
import ncsa.habanero.granted.*;
import java.awt.*;
import java.io.*;
import ncsa.awt.draw.*;
import java.util.*;

public class CollabWhiteBoard extends WhiteBoard implements Wrapped, Shutdownable {

	CollabImage ci = null;
	private Collobject collobject = null;
	private Arbitrator arbitrator = null;

	public CollabWhiteBoard() {
	}

	public void initialize (Frame frame) {
 
		thisframe = frame;
                paletteBox = new PaletteBox();
 
                Panel p1 = new Panel();
                p1.setLayout(new BorderLayout());
                p1.add("West", paletteBox);
                frame.add("North", p1);
 
                toolpanel = new ToolPanel();
 
                Panel p2 = new Panel();
                p2.setLayout(new BorderLayout());
                p2.add("North", toolpanel);
                frame.add("West", p2);
 
                frame.setMenuBar(new CollabWBMenuBar(this));
 
                frame.add("Center", canvas = new CollabWBCanvas(this));
 
	}

	public void startInFrame(MirrorFrame f) {

		f.setTitle("WhiteBoard");
		f.add("Center", this);
		this.initialize((Frame)f);
		f.pack();
		f.resize(f.preferredSize());

		f.addEventCode(Event.ACTION_EVENT);

		f.show();
		
		collobject = f.getCollobject();
		arbitrator = collobject.arbitrator();

		/*
		** Set canvas for all marshalled objects, and the Image,
		** if there is one.
		*/
		for (int i = 0; i < plots.size(); i++) {
			DrawingPrimitive p = (DrawingPrimitive)plots.elementAt(i);
			p.setCanvas(canvas);
		}
		if (ci == null)
			return;
		ImageHandler ih = ci.getImageHandler();
		Image img = ih.createImage(canvas);
		canvas.setImage(img);
	}

	public boolean doEvent(Event event, Object obj) {
		if (obj == null)
			return true;
		if (obj instanceof DrawingPrimitive) {
			DrawingPrimitive primitive = (DrawingPrimitive)obj;


			return doPrimitive(primitive);
		}
		if (obj instanceof DrawingOperation) {
			DrawingOperation dop = (DrawingOperation)obj;
			return doOperation(dop);
		}

		if (obj instanceof CollabImage) {
			CollabImage ci = (CollabImage)obj;
			ImageHandler ih = ci.getImageHandler();
			Image img = ih.createImage(canvas);
			canvas.setImage(img);
			return true;
		}

		if (obj instanceof URLString) {
			URLString url_string = (URLString)obj;
			canvas.loadURLImage(url_string.getString());
			return true;
		}
		if (obj.equals("Clear All")) {
			clearImage();
			clearPlots();
			return true;
		}
		if (obj.equals("Clear Image")) {
			clearImage();
			return true;
		}
		if (obj.equals("Clear Plots")) {
			clearPlots();
			return true;
		}

		if (event.target instanceof CollabDialogTextField) {
			canvas.loadURLImage((String)obj);
			return true;
		}
			
		return false;
	}


	public boolean doPrimitive(DrawingPrimitive primitive) {
		
		primitive.setCanvas(canvas);
		plots.addElement(primitive);
		canvas.repaint();

		return true;
	}

	public boolean doOperation(DrawingOperation dop) {
		DrawingPrimitive primitive = null;
		dop.init(canvas, plots);
		if (dop instanceof GrabOperation) {
			GrabOperation gop = (GrabOperation)dop;
			primitive = gop.getPrimitive();
			primitive.setCanvas(canvas);
			plots.setElementAt(primitive, gop.getIndex());
			canvas.repaint();
			return true;
		}
		primitive = dop.operation();
		return true;
	}

	public void MarshallSelf(DataOutput out) throws IOException {
		int total = plots.size();
		out.writeInt(total);
		for (int i = 0; i < total; i++) {
			DrawingPrimitive p = (DrawingPrimitive)plots.elementAt(i);
			out.writeUTF(p.getClass().getName());
			p.MarshallSelf(out);
		}
		WBImageInfo info = canvas.getImageInfo();
		if (info.getImage() == null) {
			out.writeInt(0);
			return;
		}
		out.writeInt(1);
		Dimension dim = info.getDimension();
		ImageHandler ih = new ImageHandler(info.getImage(),
			dim.width, dim.height);
		ih.readPixels();
		CollabImage ci = new CollabImage(ih);
		out.writeUTF(ci.getClass().getName());
		ci.MarshallSelf(out);
	}

	public void UnmarshallSelf(DataInput in) throws IOException {
		DrawingPrimitive p = null;
		String  name = null;

		int total = in.readInt();
		for (int i = 0; i < total; i++) {
			name = in.readUTF();
			try {
				p = (DrawingPrimitive)Class.forName(name).newInstance();
			} catch (Exception e) {
				System.out.println(e);
			}
			p.UnmarshallSelf(in);
			plots.addElement(p);
		}
		int flag = in.readInt();
		if (flag == 0)
			return;
		
		name = in.readUTF();
		try {
			ci = (CollabImage)Class.forName(name).newInstance();
			ci.UnmarshallSelf(in);
		} catch (Exception e) {
			System.out.println(e);
		}
	}

        public boolean handleEvent(Event e) {
		return true;
        }

	public void shutDown() {
		thisframe.setMenuBar(null);
	}

	public final Collobject getCollobject() {
		return collobject;
	}

	public final Arbitrator getArbitrator() {
		return arbitrator;
	}

}
