/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *
 * info@habanero.ncsa.uiuc.edu
 *
 * Copyright 1995-1996, Board of Trustees of the University of Illinois,
 * All Rights Reserved
 *
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 *
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 */
package WhiteBoard;

import ncsa.env.abstraction.Const;
import ncsa.awt.draw.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.net.*;

public class ToolPanel extends ImageButtonPanel {
	public int rows = 6;
	public int columns = 2;

	public ToolPanel() {
		super(6, 2);	 /* fix this */

		setBackground(Color.white);

		String homedir = Const.getProperty("habanero.home");
		String sep = System.getProperties().getProperty("file.separator");
		
		String dir = homedir+sep+"rsc"+sep+"WhiteBoard"+sep+"images"+sep;
//		String dir = homedir+ResourceLocator.imagePath("demos","wb/images");
		
System.out.println("dir = "+dir);
		/*
		 * Freehand drawing
		 */

		ToolButton drawButton = 
			new ToolButton(this, dir+"draw.gif", dir+"draw2.gif");
		addButton(drawButton);
		drawButton.setPrimitive(new Freehand());

		/*
		 * Line drawing
		 */

		ToolButton lineButton = 
			new ToolButton(this, dir+"line.gif", dir+"line2.gif");
		addButton(lineButton);
		lineButton.setPrimitive(new Line());

		/*
		 * Pointer drawing 
		 */

		ToolButton pointerButton = 
			new ToolButton(this,dir+"pointer.gif", dir+"pointer2.gif");
		addButton(pointerButton);
		pointerButton.setPrimitive(new Pointer());

		/*
		 * Text Input
		 */

		ToolButton textInputButton = 
			new ToolButton(this, dir+"text.gif", dir+"text2.gif");
		addButton(textInputButton);
		textInputButton.setPrimitive(new TextInput());

		/*
		 * Rectangle drawing
		 */

		ToolButton rectButton = 
			new ToolButton(this, dir+"box.gif", dir+"box2.gif");
		addButton(rectButton);
		rectButton.setPrimitive(new Rect());

		/*
		 * Filled Rectangle drawing
		 */

		ToolButton filledRectButton = 
			new ToolButton(this, dir+"filledbox.gif", dir+"filledbox2.gif");
		addButton(filledRectButton);
		filledRectButton.setPrimitive(new FilledRect());

		/*
		 * Oval drawing
		 */

		ToolButton ovalButton = 
			new ToolButton(this, dir+"oval.gif", dir+"oval2.gif");
		addButton(ovalButton);
		ovalButton.setPrimitive(new Oval());

		/*
		 * Filled Oval drawing
		 */

		ToolButton filledOvalButton = 
			new ToolButton(this, dir+"filledoval.gif", dir+"filledoval2.gif");
		addButton(filledOvalButton);
		filledOvalButton.setPrimitive(new FilledOval());

		/*
		 * Bring to Top
		 */

		ToolButton topButton = 
			new ToolButton(this,dir+"top.gif", dir+"top2.gif");
		topButton.setOperation(new ToTopOperation());
		addButton(topButton);

		/*
		 * Bring to bottom
		 */

		ToolButton bottomButton = 
			new ToolButton(this,dir+"bottom.gif", dir+"bottom2.gif");
		bottomButton.setOperation(new ToBottomOperation());
		addButton(bottomButton);

		/*
		 * Delete
		 */

		ToolButton deleteButton = 
			new ToolButton(this, dir+"del.gif", dir+"del2.gif");
		deleteButton.setOperation(new DeleteOperation());

		addButton(deleteButton);

		/*
		 * Grab 
		 */

		ToolButton grabButton = 
			new ToolButton(this,dir+"pick.gif", dir+"pick2.gif");
		grabButton.setOperation(new GrabOperation());
		addButton(grabButton);

		resize(preferredSize());

		setSelected(drawButton);
		
		show();
	}

	public ToolButton getSelectedButton() {
		ToolButton button = (ToolButton)getSelected();
		return button;
	}
}
