/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *
 * info@habanero.ncsa.uiuc.edu
 *
 * Copyright 1995-1996, Board of Trustees of the University of Illinois,
 * All Rights Reserved
 *
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 *
 * U.S. GOVERNMENT RESTRICTED RIGHTS
 * The Software and/or user documentation are provided with RESTRICTED AND
 * LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
 * government shall be subject to restrictions as set forth in subparagraph
 * (c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
 * Section 252.227-7013 for Department of Defense contracts and as set forth
 * in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
 * agency contracts or any successor regulations.  Contractor/Manufacturer is
 * The Board of Trustees of the University of Illinois, Urbana, Illinois
 * 61801.
 */
package WhiteBoard;

import ncsa.awt.draw.*;
import java.awt.*;
import java.net.*;
import java.util.*;
import java.io.File;

class WBCanvas extends Canvas {
	Image img = null;
	int img_height = 200;
	int img_width = 200;
	WhiteBoard parent;
	Color color;
	DrawingPrimitive primitive = null;
	DrawingPrimitive building = null;
	Image		offScreenImage = null;
	Graphics	offScreenGraphics = null;
	Dimension	imageSize = null;

	public WBCanvas() {
	}

	public WBCanvas(WhiteBoard parent) {
		super();

		this.parent = parent;
		setBackground(Color.white);
		resize(img_width, img_height);
	}

	public void loadFileImage(String image_name) {
		MediaTracker tracker = initImageLoad();

		img = Toolkit.getDefaultToolkit().getImage(image_name);

		finishImageLoad(tracker);
	}

	public void loadURLImage(String image_name) {
		MediaTracker tracker = initImageLoad();
		
		URL url = null;
		try {
			url = new URL(image_name);
		} catch (MalformedURLException e) {
			/* pop a dialog here */
			return; 
		}

		img = Toolkit.getDefaultToolkit().getImage(url);
		finishImageLoad(tracker);
	}

	MediaTracker initImageLoad() {
		MediaTracker tracker = new MediaTracker(this);
		if (img != null) {
			img.flush();
			img = null;
		}
		return tracker;
	}


	void finishImageLoad(MediaTracker tracker) {
		tracker.addImage(img, 0);

		try {
			tracker.waitForID(0);
		} catch (Exception e) {
			System.out.println(e);
			return;
		}

		img_width = img.getWidth(this);
		img_height = img.getHeight(this);

		if ((img_width == -1) || (img_height == -1)) {
			img = null;
			return;
		}

		if (offScreenImage == null)
			initDoubleBuffer();
		repaint();
	}

	public WBImageInfo getImageInfo() {
		Dimension size = new Dimension(img_width, img_height);
		WBImageInfo info = new WBImageInfo(img, size);
		return info;
	}

	public WBImageInfo getOffScreenImageInfo() {
		WBImageInfo info = new WBImageInfo(offScreenImage, imageSize);
		return info;
	}

	public void setImage(Image img) {
		this.img = img;
		repaint();
	}
	
	public final Image getImage() {
		return img;
	}

	public void clearImage() {
		img = null;
	}

	public void paint(Graphics g) {
		Dimension size = null;

		size = preferredSize();

		if (offScreenImage == null) {
			initDoubleBuffer();
		}

		/*
		** Check to make sure we haven't been resized
		*/
		if ((imageSize.width != size.width) || (imageSize.height != size.height)) {
			initDoubleBuffer();
		}
		offScreenGraphics.setColor(Color.white);
		offScreenGraphics.fillRect(0, 0, size.width, size.height);
		if (img != null) {
			offScreenGraphics.drawImage(img, 0, 0, this);
		}
			

		Vector plots = parent.getPlots();
		if (plots != null) {
			for (int i = 0; i < plots.size(); i++) {
				DrawingPrimitive p = (DrawingPrimitive)plots.elementAt(i);
//				offScreenGraphics.setColor(p.getColor());
				p.drawPlot(offScreenGraphics);
			}
		}
		if (building != null) 
			building.drawPlot(offScreenGraphics);

//		offScreenGraphics.setColor(Color.black);
//		offScreenGraphics.drawRect(0, 0, size.width-1, size.height-1);
		if (offScreenImage != null)
			g.drawImage(offScreenImage, 0, 0, this);
	}

	public void initDoubleBuffer() {
		imageSize = preferredSize();
		offScreenImage = createImage(imageSize.width, imageSize.height);
		offScreenGraphics = offScreenImage.getGraphics();
		Graphics g = this.getGraphics();
		offScreenGraphics.setFont(g.getFont());
	}

	public void update(Graphics g) {
		paint(g);
	}

	public boolean mouseDown(Event e, int x, int y) {
		requestFocus(); /* be sure the key events are sent here */
		if (offScreenImage == null) {
			initDoubleBuffer();
		}

		primitive = null;

		ToolButton button = parent.getToolPanel().getSelectedButton();

		DrawingOperation dop = button.getOperation();
		if (dop != null) {
			dop = dop.newInstance();
			dop.init(this, parent.getPlots());
			int index = dop.locatePrimitive(x,y);
			dop.setIndex(index);
			dop.setPoint(x,y);
			primitive = dop.operation();
			if (dop.getReaction() != Event.MOUSE_UP)
				primitive = null;
			return true;
		} else {
			building = button.getPrimitive().newInstance();

			building.setCanvas(this);
			color = parent.getPaletteBox().getSelectedColor();
			building.setColor(color);
//			parent.getPlots().addElement(primitive);
			primitive = building;
		}

		primitive.mouseDown(x, y);

		return true;
	}

	public boolean mouseDrag(Event e, int x, int y) {
		if (primitive != null) {
			primitive.mouseDrag(x, y);
		}
		return true;
	}

	public boolean mouseUp(Event e, int x, int y) {
		if (primitive != null) {
			if (primitive.grabbed() == true) {
				primitive.release();
			} else {
				primitive.mouseUp(x, y);
				parent.getPlots().addElement(primitive);
				building = null;
			}
		}
		return true;
	}

	public boolean keyDown(Event e, int key) {

		if (primitive == null)
			return true;

		if ((primitive instanceof TextInput) == false)
			return true;

		primitive.keyDownAction(key);
		return true;

	}

}
