/*
 * NCSA Habanero Project
 * Software Development Group
 * National Center for Supercomputing Applications
 * University of Illinois at Urbana-Champaign
 * 605 E. Springfield, Champaign, IL 61820
 *  
 * info@havefun.ncsa.uiuc.edu
 *  
 * Copyright 1995-1996, Board of Trustees of the University of Illinois, 
 * All Rights Reserved
 *  
 * NCSA Habanero software, binary, Java-bytecode, and source (hereafter,
 * Software) is copyrighted by The Board of Trustees of the University of
 * Illinois (UI), and ownership remains with the UI.
 *  
 * The UI grants you (hereafter, Licensee) a license to use the Software for
 * academic, research and internal business purposes only, without a fee.
 * Licensee may distribute the binary and source code (if released) to third
 * parties provided that the copyright notice and this statement appears on
 * all copies and that no charge is associated with such copies.
 *  
 * Licensee may make derivative works.  However, if Licensee distributes any
 * derivative work based on or derived from the Software, then Licensee will
 * (1) notify NCSA regarding its distributing of the derivative work, and (2)
 * clearly notify users that such derivative work is a modified version and
 * not the original NCSA Habanero distributed by the UI.
 *  
 * Any Licensee wishing to make commercial use of the Software should contact
 * the UI, c/o NCSA, to negotiate an appropriate license for such commercial
 * use.  Commercial use included (1) integration of all or part of the
 * Software into a product for sale or license by or on behalf of Licensee to
 * third parties, or (2) distribution of the Software to third parties that
 * need it to utilize a commercial product sold or licensed by or on behalf of
 * Licensee.
 *  
 * UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
 * PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.  THE
 * UI SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS
 * SOFTWARE.
 *  
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the U.S. including, but not
 * limited to, export control laws, and the terms of this license.  UI shall
 * have the right to terminate this license immediately by written notice upon
 * Licensee's breach of, or non-compliance with, any of its terms.  Licensee
 * may be held legally responsible for any copyright infringement that is
 * caused or encouraged by Licensee's failure to abide by the terms of this
 * license.
 * 
 */



package ncsa.habanero;
import java.awt.*;
import ncsa.habanero.MirrorFrame;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;

/***
 ***  KeyGen - generates a sequence of keys
 ***/

public class KeyGen {
    //  Local Variables
    private int nSeed;    //  A code number set on creation

    /**
     *
     *@param   nInit - the initial code
     *@return  Creates a key
     *
     */

    public KeyGen ()
        {
        nSeed = (int) (300007.0 * Math.random());

        //  This is the one stable point of the iterative equation
        if (nSeed == 18749)
            nSeed++;

        //  This is the point with a period of 2
        if ((nSeed == 17707) || (nSeed == 1040))
            nSeed++;
        }

    /**
     *
     *@return  generates a new key from the original value, estimated
     *             number of iterations before a repeated value is 150002
     *
     */

    public Key nextKey ()
        {
        nSeed = ((23 + (nSeed * 17)) % 300007);
        return new Key (nSeed);
        }

//  END of KeyGen
}
